<?php

/**
 * i-doit
 *
 * DAO: global category for logbook entries
 *
 * @package     i-doit
 * @subpackage  CMDB_Categories
 * @author      Niclas Potthast <npotthast@i-doit.org>
 * @copyright   synetics GmbH
 * @license     http://www.gnu.org/licenses/agpl-3.0.html GNU AGPLv3
 */
class isys_cmdb_dao_category_g_logb extends isys_cmdb_dao_category_global
{
    /**
     * Category's name. Will be used for the identifier, constant, main table, and many more.
     * @var  string
     */
    protected $m_category = 'logb';

	/**
	 * Category's identifier.
	 * @var    integer
     * @fixme  No standard behavior!
	 */
	protected $m_category_id = C__CATG__LOGBOOK;

    /**
     * Category's constant.
     * @var    string
     * @fixme  No standard behavior!
     */
    protected $m_category_const = 'C__CATG__LOGBOOK';

	/**
	 * Is category multi-valued or single-valued?
	 *
	 * @var  boolean
	 */
	protected $m_multivalued = true;

	/**
	 * Method for returning the properties.
	 *
	 * @return  array
	 */
	protected function properties()
	{
		return array(
			'date' => array_replace_recursive(
				isys_cmdb_dao_category_pattern::datetime(),
				array(
					C__PROPERTY__INFO => array(
						C__PROPERTY__INFO__TITLE => 'LC__CMDB__LOGBOOK__DATE',
						C__PROPERTY__INFO__DESCRIPTION => 'Date'
					),
					C__PROPERTY__DATA => array(
						C__PROPERTY__DATA__FIELD => 'isys_logbook__date'
					),
					C__PROPERTY__UI => array(
						C__PROPERTY__UI__ID => 'C__CMDB__CATG__LOGB__DATE'
					),
					C__PROPERTY__PROVIDES => array(
						C__PROPERTY__PROVIDES__LIST => false
					)
				)
			),
			'object' => array_replace_recursive(
				isys_cmdb_dao_category_pattern::int(),
				array(
					C__PROPERTY__INFO => array(
						C__PROPERTY__INFO__TITLE => 'LC__CMDB__LOGBOOK__SOURCE__OBJECT',
						C__PROPERTY__INFO__DESCRIPTION => 'Object'
					),
					C__PROPERTY__DATA => array(
						C__PROPERTY__DATA__FIELD => 'isys_catg_logb_list__isys_obj__id'
					),
					C__PROPERTY__PROVIDES => array(
						C__PROPERTY__PROVIDES__SEARCH => false,
						C__PROPERTY__PROVIDES__REPORT => false,
						C__PROPERTY__PROVIDES__LIST => false
					),
					C__PROPERTY__FORMAT => array(
						C__PROPERTY__FORMAT__CALLBACK => array(
							'isys_export_helper',
							'object'
						)
					)
				)
			),
			'event' => array_replace_recursive(
				isys_cmdb_dao_category_pattern::dialog(),
				array(
					C__PROPERTY__INFO => array(
						C__PROPERTY__INFO__TITLE => 'LC__UNIVERSAL__EVENT',
						C__PROPERTY__INFO__DESCRIPTION => 'Event'
					),
					C__PROPERTY__DATA => array(
						C__PROPERTY__DATA__FIELD => 'isys_logbook__isys_logbook_event__id',
						C__PROPERTY__DATA__REFERENCES => array(
							'isys_logbook_event',
							'isys_logbook_event__id'
						)
					),
					C__PROPERTY__PROVIDES => array(
						C__PROPERTY__PROVIDES__REPORT => false,
						C__PROPERTY__PROVIDES__SEARCH => false,
						C__PROPERTY__PROVIDES__LIST => false
					),
					C__PROPERTY__FORMAT => array(
						C__PROPERTY__FORMAT__CALLBACK => array(
							'isys_export_helper',
							'dialog'
						)
					)
				)
			),
			'source' => array_replace_recursive(
				isys_cmdb_dao_category_pattern::int(),
				array(
					C__PROPERTY__INFO => array(
						C__PROPERTY__INFO__TITLE => 'LC__UNIVERSAL__SOURCE',
						C__PROPERTY__INFO__DESCRIPTION => 'Source'
					),
					C__PROPERTY__DATA => array(
						C__PROPERTY__DATA__FIELD => 'isys_logbook__isys_logbook_source__id',
						C__PROPERTY__DATA__REFERENCES => array(
							'isys_logbook_source',
							'isys_logbook_source__id'
						)
					),
					C__PROPERTY__PROVIDES => array(
						C__PROPERTY__PROVIDES__SEARCH => false,
						C__PROPERTY__PROVIDES__LIST => false
					)
				)
			),
			'user' => array_replace_recursive(
				isys_cmdb_dao_category_pattern::int(),
				array(
					C__PROPERTY__INFO => array(
						C__PROPERTY__INFO__TITLE => 'LC__CMDB__LOGBOOK__SOURCE__USER',
						C__PROPERTY__INFO__DESCRIPTION => 'User'
					),
					C__PROPERTY__DATA => array(
						C__PROPERTY__DATA__FIELD => 'isys_logbook__isys_obj__id'
					),
					C__PROPERTY__PROVIDES => array(
						C__PROPERTY__PROVIDES__SEARCH => false,
						C__PROPERTY__PROVIDES__REPORT => false,
						C__PROPERTY__PROVIDES__LIST => false
					),
					C__PROPERTY__FORMAT => array(
						C__PROPERTY__FORMAT__CALLBACK => array(
							'isys_export_helper',
							'object'
						)
					)
				)
			),
			'object_type' => array_replace_recursive(
				isys_cmdb_dao_category_pattern::text(),
				array(
					C__PROPERTY__INFO => array(
						C__PROPERTY__INFO__TITLE => 'LC__CMDB__OBJTYPE',
						C__PROPERTY__INFO__DESCRIPTION => 'Object type'
					),
					C__PROPERTY__DATA => array(
						C__PROPERTY__DATA__FIELD => 'isys_logbook__obj_type_static'
					),
					C__PROPERTY__PROVIDES => array(
						C__PROPERTY__PROVIDES__SEARCH => false,
						C__PROPERTY__PROVIDES__LIST => false
					)
				)
			),
			'category' => array_replace_recursive(
				isys_cmdb_dao_category_pattern::text(),
				array(
					C__PROPERTY__INFO => array(
						C__PROPERTY__INFO__TITLE => 'LC__CMDB__CATG__GLOBAL_CATEGORY',
						C__PROPERTY__INFO__DESCRIPTION => 'Category'
					),
					C__PROPERTY__DATA => array(
						C__PROPERTY__DATA__FIELD => 'isys_logbook__category_static'
					),
					C__PROPERTY__PROVIDES => array(
						C__PROPERTY__PROVIDES__SEARCH => false,
						C__PROPERTY__PROVIDES__LIST => false
					)
				)
			),
			'alert_level' => array_replace_recursive(
				isys_cmdb_dao_category_pattern::text(),
				array(
					C__PROPERTY__INFO => array(
						C__PROPERTY__INFO__TITLE => 'LC__CMDB__LOGBOOK__LEVEL',
						C__PROPERTY__INFO__DESCRIPTION => 'Alarmlevel'
					),
					C__PROPERTY__DATA => array(
						C__PROPERTY__DATA__FIELD => 'isys_logbook__isys_logbook_level__id',
						C__PROPERTY__DATA__REFERENCES => array(
							'isys_logbook_level',
							'isys_logbook_level__id'
						)
					),
					C__PROPERTY__PROVIDES => array(
						C__PROPERTY__PROVIDES__SEARCH => false,
						C__PROPERTY__PROVIDES__LIST => false
					),
					C__PROPERTY__FORMAT => array(
						C__PROPERTY__FORMAT__CALLBACK => array(
							'isys_export_helper',
							'dialog'
						)
					)
				)
			),
			'user_name_static' => array_replace_recursive(
				isys_cmdb_dao_category_pattern::text(),
				array(
					C__PROPERTY__INFO => array(
						C__PROPERTY__INFO__TITLE => 'LC__CMDB__LOGBOOK__USER',
						C__PROPERTY__INFO__DESCRIPTION => 'Username'
					),
					C__PROPERTY__DATA => array(
						C__PROPERTY__DATA__FIELD => 'isys_logbook__user_name_static'
					),
					C__PROPERTY__PROVIDES => array(
						C__PROPERTY__PROVIDES__IMPORT => false,
						C__PROPERTY__PROVIDES__EXPORT => false,
						C__PROPERTY__PROVIDES__LIST => false
					)
				)
			),
			'event_static' => array_replace_recursive(
				isys_cmdb_dao_category_pattern::text(),
				array(
					C__PROPERTY__INFO => array(
						C__PROPERTY__INFO__TITLE => 'LC__CMDB__CATG__LOGB__EVENT_STATIC',
						C__PROPERTY__INFO__DESCRIPTION => 'Event'
					),
					C__PROPERTY__DATA => array(
						C__PROPERTY__DATA__FIELD => 'isys_logbook__event_static'
					),
					C__PROPERTY__PROVIDES => array(
						C__PROPERTY__PROVIDES__IMPORT => false,
						C__PROPERTY__PROVIDES__EXPORT => false,
						C__PROPERTY__PROVIDES__SEARCH => false,
						C__PROPERTY__PROVIDES__LIST => false,
						C__PROPERTY__PROVIDES__REPORT => false
					)
				)
			),
			'comment' => array_replace_recursive(
				isys_cmdb_dao_category_pattern::text(),
				array(
					C__PROPERTY__INFO => array(
						C__PROPERTY__INFO__TITLE => 'LC__CATG__CONTACT_COMMENT',
						C__PROPERTY__INFO__DESCRIPTION => 'Comment'
					),
					C__PROPERTY__DATA => array(
						C__PROPERTY__DATA__FIELD => 'isys_logbook__comment'
					),
					C__PROPERTY__PROVIDES => array(
						C__PROPERTY__PROVIDES__LIST => false
					)
				)
			),
			'changes' => array_replace_recursive(
				isys_cmdb_dao_category_pattern::text(),
				array(
					C__PROPERTY__INFO => array(
						C__PROPERTY__INFO__TITLE => 'LC__UNIVERSAL__CHANGES',
						C__PROPERTY__INFO__DESCRIPTION => 'Changes'
					),
					C__PROPERTY__DATA => array(
						C__PROPERTY__DATA__FIELD => 'isys_logbook__changes'
					),
					C__PROPERTY__UI => array(
						C__PROPERTY__UI__ID => 'C__CMDB__CATG__LOGB__CHANGES'
					),
					C__PROPERTY__PROVIDES => array(
						C__PROPERTY__PROVIDES__IMPORT => false,
						C__PROPERTY__PROVIDES__LIST => false,
						C__PROPERTY__PROVIDES__REPORT => false
					),
					C__PROPERTY__FORMAT => array(
						C__PROPERTY__FORMAT__CALLBACK => array(
							'isys_export_helper',
							'logbook_changes'
						)
					)
				)
			),
			'description' => array_replace_recursive(
				isys_cmdb_dao_category_pattern::commentary(),
				array(
					C__PROPERTY__INFO => array(
						C__PROPERTY__INFO__TITLE => 'LC__CMDB__LOGBOOK__DESCRIPTION',
						C__PROPERTY__INFO__DESCRIPTION => 'Description'
					),
					C__PROPERTY__DATA => array(
						C__PROPERTY__DATA__FIELD => 'isys_catg_logb_list__description'
					),
					C__PROPERTY__UI => array(
						C__PROPERTY__UI__ID => 'C__CMDB__CAT__COMMENTARY_' . C__CMDB__CATEGORY__TYPE_GLOBAL . C__CATG__LOGBOOK
					),
					C__PROPERTY__PROVIDES => array(
						C__PROPERTY__PROVIDES__LIST => false
					)
				)
			),
			'reason' => array_replace_recursive(
				isys_cmdb_dao_category_pattern::dialog_plus(),
				array(
					C__PROPERTY__INFO => array(
							C__PROPERTY__INFO__TITLE => 'LC__CMDB__CATG__ACCESS_TYPE',
							C__PROPERTY__INFO__DESCRIPTION => 'Access type'
					),
					C__PROPERTY__DATA => array(
							C__PROPERTY__DATA__FIELD => 'isys_logbook__isys_logbook_reason__id',
							C__PROPERTY__DATA__REFERENCES => array(
									'isys_logbook_reason',
									'isys_logbook_reason__id'
							)
					),
					C__PROPERTY__UI => array(
							C__PROPERTY__UI__ID => 'C__CMDB__LOGBOOK__REASON',
							C__PROPERTY__UI__PARAMS => array(
									'p_strTable' => 'isys_logbook_reason'
							)
					)
				)
			)
		);
	} // function


	public function sync($p_category_data, $p_object_id, $p_status = 1 /* isys_import_handler_cmdb::C__CREATE */) {
		if(is_array($p_category_data) && isset($p_category_data['properties']))
		{
			$l_dao = new isys_component_dao_logbook($this->m_db);
			switch ($p_status)
			{
				case isys_import_handler_cmdb::C__CREATE:
				case isys_import_handler_cmdb::C__UPDATE:
					$l_dao->set_entry($p_category_data['properties']['event'][C__DATA__VALUE],
									  $p_category_data['properties']['description'][C__DATA__VALUE],
									  $p_category_data['properties']['date'][C__DATA__VALUE],
									  2,
									  $p_category_data['properties']['object'][C__DATA__VALUE],
									  'Test',
									  $p_category_data['properties']['object_type'][C__DATA__VALUE],
									  $p_category_data['properties']['category'][C__DATA__VALUE],
									  null,
									  null,
									  null);
					return $p_category_data['data_id'];
					break;
			}
		}
		return false;
	}

	/**
	 * Return Category Data
	 *
	 * @param [int $p_id]
	 * @param [int $p_obj_id]
	 * @param [string $p_condition]
	 *
	 * @return isys_component_dao_result
	 */
	public function get_data($p_catg_list_id=NULL, $p_obj_id=NULL, $p_condition="", $p_filter=NULL, $p_status=NULL, $p_limit = NULL) {
		$p_condition .= $this->prepare_filter($p_filter);

		$l_sql = "SELECT * FROM isys_catg_logb_list ".
					"INNER JOIN isys_obj ".
						"ON ".
						"isys_obj__id = ".
						"isys_catg_logb_list__isys_obj__id ".
					"INNER JOIN isys_logbook ".
						"ON ".
						"isys_logbook__id = ".
						"isys_catg_logb_list__isys_logbook__id ".
					"LEFT JOIN isys_logbook_source ".
						"ON ".
						"isys_logbook__isys_logbook_source__id = ".
						"isys_logbook_source__id ".

					"WHERE TRUE ";

		$l_sql .= $p_condition;

		if (!empty($p_obj_id)) {
			$l_sql .= $this->get_object_condition($p_obj_id);
		}

		if (!empty($p_catg_list_id)) {
			$l_sql .= " AND (isys_catg_logb_list__id = ".$this->convert_sql_id($p_catg_list_id).")";
		}

		if (!empty($p_status)) {
			$l_sql .= " AND (isys_catg_logb_list__status = '{$p_status}')";
		}

		if ($p_limit)
		{
			$l_sql .= ' LIMIT ' . $this->m_db->escape_string($p_limit);
		}

		return $this->retrieve($l_sql . ";");
	}

    /**
     * @return int|mixed
     */
	public function save_element() {

		$l_dao = new isys_component_dao_logbook($this->m_db);

		$l_message = $_POST["C__CATG__LOGBOOK__MESSAGE"];
		$l_description = $_POST["C__CATG__LOGBOOK__DESCRIPTION"];

		if ($_POST["C__CATG__LOGBOOK__ALERTLEVEL"] > 0)
			$l_alert_level = $_POST["C__CATG__LOGBOOK__ALERTLEVEL"];
		else
			$l_alert_level = C__LOGBOOK__ALERT_LEVEL__0;

		if ($_GET[C__CMDB__GET__OBJECT]) {
			$l_object_id = $_GET[C__CMDB__GET__OBJECT];
		} else {
			$l_object_id = NULL;
		}

		$l_dao->set_entry(
			$l_message,
			$l_description,
			null,
			$l_alert_level,
			$l_object_id,
			$this->get_obj_name_by_id_as_string($_GET[C__CMDB__GET__OBJECT]),
			$this->get_objtype_name_by_id_as_string($_GET[C__CMDB__GET__OBJECTTYPE]),
			null,
			C__LOGBOOK_SOURCE__USER,
			"",
			$_POST['LogbookCommentary'],
			$_POST['LogbookReason']
		);

		return 2;
	}

    /**
     * @param isys_component_dao_result $p_daores
     *
     * @return bool
     */
	public function init(isys_component_dao_result &$p_daores) {
		$this->m_daores = $p_daores;
		return true;
	}

} // class