<?php

/**
 * CMDB Category view
 *
 * @package     i-doit
 * @subpackage  CMDB_Views
 * @author      Andre Wösten <awoesten@i-doit.de>
 * @version     0.9
 * @copyright   synetics GmbH
 * @license     http://www.gnu.org/licenses/agpl-3.0.html GNU AGPLv3
 */
class isys_cmdb_view_category
extends isys_cmdb_view
{
	/**
	 * Category DAO class.
	 *
	 * @var  isys_cmdb_dao_category_global
	 */
	protected $m_cat_dao;

	/**
	 * Category UI class.
	 *
	 * @var  isys_cmdb_ui_category_global
	 */
	protected $m_cat_ui;

	/**
	 * Category constant
	 *
	 * @var string
	 */
	protected $m_categoryConst;

	/**
	 * Category Data
	 *
	 * @var array
	 */
	protected $m_categoryData = array();

	/**
	 * Returns the ID for the category view.
	 *
	 * @return  integer
	 */
	public function get_id()
	{
		return C__CMDB__VIEW__CATEGORY;
	} // function

	/**
	 * Returns the name of the view.
	 *
	 * @return  string
	 */
	public function get_name()
	{
		return "Kategorieansicht";
	} // function

	/**
	 * Method which is responsible for initializing the category-specific classes $this->m_cat_dao and $this->m_cat_ui.
	 *
	 * @author  Niclas Potthast <npotthast@i-doit.org>
	 * @throws  isys_exception_cmdb
	 * @throws  Exception|isys_exception_cmdb
	 * @return  boolean
	 */
	public function category_init()
	{
		try
		{
			// Obtain necessary globals.
			$l_gets     = $this->get_module_request()->get_gets();
			$l_database = $this->get_module_request()->get_database();

			// Deactivate all navbar buttons - These will be activated in "handle_navmode()"
			$this->get_module_request()->get_navbar()
				->set_active(false, C__NAVBAR_BUTTON__ARCHIVE)
				->set_active(false, C__NAVBAR_BUTTON__DELETE)
				->set_active(false, C__NAVBAR_BUTTON__RECYCLE)
				->set_active(false, C__NAVBAR_BUTTON__DUPLICATE)
				->set_active(false, C__NAVBAR_BUTTON__PURGE)
				->set_active(false, C__NAVBAR_BUTTON__PRINT);

			/*
			 * Warning!
			 *
			 * Only cateID is processed here by the distributor. There are situations where you can't use cateID,
			 * because it's automatically initiating the list-fetching procedure in the category-factory.
			 *
			 * To skip this (thus you have to collect the data on your own)
			 * use cat1ID -> cat6ID.
			 */

			// Which category entry is chosen?
			$l_catentry  = $_GET[C__CMDB__GET__CATLEVEL];
			$l_catmeta   = $this->m_dao_cmdb->nav_get_current_category_data($l_gets);
			$l_cat_const = $l_gets[$l_catmeta["get"]];

			/* Rights and category stuff */
			$this->m_categoryData = $this->m_dao_cmdb->gui_get_info_by_category($l_catmeta["type"], $l_cat_const);
			if ($this->m_categoryData->count())
				$this->m_categoryData = $this->m_categoryData->get_row();
			else $this->m_categoryData = array();

			$this->m_categoryConst = $this->m_categoryData["isysgui_" . $l_catmeta["string"] . "__const"];

			$this->m_rights = array(
				isys_auth::VIEW => isys_auth_cmdb::instance()->has_rights_in_obj_and_category(isys_auth::VIEW, $l_gets[C__CMDB__GET__OBJECT], $this->m_categoryConst),
				isys_auth::ARCHIVE => isys_auth_cmdb::instance()->has_rights_in_obj_and_category(isys_auth::ARCHIVE, $l_gets[C__CMDB__GET__OBJECT], $this->m_categoryConst),
				isys_auth::DELETE => isys_auth_cmdb::instance()->has_rights_in_obj_and_category(isys_auth::DELETE, $l_gets[C__CMDB__GET__OBJECT], $this->m_categoryConst),
				isys_auth::EDIT => isys_auth_cmdb::instance()->has_rights_in_obj_and_category(isys_auth::EDIT, $l_gets[C__CMDB__GET__OBJECT], $this->m_categoryConst),
				isys_auth::SUPERVISOR => isys_auth_cmdb::instance()->has_rights_in_obj_and_category(isys_auth::SUPERVISOR, $l_gets[C__CMDB__GET__OBJECT], $this->m_categoryConst),
			);

			// Get all necessary data for the current category.
			$l_catdata = $this->m_categoryData;

			// Determine if it is multivalued.
			$l_is_multivalued = !empty($l_catdata['isysgui_' . $l_catmeta["string"] . '__list_multi_value']);

			// Initialize distributor.
			$l_object_id = $l_gets[C__CMDB__GET__OBJECT];
			$l_strError  = '';

			// Emit signal.
			isys_component_signalcollection::get_instance()->emit("mod.cmdb.beforeCategoryInit", $l_object_id, $l_catentry, $l_catmeta);

			if (!$l_object_id || $l_object_id == "")
			{
				throw new isys_exception_cmdb("Distributor couldn't determine object ID (id = '$l_object_id')", C__CMDB__ERROR__CATEGORY_PROCESSOR);
			} // if

			// Foreign key id.
			$l_nFKID = NULL;

			if ($l_is_multivalued)
			{
				$l_nFKID = $l_catentry;
			} // if

			// Check for object existence.
			if ($this->m_dao_cmdb->obj_exists($l_object_id))
			{
				$l_dist = new isys_cmdb_dao_distributor(
					$l_database,
					$l_object_id,
					$l_catmeta["type"],
					$l_nFKID,
					isys_cmdb_dao_distributor::make_category_filter($l_cat_const)
				);

				$l_nDCount = $l_dist->count();

				// Check distributor and count of fetched category access objects.
				if ($l_dist && $l_nDCount)
				{

					$l_cat_dao = $l_dist->get_category($l_cat_const);

					if (!$l_cat_dao)
					{
						throw new isys_exception_cmdb("Could not get category DAO for selected category (in " . $l_strError . ")", C__CMDB__ERROR__CATEGORY_PROCESSOR);
					} // if

					// Get UI class.
					$l_cat_ui = $l_cat_dao->get_ui();

					// Build cross-reference between DAO and UI (so that both reference each other in order to work together).
					$l_cat_ui->init($l_cat_dao);

					$this->m_cat_ui  = $l_cat_ui;
					$this->m_cat_dao = $l_cat_dao
						->set_object_id($l_object_id)
						->set_object_type_id($l_cat_dao->get_type_by_object_id($l_object_id)->get_row_value('isys_obj_type__id'));
				}
				else
				{
					if (class_exists($l_catdata["isysgui_{$l_catmeta["string"]}__class_name"]))
					{
						throw new isys_exception_cmdb("Your selected category '" . _L($l_catdata["isysgui_{$l_catmeta["string"]}__title"]) . "' with ID '{$l_cat_const}' does not work properly. The class exists, but your database table '{$l_catdata["isysgui_{$l_catmeta["string"]}__source_table"]}' may not.", C__CMDB__ERROR__CATEGORY_PROCESSOR);
					}
					else
					{
						throw new isys_exception_cmdb("The class {$l_catdata["isysgui_{$l_catmeta["string"]}__class_name"]} for your selected category '" . _L($l_catdata["isysgui_{$l_catmeta["string"]}__title"]) . "' with ID '{$l_cat_const}' does not exist.", C__CMDB__ERROR__CATEGORY_PROCESSOR);
					} // if
				} // if
			}
			else
			{
				global $g_error;
				$g_error = "Object with ID: " . $l_object_id . " does not exist.\n" .
				"If you clicked on a link to reach this page (e.g. a bookmarked page), it might be broken and should be deleted.";

				throw new isys_exception_cmdb($g_error);
			} // if

			isys_component_signalcollection::get_instance()->emit("mod.cmdb.afterCategoryInit", $l_object_id, $l_cat_dao, $l_dist);
		}
		catch (isys_exception_cmdb $e)
		{
			throw $e;
		}
		catch (Exception $e)
		{
			throw new isys_exception_cmdb($e->getMessage());
		} // try

		return true;
	} // function

	/**
	 * Dynamically add data identifiers to smarty plugins
	 */
	protected function assign_data_identifiers ()
	{
		if (is_object($this->m_cat_dao) && $this->m_cat_dao->get_category_id() != C__CATG__OVERVIEW)
		{
			$l_str_dao = get_class($this->m_cat_dao);
			$l_properties = $this->m_cat_dao->get_properties();
			$l_rules = array();

			if (is_array($l_properties))
			{
				foreach ($l_properties as $l_key => $l_prop)
				{
					if (isset($l_prop[C__PROPERTY__UI][C__PROPERTY__UI__ID]))
					{
						$l_rules[$l_prop[C__PROPERTY__UI][C__PROPERTY__UI__ID]]['p_dataIdentifier'] = $l_str_dao . '::' . $l_key;
					} // if
				} // foreach

				if (count($l_rules) > 0)
				{
					$this->m_cat_ui->get_template_component()->smarty_tom_add_rules("tom.content.bottom.content", $l_rules);
				} // if
			} // if
		} // if
	} // function


	/**
	 * @param null $p_result
	 *
	 * @throws isys_exception_cmdb
	 */
	public function category_process($p_result = NULL)
	{
        global $index_includes, $g_comp_registry;

        $l_gets   = $this->get_module_request()->get_gets();
        $l_navbar = $this->get_module_request()->get_navbar();

        // Check for specific edit and delete rights.
		$l_edit_right = $this->m_rights[isys_auth::EDIT];
		$l_archive_right = $this->m_rights[isys_auth::ARCHIVE];
		$l_delete_right = $this->m_rights[isys_auth::DELETE];
		$l_supervisor_right = $this->m_rights[isys_auth::SUPERVISOR];

        // Rights-Check: Are we allowd to view this category.
        isys_auth_cmdb::instance()
                 ->check_rights_obj_and_category(isys_auth::VIEW, $l_gets[C__CMDB__GET__OBJECT], $this->m_categoryConst);

        if ($this->m_cat_dao && $this->m_cat_ui)
        {
            $l_catmeta = $this->m_dao_cmdb->nav_get_current_category_data($l_gets);
            $l_catdata = $this->m_categoryData;

	        $this->m_cat_ui->get_template_component()
		        ->assign('category_entry_id', $this->m_cat_dao->get_list_id())
		        ->assign(C__GET__ID, $this->m_cat_dao->get_list_id())
		        ->assign('object_id', $this->m_cat_dao->get_object_id())
		        ->assign('object_type_id', $this->m_cat_dao->get_object_type_id());

            $l_is_multi   = !empty($l_catdata["isysgui_" . $l_catmeta["string"] . "__list_multi_value"]);
            $l_quickpurge = (isys_settings::get('cmdb.quickpurge') == '1') ? true : false;

            if (!$l_is_multi)
            {
                // Show print view button
                $l_navbar
                    ->set_active(true, C__NAVBAR_BUTTON__PRINT)
                    ->set_visible(true, C__NAVBAR_BUTTON__PRINT);
            }

            $l_catlevel = 0;
            $l_catentry = $this->get_next_category_level($l_catlevel);

            // If navmode = 'save' then we want to see a single object-processor, no list.
            if ($_POST[C__GET__NAVMODE] == C__NAVMODE__SAVE)
            {
                $l_is_multi = false;
            } // if

            if (!$l_is_multi || ($l_is_multi && ($l_catentry !== NULL) && ($l_catlevel == 0)))
            {
                // UI Category Process.
                $this->m_cat_ui->activate_commentary($this->m_cat_dao);
                $index_includes["contentbottomcontent"] = $this->m_cat_ui->get_template();

                // Enables the ajax saving.
                if ($g_comp_registry->{'[Root]/Idoit/General/Quicksave/b_showQuickSaveButton'})
                {
                    $this->m_cat_ui->enable_ajax_save();
                } // if

                // Emit signal.
                isys_component_signalcollection::get_instance()->emit("mod.cmdb.beforeProcess", $this->m_cat_dao, $index_includes["contentbottomcontent"]);

                // Set property's ui parameters
                $this->m_cat_ui->process_ui_rules($this->m_cat_dao->get_properties(C__PROPERTY__WITH__VALIDATION));

                // Process category.
                $this->m_cat_ui->process($this->m_cat_dao);

                if ($this->m_cat_dao->category_entries_purgable() && $_POST[C__GET__NAVMODE] != C__NAVMODE__EDIT)
                {
                    $l_navbar
                        ->set_active($l_supervisor_right, C__NAVBAR_BUTTON__PURGE)
                        ->set_visible(true, C__NAVBAR_BUTTON__PURGE);
                } // if

                // Assign data-identifier attributes to smarty plugins.
                $this->assign_data_identifiers();

                $l_navbar
                    ->set_visible(false, C__NAVBAR_BUTTON__NEW)
                    ->set_visible(false, C__NAVBAR_BUTTON__ARCHIVE)
                    ->set_visible(false, C__NAVBAR_BUTTON__QUICK_PURGE);
            }
            else
            {
                /*
                 * JUMP "isys_cmdb_view_list_category"
                 * This category is multivalued, show its category list.
                 */
                $l_gets[C__CMDB__GET__VIEWMODE] = C__CMDB__VIEW__LIST_CATEGORY;

                $this->get_module_request()->_internal_set_private("m_get", $l_gets);
                $this->readapt_form_action();

                $l_navbar
                    ->set_active($l_edit_right, C__NAVBAR_BUTTON__NEW)
                    ->set_active($l_archive_right || $l_delete_right || $l_supervisor_right, C__NAVBAR_BUTTON__ARCHIVE)
                    ->set_active(($l_quickpurge && $l_supervisor_right), C__NAVBAR_BUTTON__QUICK_PURGE)
                    ->set_visible(true, C__NAVBAR_BUTTON__NEW)
                    ->set_visible(true, C__NAVBAR_BUTTON__ARCHIVE)
                    ->set_visible(($l_quickpurge && $l_supervisor_right), C__NAVBAR_BUTTON__QUICK_PURGE);

                // Emit signal.
                isys_component_signalcollection::get_instance()->emit("mod.cmdb.beforeProcessList", $this->m_cat_dao, $index_includes["contentbottomcontent"]);

                // Process list.
                $this->m_cat_ui->process_list($this->m_cat_dao);
            } // if

            // We need the DAO-class name for the Javascript validation.
            $this->m_cat_ui->get_template_component()
                           ->assign('cmdb_category_view_context', true)
                           ->assign('dao_class_name', get_class($this->m_cat_dao));
        }
        else
        {
            throw new isys_exception_cmdb("Could not process category (DAO or UI not set)", C__CMDB__ERROR__CATEGORY_PROCESSOR);
        } // if
	} // function


	/**
	 * Method for handling the given nav-mode.
	 *
	 * @param   integer $p_navmode
	 *
	 * @throws  isys_exception_auth
	 * @return  mixed
	 */
	public function handle_navmode($p_navmode)
	{
		$l_auth_instance = isys_auth_cmdb::instance();
		$l_gets          = $this->get_module_request()->get_gets();
		$l_posts         = $this->get_module_request()->get_posts();
		$l_navbar        = $this->get_module_request()->get_navbar();
		$l_tpl           = $this->get_module_request()->get_template();
		$l_actionproc    = $this->get_action_processor();

		$l_navbar::set_locked(false);

		$l_result    = NULL;
		$l_object_id = $l_gets[C__CMDB__GET__OBJECT];

		// Get rights.
		$l_edit_right = $this->m_rights[isys_auth::EDIT];

		// Get responsible category data.
		$l_catmeta = $this->m_dao_cmdb->nav_get_current_category_data();

		// Get lock stuff and check if object id is logged.
		$l_dao_lock = new isys_component_dao_lock($this->get_module_request()->get_database());

		if (($l_locked = $l_dao_lock->is_locked($l_object_id)))
		{
			$l_res = $l_dao_lock->get_lock_information($l_object_id);
			$l_row = $l_res->get_row();

			$l_tpl
			->assign("g_locked", "1")
			->assign("lock_user", $this->m_dao_cmdb->get_obj_name_by_id_as_string($l_row["isys_user_session__isys_obj__id"]));
		} // if

		// Get primary multivalued information from database.
		$l_multi_gui = $this->m_dao_cmdb->gui_is_multivalued_by_category(
            $l_catmeta["type"],
            ($this->m_cat_dao->get_category_id() != C__CATG__CUSTOM_FIELDS) ? $this->m_cat_dao->get_category_id() : $l_catmeta["id"]
		);

		switch ($p_navmode)
		{
			case C__NAVMODE__JS_ACTION:
			case C__NAVMODE__NEW:
				// Check, if the user is allowed to create a new category entry.
				if (!$this->m_rights[isys_auth::EDIT])
				{
					throw new isys_exception_auth(_L(
					                              'LC__AUTH__CMDB_EXCEPTION__MISSING_RIGHT_FOR_CATEGORY',
					                              array(isys_auth::get_right_name(isys_auth::EDIT), _L($this->m_cat_dao->get_catg_name_by_id_as_string($this->m_cat_dao->get_category_id())))
					                              ));
				} // if

				$l_actionproc->insert(
				C__CMDB__ACTION__CATEGORY_CREATE,
				array(
				&$this->m_cat_dao,
				&$this->m_cat_ui
				)
				);

				$l_actionproc->process();

				$l_saveret = $l_actionproc->result_pop();

				if (is_null($l_saveret))
				{
					// Do nothing and break up.
					break;
				}
				elseif (is_numeric($l_saveret))
				{
					$l_result                       = $l_saveret;
					$l_gets[C__CMDB__GET__CATLEVEL] = $l_result;
				}
				elseif (is_array($l_saveret))
				{
					/* New level->category ID assignment */
					list($l_newlevel, $l_newid) = $l_saveret;
					$l_gets[constant("C__CMDB__GET__CATLEVEL_" . $l_newlevel)] = $l_newid;
				}

				$l_gets[C__CMDB__GET__EDITMODE] = C__EDITMODE__ON;

				if ($p_navmode == C__NAVMODE__JS_ACTION)
				{
					$l_navbar->set_active(true, C__NAVBAR_BUTTON__NEW);
				}

				$l_navbar
				->set_active(true, C__NAVBAR_BUTTON__SAVE)
				->set_active(true, C__NAVBAR_BUTTON__CANCEL);

				unset($l_gets[C__CMDB__GET__CAT_LIST_VIEW]);
				break;

			case C__NAVMODE__SAVE:
				// Check, if the user is allowed to edit the category entry.

				if (!$this->m_rights[isys_auth::EDIT])
				{
					throw new isys_exception_auth(_L(
					                              'LC__AUTH__CMDB_EXCEPTION__MISSING_RIGHT_FOR_CATEGORY',
					                              array(isys_auth::get_right_name(isys_auth::EDIT), _L($this->m_cat_dao->get_catg_name_by_id_as_string($this->m_cat_dao->get_category_id())))
					                              ));
				} // if

				// Get last, most important multivalued information from UI category object.
				$l_multi_uic = $this->m_cat_dao->is_multivalued();
				if ($l_multi_uic !== NULL)
				{
					// The user interface class of the category can decide, whether a category is handled multivalued or not.
					$l_multi_gui = $l_multi_uic;
				} // if

				// In case we're single valued.
				if (!$l_multi_gui)
				{
					// Ask CMDB for category entry count.
					$l_catcount = $this->m_cat_dao->get_data(NULL, $_GET[C__CMDB__GET__OBJECT])->num_rows();

					if ($l_catcount === 0)
					{
						// If there is no list entry for this single-valued category, create it.
						$l_actionproc->insert(C__CMDB__ACTION__CATEGORY_CREATE, array(&$this->m_cat_dao, &$this->m_cat_ui));

						/*
						 * Process action queue and directly clear action stack - we just
						 * want to create this entry. get_general_data of isys_cmdb_dao_category
						 * will read it automagically :-)
						 */
						$l_actionproc->process();
						$l_actionproc->result_clear();
					} // if
				} // if

				// Okay, everything done - now we're going to update the category entry.
				$l_actionproc->insert(C__CMDB__ACTION__CATEGORY_UPDATE, array(&$this->m_cat_dao, &$this->m_cat_ui));

				// Process category update.
				$l_result = $l_actionproc->process();

				// Check result of category update.
				$l_saveret = $l_actionproc->result_pop();

				if ($l_saveret == NULL)
				{
					// Save was successful.
					$l_gets[C__CMDB__GET__EDITMODE] = C__EDITMODE__OFF;

					if ($l_edit_right && !$l_locked)
					{
						$l_navbar->set_active(true, C__NAVBAR_BUTTON__EDIT);
					} // if
				}
				else if (is_array($l_saveret))
				{
					list($l_newlevel, $l_newid) = $l_saveret;

					if ($l_newlevel == 1)
					{
						$l_gets[C__CMDB__GET__CATLEVEL] = $l_newid;
					}
					else
					{
						if (defined("C__CMDB__GET__CATLEVEL_" . $l_newlevel))
						{
							$l_gets[constant("C__CMDB__GET__CATLEVEL_" . $l_newlevel)] = $l_newid;
						} // if
					} // if

					// Cancel navmodes.
					$l_gets[C__CMDB__GET__EDITMODE] = C__EDITMODE__OFF;
					if ($l_edit_right && !$l_locked)
					{
						$l_navbar->set_active(true, C__NAVBAR_BUTTON__EDIT);
					} // if

					unset($l_posts[C__GET__NAVMODE]);
				}
				else
				{
					// Save was bad, attach validation rules to TOM.
					$l_tpl->smarty_tom_add_rules("tom.content.bottom.content", $this->m_cat_dao->get_additional_rules());

					if ($l_edit_right && !$l_locked)
					{
						$l_gets[C__CMDB__GET__EDITMODE] = C__EDITMODE__ON;
						$l_navbar
						->set_active(true, C__NAVBAR_BUTTON__SAVE)
						->set_active(true, C__NAVBAR_BUTTON__CANCEL)
						->set_active(true, C__NAVBAR_BUTTON__EDIT);

						$l_dao_lock->add_lock($l_object_id);
					}
				}
				break;

			case C__NAVMODE__CANCEL:
				$l_gets[C__CMDB__GET__EDITMODE] = C__EDITMODE__OFF;
				$l_navbar
				->set_active($l_edit_right, C__NAVBAR_BUTTON__EDIT)
				->set_visible(true, C__NAVBAR_BUTTON__EDIT);
				break;

			case C__NAVMODE__EDIT:
				// Check, if the user is allowed to edit the category entry.
				if (!$this->m_rights[isys_auth::EDIT])
				{
					throw new isys_exception_auth(_L(
					                              'LC__AUTH__CMDB_EXCEPTION__MISSING_RIGHT_FOR_CATEGORY',
					                              array(isys_auth::get_right_name(isys_auth::EDIT), _L($this->m_cat_dao->get_catg_name_by_id_as_string($this->m_cat_dao->get_category_id())))
					                              ));
				} // if

				if ($l_edit_right && !$l_locked)
				{
					$l_gets[C__CMDB__GET__EDITMODE] = C__EDITMODE__ON;
					$l_navbar
					->set_active(true, C__NAVBAR_BUTTON__SAVE)
					->set_active(true, C__NAVBAR_BUTTON__CANCEL);

					$l_dao_lock->add_lock($l_object_id);
				} // if
				break;

			case C__NAVMODE__UP:
				// Set new GET parameters for category view and reload module.
				if (isset($l_gets[C__CMDB__GET__CATLEVEL]))
				{
					$l_gets[C__CMDB__GET__VIEWMODE] = C__CMDB__VIEW__LIST_CATEGORY;
					$l_gets[C__CMDB__GET__EDITMODE] = C__EDITMODE__OFF;
					$l_gets[C__CMDB__GET__CATLEVEL] = NULL;
					unset($l_gets[C__CMDB__GET__CATLEVEL]);
				}
				else
				{
					$l_gets[C__CMDB__GET__VIEWMODE] = C__CMDB__VIEW__LIST_OBJECT;
					$l_gets[C__CMDB__GET__TREEMODE] = C__CMDB__VIEW__TREE_OBJECTTYPE;
				} // if

				unset($l_posts[C__GET__NAVMODE]);
				break;
			case C__NAVMODE__PURGE:

				// Check, if the user is allowed to create a new category entry.
				if (!$this->m_rights[isys_auth::DELETE])
				{
					throw new isys_exception_auth(_L(
					                              'LC__AUTH__CMDB_EXCEPTION__MISSING_RIGHT_FOR_CATEGORY',
					                              array(isys_auth::get_right_name(isys_auth::DELETE), _L($this->m_cat_dao->get_catg_name_by_id_as_string($this->m_cat_dao->get_category_id())))
					                              ));
				} // if

				// Ask CMDB for category entry count
				$l_res = $this->m_cat_dao->get_data(NULL, $_GET[C__CMDB__GET__OBJECT]);

				if ($l_res->num_rows() === 1)
				{
					$l_catdata = $this->m_dao_cmdb->nav_get_current_category_data($l_gets);
					$l_data    = $l_res->get_row();

					if (strpos($l_catdata["table_list"], "isys_cats") === 0)
					{
						if (strripos($l_catdata["table_list"], "_list"))
						{
							$l_catdata["table_list"] = substr($l_catdata["table_list"], 0, strripos($l_catdata["table_list"], "_list"));
						} // if

						$l_catdata["table_list"] = trim($l_catdata["table_list"]) . '_list';
					} // if

					$l_actionproc->insert(C__CMDB__ACTION__CATEGORY_RANK, array(
							C__CMDB__RANK__DIRECTION_DELETE,
							$this->m_cat_dao,
							$l_catdata["table_list"], // source list
							array($l_data[$l_catdata["table_list"] . '__id']),
							C__NAVMODE__PURGE
						));

					$l_result = $l_actionproc->process();
					$l_actionproc->result_clear();

					if (method_exists($this->m_cat_dao, "create_element"))
					{
						$l_tmp = 0;
						$this->m_cat_dao->create_element($l_tmp, $l_list_id);
						$l_gets[C__CMDB__GET__CATLEVEL] = $l_list_id;

						$l_navbar
						->set_active($l_edit_right, C__NAVBAR_BUTTON__EDIT)
						->set_visible(true, C__NAVBAR_BUTTON__EDIT);

						if ($l_gets[C__CMDB__GET__EDITMODE] == C__EDITMODE__ON)
						{
							$l_navbar
							->set_active($l_edit_right, C__NAVBAR_BUTTON__SAVE)
							->set_active($l_edit_right, C__NAVBAR_BUTTON__CANCEL)
							->set_visible(true, C__NAVBAR_BUTTON__SAVE)
							->set_visible(true, C__NAVBAR_BUTTON__CANCEL);
						} // if
					} // if
				} // if
				break;

			default:
				// Check, if the user is allowed to view this category.
				$l_auth_instance->check(isys_auth::VIEW, 'OBJ_ID/' . $l_gets[C__CMDB__GET__OBJECT]);

				// Block detecting if the edit mode should be used. This is necessary for newborn entries.
				if ($l_catmeta["type"] != C__CMDB__CATEGORY__TYPE_SPECIFIC && $l_catmeta["type"] != C__CMDB__CATEGORY__TYPE_CUSTOM)
				{
					// Get entry ID.
					$l_catdata = $this->m_cat_dao->get_general_data();

					// Get source table for selected category.
					$l_srctable = $this->m_dao_cmdb->gui_get_source_table_by_category(
					$l_catmeta["type"],
					$this->m_cat_dao->get_category_id()
					);

					// Set entrystatus to normal in overview category, because there will never be data inside isys_catg_overview_list.
					if ($l_gets[C__CMDB__GET__CATG] == C__CATG__OVERVIEW || $l_catmeta["table"] == "isys_catg_virtual")
					{
						$l_entrystatus = C__RECORD_STATUS__NORMAL;
					}
					else
					{
						// Otherwize determine status of entry.
						$l_entrystatus = $l_catdata[$l_srctable . "_list__status"];
					} // if

					// If there is no entry status (i.e. no entry) or the entry is a newborn, switch directly to editmode.
					if (($l_entrystatus == NULL || $l_entrystatus == C__RECORD_STATUS__BIRTH) && !$l_multi_gui && $l_edit_right)
					{
						$l_gets[C__CMDB__GET__EDITMODE] = C__EDITMODE__ON;
						$l_navbar
							->set_active(false, C__NAVBAR_BUTTON__EDIT)
							->set_active(true, C__NAVBAR_BUTTON__SAVE)
							->set_active(true, C__NAVBAR_BUTTON__CANCEL)
							->set_visible(true, C__NAVBAR_BUTTON__SAVE)
							->set_visible(true, C__NAVBAR_BUTTON__CANCEL);

						// Create element if no entry exists.
						if (($l_catdata == NULL || count($l_catdata) == 0) && method_exists($this->m_cat_dao, "create_element"))
						{
							$l_tmp = 0;
							$this->m_cat_dao->create_element($l_tmp, $l_list_id);
							$l_gets[C__CMDB__GET__CATLEVEL] = $l_list_id;
						} // if
					}
					else
					{
						$l_gets[C__CMDB__GET__EDITMODE] = C__EDITMODE__OFF;
						if (!$l_locked)
						{
							$l_navbar
								->set_active($l_edit_right, C__NAVBAR_BUTTON__EDIT)
								->set_visible(true, C__NAVBAR_BUTTON__EDIT);
						}
					} // if
				}
				else
				{
					/*
					 * On specific categories, we cannot handle this situation at the moment,
					 * since we're binding them directly to the distributor and sub-categories
					 * are handled manually.
					 */
					$l_gets[C__CMDB__GET__EDITMODE] = C__EDITMODE__OFF;
					if (!$l_locked)
					{
						$l_navbar
							->set_active($l_edit_right, C__NAVBAR_BUTTON__EDIT)
							->set_visible(true, C__NAVBAR_BUTTON__EDIT);
					}
				} // if

				// Inner-category list view active?
				if (!empty($l_gets[C__CMDB__GET__CAT_LIST_VIEW]))
				{
					$l_navbar
						->set_active(false, C__NAVBAR_BUTTON__EDIT)
						->set_active($l_edit_right, C__NAVBAR_BUTTON__NEW)
						->set_visible(true, C__NAVBAR_BUTTON__NEW);
				} // if
		} // switch

		if ($l_locked)
		{
			$l_navbar::set_locked(true);
			$l_navbar
				->hide_all_buttons()
				->deactivate_all_buttons();
			$l_gets[C__CMDB__GET__EDITMODE] = C__EDITMODE__OFF;
		} // if

		$l_navbar->set_active(true, C__NAVBAR_BUTTON__UP);

		$this->get_module_request()->_internal_set_private("m_get", $l_gets);
		$this->get_module_request()->_internal_set_private("m_post", $l_posts);
		$this->get_module_request()->_internal_set_private("m_navbar", $l_navbar);
		$this->readapt_form_action();

		return $l_result;
	} // function

	/**
	 * Save process.
	 *
	 * @return  null
	 */
	public function process_save ()
	{
		// Prepare operation data.
		$l_posts = $this->get_module_request()->get_posts();
		$l_actionproc = $this->get_action_processor();
		$l_tpl = $this->get_module_request()->get_template();

		try
		{
			if ($this->category_init())
			{
				// Handle selected navigation mode and fill action processor.
				$l_navModeResult = $this->handle_navmode($l_posts[C__GET__NAVMODE]);

				// Process actions (if there are any).
				$l_actionproc->process();

				// Show success message.
				isys_notify::success(_L('LC__INFOBOX__DATA_WAS_SAVED'));

				if (isset($l_navModeResult[0][1]) && $l_navModeResult[0][1] !== true && $l_navModeResult[0][1] > 0)
				{
					$l_tpl->assign('categoryID', $l_navModeResult[0][1]);
				} // if
			} // if
		}
		catch (isys_exception_auth $e)
		{
			$index_includes["contentbottomcontent"] = 'exception-auth.tpl';

			isys_notify::error(_L('LC__INFOBOX__DATA_WAS_NOT_SAVED') . ' ' . $e->getMessage(), array('sticky' => true));
			$l_tpl->assign('exception', $e->write_log());
		}
		catch (isys_exception_cmdb $e)
		{
			isys_notify::error(_L('LC__INFOBOX__DATA_WAS_NOT_SAVED') . ' ' . $e->getMessage(), array('sticky' => true));
			$l_tpl->assign('g_error', $e->getMessage());
		}
		catch (Exception $e)
		{
			isys_notify::error(_L('LC__INFOBOX__DATA_WAS_NOT_SAVED') . ' ' . $e->getMessage(), array('sticky' => true));
			$l_tpl->assign('g_error', $e->getMessage());
		} // try

		return null;
	} // function

	/**
	 * Process category view and handles the "authorization-fail" view.
	 *
	 * @throws  isys_exception_cmdb
	 * @throws  Exception|isys_exception_cmdb
	 * @return  null
	 */
	public function process()
	{
		// Prepare operation data
		$l_posts      = $this->get_module_request()->get_posts();
		$l_actionproc = $this->get_action_processor();

		try
		{
			if ($this->category_init())
			{
				// Handle selected navigation mode and fill action processor.
				$l_result = $this->handle_navmode($l_posts[C__GET__NAVMODE]);

				// Process actions (if there are any).
				$l_actionproc->process();

				// Process object overview.
				$l_contentTop = new isys_cmdb_view_contenttop($this->m_modreq);
				$l_contentTop->process();

				// Process category view.
				$this->category_process($l_result);
			}
		}
		catch (isys_exception_auth $e)
		{
			global $index_includes;

			$index_includes["contentbottomcontent"] = 'exception-auth.tpl';
			$this->deactivate_view();
			$this->get_module_request()->get_template()->assign('exception', $e->write_log());
		}
		catch (isys_exception_cmdb $e)
		{
			throw $e;
		}
		catch (Exception $e)
		{
			throw new isys_exception_cmdb($e->getMessage());
		} // try

		return NULL;
	} // function

	/**
	 * Hides all buttons and the commentary field
	 *
	 * @author    Van Quyen Hoang <qhoang@i-doit.org>
	 */
	public function deactivate_view()
	{
		$this->get_module_request()->get_navbar()->hide_all_buttons();
		$this->get_module_request()->get_navbar()->deactivate_all_buttons();
		$this->get_module_request()->get_template()->assign('bShowCommentary', false);
	} // function

	/**
	 * Returns the filepath of the "top" template.
	 *
	 * @return  string
	 */
	public function get_template_top()
	{
		return "content/top/main_objectdetail.tpl";
	} // function

	/**
	 * Returns the filepath of the "bottom" template.
	 *
	 * @return  null
	 */
	public function get_template_bottom()
	{
		return NULL;
	} // function

	/**
	 * Returns the mandatory parameters.
	 *
	 * @param  array &$l_gets
	 */
	public function get_mandatory_parameters(&$l_gets)
	{
		$l_gets[C__CMDB__GET__OBJECT] = true;
	} // function

	/**
	 * Method for setting the optional parameters.
	 *
	 * @param  array &$l_gets
	 */
	public function get_optional_parameters(&$l_gets)
	{
		$l_gets[C__CMDB__GET__OBJECTTYPE]         = true;
		$l_gets[C__CMDB__GET__CAT_MENU_SELECTION] = true;
		$l_gets[C__CMDB__GET__CAT_LIST_VIEW]      = true;
		$l_gets[C__CMDB__GET__CATLEVEL]           = true;
		$l_gets[C__CMDB__GET__CATLEVEL_1]         = true;
		$l_gets[C__CMDB__GET__CATLEVEL_2]         = true;
		$l_gets[C__CMDB__GET__CATLEVEL_3]         = true;
		$l_gets[C__CMDB__GET__CATLEVEL_4]         = true;
		$l_gets[C__CMDB__GET__CATLEVEL_5]         = true;
		$l_gets[C__CMDB__GET__CATG]               = true;
		$l_gets[C__CMDB__GET__CATS]               = true;
	} // function

	/**
	 * Public constructor overwrites the protected one from isys_cmdb_view.
	 *
	 * @param  isys_module_request $p_modreq
	 */
	public function __construct(isys_module_request $p_modreq)
	{
		parent::__construct($p_modreq);
	} // function
} // class
?>