<?php
/**
 * i-doit - Updates
 *
 * Migration of duplicate entries
 *
 * @package     i-doit
 * @subpackage  Update
 * @copyright   synetics GmbH
 * @license     http://www.gnu.org/licenses/agpl-3.0.html GNU AGPLv3
 * @author      Van Quyen Hoang <qhoang@i-doit.com>
 */

// Initialization
global $g_comp_database, $g_absdir;
$g_migration_identifier = "duplicated_apps";

if ($this->is_migration_done($g_migration_identifier))
{
	$g_migration_log[] = "<span class=\"bold grey indent\">Migration of duplicate applications has already been done.</span>";
} else
{
	try
	{
		// Flags
		$l_migrate_application = true;
		$l_set_flag = true;

		$l_dao = new isys_cmdb_dao_category_g_application($g_comp_database);

		// Migration log instance.
		$l_mig_log = isys_log_migration::get_instance()
			->reset_log()
			->add_migration_log('notice', 'Starting migration of duplicate application assignments');

		// Remove duplicate applications and assignments
		if($l_migrate_application)
		{
			// Retrieve all duplicate applications, operating systems and services
			$l_query = 'SELECT isys_obj__title, COUNT(isys_obj__id) AS cnt FROM `isys_obj` WHERE isys_obj__isys_obj_type__id IN
				(SELECT isys_obj_type__id FROM isys_obj_type
				WHERE isys_obj_type__const = ' . $l_dao->convert_sql_text('C__OBJTYPE__APPLICATION') . ' OR
				isys_obj_type__const = ' . $l_dao->convert_sql_text('C__OBJTYPE__OPERATING_SYSTEM') . ' OR
				isys_obj_type__const = ' . $l_dao->convert_sql_text('C__OBJTYPE__SERVICE') . ') GROUP BY isys_obj__title HAVING cnt > 1';

			$l_res = $l_dao->retrieve($l_query);
			$l_count = $l_res->num_rows();
			if ($l_count > 0)
			{
				$l_mig_log->notice($l_count . ' duplicate applications found.');
				$l_assignments = array();
				while ($l_row = $l_res->get_row())
				{
					$l_title = $l_row['isys_obj__title'];
					$l_sql = 'SELECT isys_obj__id FROM isys_obj WHERE isys_obj__title = ' . $l_dao->convert_sql_text($l_title);
					$l_res2 = $l_dao->retrieve($l_sql);
					$l_counter = 0;
					while ($l_row2 = $l_res2->get_row())
					{
						// Check if application is assigned
						$l_check_assign = 'SELECT * FROM isys_catg_application_list
						INNER JOIN isys_connection ON isys_connection__id = isys_catg_application_list__isys_connection__id
						WHERE isys_connection__isys_obj__id = ' . $l_dao->convert_sql_id($l_row2['isys_obj__id']);
						$l_res3 = $l_dao->retrieve($l_check_assign);
						$l_count2 = $l_res3->num_rows();
						if ($l_count2 > 0)
						{
							while ($l_row3 = $l_res3->get_row())
							{
								// assigned it-services
								$l_it_services = $l_dao->get_assigned_it_services($l_row3["isys_catg_application_list__isys_catg_relation_list__id"]);
								$l_req = new isys_request();
								$l_req->set_row($l_row3);
								$l_db_schema = $l_dao->callback_property_assigned_database_schema($l_req);
								// Cache the assignments
								$l_assignments[$l_title][] = array(
									$l_row3['isys_catg_application_list__id'],
									$l_row3['isys_catg_application_list__isys_obj__id'],
									$l_db_schema,
									$l_it_services
								);
								// remove assignment
								$l_dao->save($l_row3['isys_catg_application_list__id'],
											 $l_row3['isys_catg_application_list__status'],
											 null,
											 $l_row3['isys_catg_application_list__description'],
											 $l_row3['isys_catg_application_list__isys_cats_lic_list__id'],
											 null,
											 $l_it_services,
											 $l_row3['isys_catg_application_list__isys_cats_app_variant_list__id'],
											 $l_row3['isys_catg_application_list__bequest_nagios_services']);
							} // while
						} // if
						if ($l_counter > 0)
						{
							// Remove duplicate application
							$l_dao->delete_object($l_row2['isys_obj__id']);
						} // if
						$l_counter++;
					} // while
				} // while
				// Reassign applications
				$l_mig_log->notice('Reassigning applications..');
				if (count($l_assignments) > 0)
				{
					foreach ($l_assignments AS $l_title => $l_connections)
					{
						$l_sql = 'SELECT isys_obj__id FROM isys_obj WHERE isys_obj__title = ' . $l_dao->convert_sql_text($l_title);
						$l_app_id = $l_dao->retrieve($l_sql)->get_row_value('isys_obj__id');
						foreach ($l_connections AS $l_connected_object)
						{
							// reassign applications
							$l_check_assign = 'SELECT * FROM isys_catg_application_list
								WHERE isys_catg_application_list__id = ' . $l_dao->convert_sql_id($l_connected_object[0]);
							$l_res3 = $l_dao->retrieve($l_check_assign);
							$l_count2 = $l_res3->num_rows();
							if ($l_count2 > 0)
							{
								while ($l_row3 = $l_res3->get_row())
								{
									// assigned applications
									$l_res4 = $l_dao->get_applications($l_row3['isys_catg_application_list__isys_obj__id']);
									$l_assigned_applications = array();
									while ($l_row4 = $l_res4->get_row())
									{
										$l_assigned_applications[] = $l_row4['isys_obj__title'];
									} // while
									if (!in_array($l_connected_object[1], $l_assigned_applications))
									{
										// reassign application assignment
										$l_dao->save($l_row3['isys_catg_application_list__id'],
													 $l_row3['isys_catg_application_list__status'],
													 $l_app_id,
													 $l_row3['isys_catg_application_list__description'],
													 $l_row3['isys_catg_application_list__isys_cats_lic_list__id'],
													 $l_connected_object[2],
													 $l_connected_object[3],
													 $l_row3['isys_catg_application_list__isys_cats_app_variant_list__id'],
													 $l_row3['isys_catg_application_list__bequest_nagios_services']);
									} // while
								} // while
							} // if
						} // foreach
					} // foreach
				} // if
				$l_mig_log->notice('Removing duplicate applications was successful.');
			}
			else
			{
				$l_mig_log->notice('No duplicate applications found.');
			} // if
		} // if

		$l_dao->apply_update();

		// Set flag in db
		if($l_set_flag)
		{
			$this->migration_done($g_migration_identifier);
		} // if

		$g_migration_log[] = "<span class=\"bold indent\">Migration successful.</span>";

		$g_migration_log = array_merge($l_mig_log->get_migration_log(), $g_migration_log);
	} catch (Exception $e)
	{
		$g_migration_log[] = "<span class=\"bold red\">" . $e->getMessage() . "</span>";
	} // try
}