<?php
/**
 * i-doit - Updates
 *
 * Migration of object list person
 *
 * @package     i-doit
 * @subpackage  Update
 * @copyright   synetics GmbH
 * @license     http://www.gnu.org/licenses/agpl-3.0.html GNU AGPLv3
 * @author      Van Quyen Hoang <qhoang@i-doit.com>
 */

// Initialization
global $g_comp_database, $g_absdir;
$g_migration_identifier = "migration_contact_obj_list";

if ($this->is_migration_done($g_migration_identifier))
{
	$g_migration_log[] = "<span class=\"bold grey indent\">Migration of the object list for object type person has already been done.</span>";
} else
{
	try
	{
		$l_dao = new isys_cmdb_dao($g_comp_database);
		$l_dao_property = new isys_cmdb_dao_category_property($g_comp_database);

		// Migration log instance.
		$l_mig_log = isys_log_migration::get_instance()
			->reset_log()
			->add_migration_log('notice', 'Starting migration of the object list for object type person.');

		$l_sql = 'SELECT isysgui_cats__id FROM isysgui_cats WHERE isysgui_cats__const = \'C__CATS__PERSON\';';
		$l_cats_person = $l_dao->retrieve($l_sql)->get_row_value('isysgui_cats__id');

		$l_obj_types_res = $l_dao->get_objtype_by_cats_id($l_cats_person);
		$l_obj_types = array();
		while($l_row = $l_obj_types_res->get_row())
		{
			$l_obj_types[] = $l_row['isys_obj_type__id'];
		} // while

		if(count($l_obj_types) > 0)
		{
			$l_sql = 'SELECT isys_property_2_cat__id, isys_property_2_cat__prop_key FROM isys_property_2_cat WHERE isys_property_2_cat__cat_const = \'C__CATS__PERSON_MASTER\';';
			$l_person_prop_res = $l_dao->retrieve($l_sql);
			$l_person_properties = array();
			while($l_row = $l_person_prop_res->get_row())
			{
				$l_person_properties[$l_row['isys_property_2_cat__prop_key']] = $l_row['isys_property_2_cat__id'];
			} // while

			$l_sql = 'SELECT * FROM isys_obj_type_list WHERE isys_obj_type_list__isys_obj_type__id IN (' . implode(',', $l_obj_types) . ');';
			$l_res = $l_dao->retrieve($l_sql);

			$l_first_name = array(
				C__PROPERTY_TYPE__STATIC,
				'first_name',
				'isys_cats_person_list__first_name',
				'LC__CONTACT__PERSON_FIRST_NAME',
				'isys_cmdb_dao_category_s_person_master::get_properties_ng',
				false
			);

			$l_last_name = array(
				C__PROPERTY_TYPE__STATIC,
				'last_name',
				'isys_cats_person_list__last_name',
				'LC__CONTACT__PERSON_LAST_NAME',
				'isys_cmdb_dao_category_s_person_master::get_properties_ng',
				false
			);

			while ($l_row = $l_res->get_row())
			{
				$l_properties = isys_format_json::decode($l_row['isys_obj_type_list__config']);
				$l_prop_ids = array();
				foreach($l_properties AS $l_key => $l_property)
				{
					if($l_property[1] == '_lastname')
					{
						$l_properties[$l_key] = $l_last_name;
					}
					elseif($l_property[1] == '_firstname')
					{
						$l_properties[$l_key] = $l_first_name;
					}

					if($l_property[1] !== '_id')
					{
						if($l_property[1] === '_firstname')
						{
							$l_prop_ids[] = $l_person_properties['first_name'];
						}
						elseif($l_property[1] === '_lastname')
						{
							$l_prop_ids[] = $l_person_properties['last_name'];
						}
						else
						{
							$l_prop_ids[] = $l_person_properties[$l_property[1]];
						} // if
					} // if
				} // foreach
				$l_new_properties = isys_format_json::encode($l_properties);

				$l_new_query = $l_dao_property->create_property_query_for_lists($l_prop_ids, $l_row['isys_obj_type_list__isys_obj_type__id']);

				$l_update = 'UPDATE isys_obj_type_list SET isys_obj_type_list__query = ' . $l_dao->convert_sql_text($l_new_query) .
					', isys_obj_type_list__config = ' . $l_dao->convert_sql_text($l_new_properties) .
					' WHERE isys_obj_type_list__id = ' . $l_dao->convert_sql_id($l_row['isys_obj_type_list__id']);

				$l_dao->update($l_update);
			} // while
		} // if

		$l_dao->apply_update();

		// Set flag in db
		$this->migration_done($g_migration_identifier);

		$g_migration_log[] = "<span class=\"bold indent\">Migration successfull.</span>";

		$g_migration_log = array_merge($l_mig_log->get_migration_log(), $g_migration_log);
	} catch (Exception $e)
	{
		$g_migration_log[] = "<span class=\"bold red\">" . $e->getMessage() . "</span>";
	} // try
}