<?php
/**
 * i-doit - Updates
 *
 * Migration csv profiles
 *
 * @package     i-doit
 * @subpackage  Update
 * @copyright   synetics GmbH
 * @license     http://www.gnu.org/licenses/agpl-3.0.html GNU AGPLv3
 * @author      Selcuk Kekec <skekec@i-doit.com>
 */

// Initialization
global $g_comp_database, $g_comp_database_system;
$g_migration_identifier = "migration_csv_profiles";

if ($this->is_migration_done($g_migration_identifier))
{
    $g_migration_log[] = "<span class=\"bold grey indent\">Migration of the csv profiles has already been executed.</span>";
}
else
{
    try
    {
        /** @var isys_cmdb_dao $l_dao */
        $l_dao = isys_factory_cmdb_dao::get_instance('isys_cmdb_dao', $g_comp_database);
        $l_set_migration_flag = true;

        // Migration log instance.
        $l_mig_log = isys_log_migration::get_instance()
                                       ->reset_log()
                                       ->add_migration_log('notice', 'Starting migration of csv profiles.');

        $l_sql = 'SELECT * FROM isys_csv_profile WHERE TRUE';
        $l_res = $l_dao->retrieve($l_sql);

        if ($l_res->num_rows())
        {
            while ($l_row = $l_res->get_row())
            {
                $l_mig_log->add_migration_log('notice',
                                              'Handling csv profile \'' . $l_row['isys_csv_profile__title'] . '\'');

                $l_profile_data = isys_format_json::decode($l_row['isys_csv_profile__data']);

                // Are there assignments in this profile
                if (is_array($l_profile_data) && is_array($l_profile_data['assignments']))
                {
                    // Transform category ids into constants
                    foreach ($l_profile_data['assignments'] AS &$l_assignment)
                    {
                        if (is_numeric($l_assignment['category']))
                        {
                            $l_assignment['category'] = $l_dao->get_cat_const_by_id($l_assignment['category']);
                        } // if
                    } // foreach
                } // if

                // Save transformed profile
                $l_dao->update('UPDATE isys_csv_profile SET isys_csv_profile__data = ' .
                               $l_dao->convert_sql_text(isys_format_json::encode($l_profile_data)) . ' WHERE isys_csv_profile__id = ' . $l_row['isys_csv_profile__id'] . ';');
            } // while

            $l_dao->apply_update();
        } // if


        // Set flag in db
        if ($l_set_migration_flag)
        {
            $this->migration_done($g_migration_identifier);
        } // if

        $g_migration_log[] = "<span class=\"bold indent\">Migration successful.</span>";

        $g_migration_log = array_merge($l_mig_log->get_migration_log(), $g_migration_log);
    }
    catch (Exception $e)
    {
        $g_migration_log[] = "<span class=\"bold red\">" . $e->getMessage() . "</span>";
    } // try
} // if
