<?php
namespace idoit\Module\Cmdb\Model\Ci\Table;

/**
 * i-doit
 *
 * Ci Table Config
 *
 * @package     i-doit
 * @subpackage  Cmdb
 * @author      Leonard Fischer <lfischer@i-doit.com>
 * @version     1.0
 * @since       1.8
 * @copyright   synetics GmbH
 * @license     http://www.gnu.org/licenses/agpl-3.0.html GNU AGPLv3
 */
class Config
{
    /**
     * @var  array
     */
    private $properties;

    /**
     * @var  string
     */
    private $sortingProperty;

    /**
     * @var  string
     */
    private $sortingDirection;

    /**
     * @var  string
     */
    private $filterProperty;

    /**
     * @var  string
     */
    private $filterValue;

    /**
     * @var  boolean
     */
    private $rowClickable;

    /**
     * @var  integer
     */
    private $groupingType;

    /**
     * @return Property[]
     */
    public function getProperties()
    {
        return $this->properties;
    } // function

    /**
     * @param Property $properties
     *
     * @return Config
     */
    public function addProperty(Property $properties)
    {
        $this->properties[] = $properties;

        return $this;
    } // function

    /**
     * @param Property[] $properties
     *
     * @return Config
     * @throws \isys_exception_general
     */
    public function setProperties(array $properties)
    {
        foreach ($properties as $property)
        {
            if (!is_a($property, 'idoit\\Module\\Cmdb\\Model\\Ci\\Table\\Property'))
            {
                throw new \isys_exception_general('All array items need to be instances of "idoit\\Module\\Cmdb\\Model\\Ci\\Table\\Property"!');
            } // if
        } // function

        $this->properties = $properties;

        return $this;
    } // function

    /**
     * @return  string
     */
    public function getSortingProperty()
    {
        return $this->sortingProperty;
    } // function

    /**
     * @param string $sortingProperty
     *
     * @return  Config
     */
    public function setSortingProperty($sortingProperty)
    {
        $this->sortingProperty = $sortingProperty;

        return $this;
    } // function

    /**
     * @return  string
     */
    public function getSortingDirection()
    {
        return $this->sortingDirection;
    } // function

    /**
     * @param string $sortingDirection
     *
     * @return  Config
     */
    public function setSortingDirection($sortingDirection)
    {
        $sortingDirection = strtoupper($sortingDirection);

        if ($sortingDirection === 'ASC' || $sortingDirection === 'DESC')
        {
            $this->sortingDirection = $sortingDirection;
        } // if

        return $this;
    } // function

    /**
     * @return  string
     */
    public function getFilterProperty()
    {
        return $this->filterProperty;
    } // function

    /**
     * @param string $filterProperty
     *
     * @return  Config
     */
    public function setFilterProperty($filterProperty)
    {
        $this->filterProperty = $filterProperty;

        return $this;
    } // function

    /**
     * @return  string
     */
    public function getFilterValue()
    {
        return $this->filterValue;
    } // function

    /**
     * @param string $filterValue
     *
     * @return  Config
     */
    public function setFilterValue($filterValue)
    {
        $this->filterValue = $filterValue;

        return $this;
    } // function

    /**
     * @return  boolean
     */
    public function isRowClickable()
    {
        return $this->rowClickable;
    } // function

    /**
     * @param boolean $rowClickable
     *
     * @return  Config
     */
    public function setRowClickable($rowClickable)
    {
        $this->rowClickable = !!$rowClickable;

        return $this;
    } // function

    /**
     * @return  integer
     */
    public function getGroupingType()
    {
        return $this->groupingType;
    } // function

    /**
     * @param integer $groupingType
     *
     * @return  Config
     */
    public function setGroupingType($groupingType)
    {
        $this->groupingType = (int) $groupingType;

        return $this;
    } // function
} // class