<?php
namespace idoit\Module\Cmdb\Model\Matcher\Identifier;

use idoit\Module\Cmdb\Model\Matcher\AbstractIdentifier;

/**
 * i-doit
 *
 * Ci Models
 *
 * @package     i-doit
 * @subpackage  Cmdb
 * @author      Dennis Stücken <dstuecken@i-doit.com>
 * @version     1.8
 * @copyright   synetics GmbH
 * @license     http://www.gnu.org/licenses/agpl-3.0.html GNU AGPLv3
 */
class Fqdn extends AbstractIdentifier
{
    /**
     * Key for this identifier, has to be unique
     */
    const KEY = 'fqdn';

    /**
     * @inherit
     * @var string
     */
    protected $title = 'FQDN';

    /**
     * @inherit
     * @var int
     */
    protected static $bit = 64;

    /**
     * @inherit
     * @var string
     */
    protected $sqlSelect = '';

    /**
     * Usage options for Match Identifier
     *
     * @var array
     */
    protected $usableIn = [
        'JDisc',
        'H-Inventory',
        'OCS',
        'CSV'
    ];

    /**
     * Fqdn constructor.
     */
    public function __construct()
    {
        $this->sqlSelect = 'SELECT isys_obj__id AS id, isys_obj__title AS title, isys_obj__isys_obj_type__id AS type, \'' . self::KEY . '\' AS identKey
            FROM isys_obj
            INNER JOIN (
                SELECT CONCAT_WS(\'.\', ip.isys_catg_ip_list__hostname, dns.isys_net_dns_domain__title) AS fqdn, isys_catg_ip_list__isys_obj__id AS id
                FROM isys_catg_ip_list AS ip
                INNER JOIN `isys_catg_ip_list_2_isys_net_dns_domain` AS conn ON ip.isys_catg_ip_list__id = conn.isys_catg_ip_list__id
                INNER JOIN isys_net_dns_domain AS dns ON dns.isys_net_dns_domain__id = conn.isys_net_dns_domain__id
                WHERE ip.isys_catg_ip_list__hostname != \'\') AS fqdnjoin ON fqdnjoin.id = `isys_obj__id`
            WHERE fqdnjoin.fqdn = :value:
            AND isys_obj__status = :status: :condition:';
    } // function
} // class