<?php
/**
 * i-doit
 *
 * Monitoring module class.
 *
 * @package     i-doit
 * @subpackage  Modules
 * @author      Leonard Fischer <lfischer@i-doit.com>
 * @version     1.0.0
 * @copyright   synetics GmbH
 * @license     http://www.gnu.org/licenses/agpl-3.0.html GNU AGPLv3
 * @since       i-doit 1.3.0
 */

class isys_module_monitoring extends isys_module implements isys_module_interface
{
	/**
	 * @var bool
	 */
	protected static $m_licenced = true;

	// Define, if this module shall be displayed in the named menus.
	const DISPLAY_IN_MAIN_MENU   = false;
	const DISPLAY_IN_SYSTEM_MENU = true;

	/**
	 * Variable which holds the template component.
	 * @var  isys_component_template
	 */
	protected $m_tpl = null;

	/**
	 * Variable which holds the monitoring DAO class.
	 * @var  isys_monitoring_dao_hosts
	 */
	protected $m_dao = null;

	/**
	 * Variable which holds the database component.
	 * @var  isys_component_database
	 */
	protected $m_db = null;


	/**
	 * Initializes the module.
	 *
	 * @param   isys_module_request& $p_req
	 * @return  isys_module_monitoring
	 * @author  Leonard Fischer <lfischer@i-doit.com>
	 */
	public function init(isys_module_request &$p_req)
	{
		$this->m_db = $p_req->get_database();
		$this->m_tpl = $p_req->get_template()->assign('www_dir', self::get_tpl_www_dir());
		$this->m_dao = new isys_monitoring_dao_hosts($this->m_db);

		return $this;
	} // function


	/**
	 * Static method for retrieving the path, to the modules templates.
	 *
	 * @static
	 * @return  string
	 * @author  Leonard Fischer <lfischer@i-doit.com>
	 */
	public static function get_tpl_dir ()
	{
		return __DIR__ . DS . 'templates' . DS;
	} // function


	/**
	 * Static method for retrieving the path, to the modules templates.
	 *
	 * @static
	 * @global  array $g_dirs
	 * @return  string
	 * @author  Leonard Fischer <lfischer@i-doit.com>
	 */
	public static function get_tpl_www_dir ()
	{
		global $g_config;

		return $g_config['www_dir'] . 'src/classes/modules/monitoring/templates/';
	} // function


	/**
	 * This method builds the tree for the menu.
	 *
	 * @param   isys_component_tree  $p_tree
	 * @param   boolean              $p_system_module
	 * @param   integer              $p_parent
	 * @author  Leonard Fischer <lfischer@i-doit.com>
	 * @see     isys_module_cmdb->build_tree();
	 */
	public function build_tree(isys_component_tree &$p_tree, $p_system_module = true, $p_parent)
	{
		global $g_dirs;

		if ($p_system_module)
		{
			$p_tree->add_node(
				C__MODULE__MONITORING . 0,
				$p_parent,
				_L('LC__MONITORING__LIVESTATUS_NDO__CONFIGURATION'),
				isys_helper_link::create_url(array(
					C__GET__MODULE_ID => C__MODULE__SYSTEM,
					C__GET__MODULE_SUB_ID => C__MODULE__MONITORING,
					C__GET__SETTINGS_PAGE => 'livestatus_ndo_config',
					C__GET__TREE_NODE => C__MODULE__MONITORING . 0
				)),
				'',
				$g_dirs['images'] . 'icons/silk/cog_edit.png',
				(int) ($_GET[C__GET__TREE_NODE] == C__MODULE__MONITORING . 0));

			$p_tree->add_node(
				C__MODULE__MONITORING . 1,
				$p_parent,
				_L('LC__MONITORING__EXPORT__CONFIGURATION'),
				isys_helper_link::create_url(array(
					C__GET__MODULE_ID => C__MODULE__SYSTEM,
					C__GET__MODULE_SUB_ID => C__MODULE__MONITORING,
					C__GET__SETTINGS_PAGE => 'export_config',
					C__GET__TREE_NODE => C__MODULE__MONITORING . 1
				)),
				'',
				$g_dirs['images'] . 'icons/silk/pencil.png',
				(int) ($_GET[C__GET__TREE_NODE] == C__MODULE__MONITORING . 1));
		} // if
	} // function


	/**
	 * Start method.
	 */
	public function start()
	{
		$this->m_tpl->assign('tpl_www_dir', self::get_tpl_www_dir());

		switch ($_GET[C__GET__SETTINGS_PAGE])
		{
			default:
			case 'livestatus_ndo_config':
				$this->process_livestatus_ndo_config();
				break;

			case 'export_config':
				$this->process_export_config();
				break;
		} // switch
	} // function


    /**
     * Attach live status of object to contenttop header
     *
     * @param $p_catdata
     */
    public static function process_content_top($p_catdata)
    {
        global $g_comp_database;

        if (defined('C__MONITORING__TYPE_LIVESTATUS'))
        {
            if (count(isys_factory::get_instance('isys_monitoring_dao_hosts', $g_comp_database)->get_data(null, C__MONITORING__TYPE_LIVESTATUS)) > 0)
            {
                global $index_includes;
                $index_includes['contenttopobjectdetail'][] = __DIR__ . '/templates/contenttop/main_objectdetail_livestatus.tpl';
            } // if
        } // if
    } // function


	/**
	 * Method for processing the monitoring configuration pages.
	 *
	 * @author  Leonard Fischer <lfischer@i-doit.com>
	 */
	protected function process_livestatus_ndo_config ()
	{
		$l_id = $_POST['id'][0] ? : $_GET['id'];
		$l_navmode = $_POST[C__GET__NAVMODE] ? : $_GET[C__GET__NAVMODE];

		switch ($l_navmode)
		{
			case C__NAVMODE__DELETE:
				$this->process_livestatus_ndo_config__delete($_POST['id']);
				$this->process_livestatus_ndo_config__list();
				break;

			case C__NAVMODE__SAVE:
				$this->process_livestatus_ndo_config__save($_POST['config_id'], $_POST);
				$this->process_livestatus_ndo_config__list();
				break;

			case C__NAVMODE__EDIT:
				$this->process_livestatus_ndo_config__edit($l_id);
				break;

			case C__NAVMODE__NEW:
				$this->process_livestatus_ndo_config__edit();
				break;

			default:
				$this->process_livestatus_ndo_config__list();
		} // swtich
	} // function


	/**
	 * Delete-action for removing the selected configuration from the database.
	 *
	 * @param   mixed  $p_config_id
	 * @author  Leonard Fischer <lfischer@i-doit.com>
	 */
	protected function process_livestatus_ndo_config__delete ($p_config_id)
	{
		$this->m_dao->delete($p_config_id);
	} // function


	/**
	 * Save-action for writing the configuration to the database. This is used for "CREATE" and "UPDATE".
	 *
	 * @param   integer  $p_config_id
	 * @param   array    $p_configuration
	 * @author  Leonard Fischer <lfischer@i-doit.com>
	 */
	protected function process_livestatus_ndo_config__save ($p_config_id = null, array $p_configuration = array())
	{
		$this->m_dao->save($p_config_id, array(
			'title' => $p_configuration['C__MONITORING__CONFIG__TITLE'],
			'active' => $p_configuration['C__MONITORING__CONFIG__ACTIVE'],
			'type' => $p_configuration['C__MONITORING__CONFIG__TYPE'],
			'connection' => $p_configuration['C__MONITORING__CONFIG__CONNECTION'],
			'path' => $p_configuration['C__MONITORING__CONFIG__PATH'],
			'address' => $p_configuration['C__MONITORING__CONFIG__ADDRESS'],
			'port' => $p_configuration['C__MONITORING__CONFIG__PORT'],
			'dbname' => $p_configuration['C__MONITORING__CONFIG__DBNAME'],
			'dbprefix' => $p_configuration['C__MONITORING__CONFIG__DBPREFIX'],
			'username' => $p_configuration['C__MONITORING__CONFIG__USERNAME'],
			'password' => isys_helper_crypt::encrypt($p_configuration['C__MONITORING__CONFIG__PASSWORD'])
		));
	} // function


	/**
	 * Edit-action for displaying the configuration form. This is used for "NEW" and "EDIT".
	 *
	 * @param   integer  $p_config_id
	 * @author  Leonard Fischer <lfischer@i-doit.com>
	 */
	protected function process_livestatus_ndo_config__edit ($p_config_id = null)
	{
		global $index_includes;

		$l_yes_no = serialize(get_smarty_arr_YES_NO());
		$l_livestatus_connections = serialize(array(
			C__MONITORING__LIVESTATUS_TYPE__TCP => 'TCP',
			C__MONITORING__LIVESTATUS_TYPE__UNIX => 'UNIX Socket'
		));
		$l_monitoring_type = serialize(array(
			C__MONITORING__TYPE_LIVESTATUS => 'Livestatus',
			C__MONITORING__TYPE_NDO => 'NDO',
		));

		$l_rules = array();
		$l_rules['C__MONITORING__CONFIG__ACTIVE']['p_strSelectedID'] = 1;
		$l_rules['C__MONITORING__CONFIG__ACTIVE']['p_arData'] = $l_yes_no;
		$l_rules['C__MONITORING__CONFIG__TYPE']['p_strSelectedID'] = C__MONITORING__TYPE_LIVESTATUS;
		$l_rules['C__MONITORING__CONFIG__TYPE']['p_arData'] = $l_monitoring_type;
		$l_rules['C__MONITORING__CONFIG__CONNECTION']['p_arData'] = $l_livestatus_connections;
		$l_rules['C__MONITORING__CONFIG__CONNECTION']['p_strSelectedID'] = C__MONITORING__LIVESTATUS_TYPE__TCP;

		if ($p_config_id !== null && $p_config_id > 0)
		{
			$l_host = isys_factory::get_instance('isys_monitoring_dao_hosts', $this->m_db)->get_data($p_config_id)->get_row();

			$l_rules['config_id']['p_strValue'] = $p_config_id;
			$l_rules['C__MONITORING__CONFIG__TITLE']['p_strValue'] = $l_host['isys_monitoring_hosts__title'];
			$l_rules['C__MONITORING__CONFIG__ACTIVE']['p_strSelectedID'] = $l_host['isys_monitoring_hosts__active'] ?: 1;
			$l_rules['C__MONITORING__CONFIG__TYPE']['p_strSelectedID'] = $l_host['isys_monitoring_hosts__type'] ?: C__MONITORING__TYPE_LIVESTATUS;
			$l_rules['C__MONITORING__CONFIG__CONNECTION']['p_strSelectedID'] = $l_host['isys_monitoring_hosts__connection'] ?: C__MONITORING__LIVESTATUS_TYPE__TCP;
			$l_rules['C__MONITORING__CONFIG__ADDRESS']['p_strValue'] = $l_host['isys_monitoring_hosts__address'];
			$l_rules['C__MONITORING__CONFIG__PORT']['p_strValue'] = $l_host['isys_monitoring_hosts__port'];
			$l_rules['C__MONITORING__CONFIG__PATH']['p_strValue'] = $l_host['isys_monitoring_hosts__path'];
			$l_rules['C__MONITORING__CONFIG__DBNAME']['p_strValue'] = $l_host['isys_monitoring_hosts__dbname'];
			$l_rules['C__MONITORING__CONFIG__DBPREFIX']['p_strValue'] = $l_host['isys_monitoring_hosts__dbprefix'];
			$l_rules['C__MONITORING__CONFIG__USERNAME']['p_strValue'] = $l_host['isys_monitoring_hosts__username'];
			$l_rules['C__MONITORING__CONFIG__PASSWORD']['p_strValue'] = isys_helper_crypt::decrypt($l_host['isys_monitoring_hosts__password']);

			$l_show = array(
				$l_rules['C__MONITORING__CONFIG__TYPE']['p_strSelectedID']
			);

			// Only hide the tcp/unix fields when the type is "livestatus".
			if ($l_rules['C__MONITORING__CONFIG__TYPE']['p_strSelectedID'] == C__MONITORING__TYPE_LIVESTATUS)
			{
				$l_show[] = $l_rules['C__MONITORING__CONFIG__CONNECTION']['p_strSelectedID'];
			} // if
		} // if

		$this->m_tpl
			->activate_editmode()
			->assign('show_options', isys_format_json::encode($l_show))
			->smarty_tom_add_rules('tom.content.bottom.content', $l_rules);

		isys_component_template_navbar::getInstance()
			->set_active(true, C__NAVBAR_BUTTON__SAVE)
			->set_active(true, C__NAVBAR_BUTTON__CANCEL);

		$index_includes['contentbottomcontent'] = self::get_tpl_dir() . 'livestatus_ndo_config_form.tpl';
	} // function


	/**
	 * List-action for displaying the Livestatus / NDO configurations.
	 *
	 * @author  Leonard Fischer <lfischer@i-doit.com>
	 */
	protected function process_livestatus_ndo_config__list ()
	{
		global $index_includes;

		$l_url_params = $_GET;
		$l_url_params[C__GET__NAVMODE] = C__NAVMODE__EDIT;
		unset($l_url_params[C__GET__MAIN_MENU__NAVIGATION_ID], $l_url_params['id']);

		$l_list_headers = array(
			'isys_monitoring_hosts__id' => 'ID',
			'isys_monitoring_hosts__active' => 'aktiv',
			'isys_monitoring_hosts__title' => 'LC__CMDB__CATG__UI_TITLE',
			'isys_monitoring_hosts__type' => 'LC__MONITORING__TYPE'
		);

		$l_nagios_hosts_result = $this->m_dao->get_data();
		$l_nagios_hosts_count = count($l_nagios_hosts_result);

		$l_list = isys_factory::get_instance('isys_component_list')
			->set_data(null, $l_nagios_hosts_result)
			->set_row_modifier($this, 'process_livestatus_ndo_config__row_modifier')
			->config($l_list_headers, isys_helper_link::create_url($l_url_params) . '&id=[{isys_monitoring_hosts__id}]', '[{isys_monitoring_hosts__id}]');

		if ($l_list->createTempTable())
		{
			$this->m_tpl->assign('configuration_table', $l_list->getTempTableHtml());
		} // if

		$this->m_tpl->assign('content_title', _L('LC__MONITORING'));

		isys_component_template_navbar::getInstance()
			->set_active(true, C__NAVBAR_BUTTON__NEW)
			->set_active(($l_nagios_hosts_count > 0), C__NAVBAR_BUTTON__EDIT)
			->set_active(($l_nagios_hosts_count > 0), C__NAVBAR_BUTTON__DELETE)
			->set_visible(true, C__NAVBAR_BUTTON__NEW)
			->set_visible(true, C__NAVBAR_BUTTON__EDIT)
			->set_visible(true, C__NAVBAR_BUTTON__DELETE);

		$index_includes['contentbottomcontent'] = self::get_tpl_dir() . 'livestatus_ndo_config_list.tpl';
	} // function


	/**
	 * Row modifier for the configuration list.
	 *
	 * @param   array  $p_row
	 * @author  Leonard Fischer <lfischer@i-doit.com>
	 */
	public function process_livestatus_ndo_config__row_modifier (&$p_row)
	{
		global $g_dirs;

		$p_row['isys_monitoring_hosts__active'] = ($p_row['isys_monitoring_hosts__active'] > 0) ?
			'<img src="' . $g_dirs['images'] . 'icons/silk/bullet_green.png" class="vam mr5" /><span class="vam">' . _L('LC__UNIVERSAL__YES') . '</span>' :
			'<img src="' . $g_dirs['images'] . 'icons/silk/bullet_red.png" class="vam mr5" /><span class="vam">' . _L('LC__UNIVERSAL__NO') . '</span>';
	} // function


	/**
	 * Method for processing the Monitoring export configuration pages.
	 *
	 * @author  Leonard Fischer <lfischer@i-doit.com>
	 */
	protected function process_export_config ()
	{
		$l_id = $_POST['id'][0] ? : $_GET['id'];
		$l_navmode = $_POST[C__GET__NAVMODE] ? : $_GET[C__GET__NAVMODE];

		switch ($l_navmode)
		{
			case C__NAVMODE__DELETE:
				$this->process_export_config__delete($_POST['id']);
				$this->process_export_config__list();
				break;

			case C__NAVMODE__SAVE:
				$this->process_export_config__save($_POST['config_id'], $_POST);
				$this->process_export_config__list();
				break;

			case C__NAVMODE__EDIT:
				$this->process_export_config__edit($l_id);
				break;

			case C__NAVMODE__NEW:
				$this->process_export_config__edit();
				break;

			default:
				$this->process_export_config__list();
		} // swtich
	} // function


	/**
	 * Delete-action for removing the selected export configuration from the database.
	 *
	 * @param   mixed  $p_config_id
	 * @author  Leonard Fischer <lfischer@i-doit.com>
	 */
	protected function process_export_config__delete ($p_config_id)
	{
		$this->m_dao->delete_export_config($p_config_id);
	} // function


	/**
	 * Save-action for writing the export configuration to the database. This is used for "CREATE" and "UPDATE".
	 *
	 * @param   integer  $p_config_id
	 * @param   array    $p_configuration
	 * @author  Leonard Fischer <lfischer@i-doit.com>
	 */
	protected function process_export_config__save ($p_config_id = null, array $p_configuration = array())
	{
		$this->m_dao->save_export_config($p_config_id, array(
			'title' => $p_configuration['C__MONITORING__CONFIG__TITLE'],
			'path' => $p_configuration['C__MONITORING__CONFIG__PATH'],
			'address' => $p_configuration['C__MONITORING__CONFIG__ADDRESS']
		));
	} // function


	/**
	 * Edit-action for displaying the export configuration form. This is used for "NEW" and "EDIT".
	 *
	 * @param   integer  $p_config_id
	 * @author  Leonard Fischer <lfischer@i-doit.com>
	 */
	protected function process_export_config__edit ($p_config_id = null)
	{
		global $index_includes, $g_absdir;

		$l_rules = array();

		if ($p_config_id !== null && $p_config_id > 0)
		{
			$l_host = isys_factory::get_instance('isys_monitoring_dao_hosts', $this->m_db)->get_export_data($p_config_id)->get_row();

			$l_rules['config_id']['p_strValue'] = $p_config_id;
			$l_rules['C__MONITORING__CONFIG__TITLE']['p_strValue'] = $l_host['isys_monitoring_export_config__title'];
			$l_rules['C__MONITORING__CONFIG__ADDRESS']['p_strValue'] = $l_host['isys_monitoring_export_config__address'];
			$l_rules['C__MONITORING__CONFIG__PATH']['p_strValue'] = $l_host['isys_monitoring_export_config__path'];
		} // if

		$l_rules['C__MONITORING__CONFIG__ADDRESS']['p_strPlaceholder'] = 'http://internal-monitoring-host.int/';
		$l_rules['C__MONITORING__CONFIG__PATH']['p_strPlaceholder'] = $g_absdir;

		$this->m_tpl
			->activate_editmode()
			->smarty_tom_add_rules('tom.content.bottom.content', $l_rules);

		isys_component_template_navbar::getInstance()
			->set_active(true, C__NAVBAR_BUTTON__SAVE)
			->set_active(true, C__NAVBAR_BUTTON__CANCEL);

		$index_includes['contentbottomcontent'] = self::get_tpl_dir() . 'export_config_form.tpl';
	} // function


	/**
	 * List-action for displaying the export configurations.
	 *
	 * @author  Leonard Fischer <lfischer@i-doit.com>
	 */
	protected function process_export_config__list ()
	{
		global $index_includes;

		$l_url_params = $_GET;
		$l_url_params[C__GET__NAVMODE] = C__NAVMODE__EDIT;
		unset($l_url_params[C__GET__MAIN_MENU__NAVIGATION_ID], $l_url_params['id']);

		$l_list_headers = array(
			'isys_monitoring_export_config__id' => 'ID',
			'isys_monitoring_export_config__title' => 'LC__CMDB__CATG__UI_TITLE',
			'isys_monitoring_export_config__path' => 'LC__MONITORING__PATH',
			'isys_monitoring_export_config__address' => 'LC__MONITORING__ADDRESS'
		);

		$l_nagios_hosts_result = $this->m_dao->get_export_data();
		$l_nagios_hosts_count = count($l_nagios_hosts_result);

		$l_list = isys_factory::get_instance('isys_component_list')
			->set_data(null, $l_nagios_hosts_result)
			->config($l_list_headers, isys_helper_link::create_url($l_url_params) . '&id=[{isys_monitoring_export_config__id}]', '[{isys_monitoring_export_config__id}]');

		if ($l_list->createTempTable())
		{
			$this->m_tpl->assign('configuration_table', $l_list->getTempTableHtml());
		} // if

		isys_component_template_navbar::getInstance()
			->set_active(true, C__NAVBAR_BUTTON__NEW)
			->set_active(($l_nagios_hosts_count > 0), C__NAVBAR_BUTTON__EDIT)
			->set_active(($l_nagios_hosts_count > 0), C__NAVBAR_BUTTON__DELETE)
			->set_visible(true, C__NAVBAR_BUTTON__NEW)
			->set_visible(true, C__NAVBAR_BUTTON__EDIT)
			->set_visible(true, C__NAVBAR_BUTTON__DELETE);

		$this->m_tpl->assign('content_title', _L('LC__MONITORING'));

		$index_includes['contentbottomcontent'] = self::get_tpl_dir() . 'export_config_list.tpl';
	} // function
} // class