"use strict";

/**
 * i-doit javascript callback manager class for the new TableComponent.
 *
 * @author  Leonard Fischer <lfischer@i-doit.com>
 */
window.TableComponentCallbackManager = Class.create({
    initialize: function ($table, options) {
        this.$table = $table;
        this.$head = this.$table.down('thead');
        this.$body = this.$table.down('tbody');

        this.options = {
            rowClick:       false,
            loadingMessage: ''
        };

        Object.extend(this.options, options || {});

        this.prepareObserver();

        this.$body.select('.autostart').invoke('simulate', 'click');
    },

    prepareObserver: function () {
        if (this.options.order) {
            this.$head.on('click', '[data-action]', this.dispatch.bindAsEventListener(this));
        }

        if (this.options.rowClick)
        {
            // We can not use "this.$body.on('click', '[data-action]', ...);" because of event bubbling.
            this.$body.select('[data-action]').invoke('on', 'click', this.dispatch.bindAsEventListener(this));
        }
        else
        {
            this.$body.on('click', '[data-action]', this.dispatch.bindAsEventListener(this));
        }
    },

    dispatch: function (ev) {
        var $element = ev.findElement('[data-action]');

        ev.stopPropagation();

        switch ($element.readAttribute('data-action'))
        {
            case 'order-by-column':
                this.orderByColumn($element);
                break;

            case 'load-ndo-state':
                this.loadNDOState($element);
                break;

            case 'load-livestatus-state':
                this.loadLivestatusState($element);
                break;
        }
    },

    orderByColumn: function ($image) {
        var urlParts = document.location.href.parseQuery();

        $image.writeAttribute('src', window.dir_images + 'ajax-loading.gif');

        urlParts.orderBy = $image.readAttribute('data-order-field');
        urlParts.orderByDir = ($image.readAttribute('data-order-direction') == 'ASC' ? 'DESC' : 'ASC');

        document.location.href = window.www_dir + '?' + Object.toQueryString(urlParts);
    },

    loadNDOState: function ($button) {
        var $parent  = $button.up(),
            objectID = $button.readAttribute('data-object-id');

        $button.disable()
            .update(new Element('img', {src: window.dir_images + 'ajax-loading.gif', className: 'mr5'}))
            .insert(new Element('span').update(this.options.loadingMessage));

        new Ajax.Request('?call=monitoring_ndo&ajax=1&func=load_ndo_state', {
            method:     'post',
            parameters: {
                objID: objectID
            },
            onSuccess:  function (xhr) {
                var json = xhr.responseJSON;

                if (json.success)
                {
                    if (json.data)
                    {
                        $parent.update(display_monitoring_state(json.data));
                    }
                    else
                    {
                        $parent.addClassName('text-grey').update('-');
                    }
                }
                else
                {
                    $parent.addClassName('text-red').update(json.message || '-');
                }
            }.bind(this)
        });
    },

    loadLivestatusState: function ($button) {
        var $parent  = $button.up(),
            objectID = $button.readAttribute('data-object-id');

        $button.disable()
            .update(new Element('img', {src: window.dir_images + 'ajax-loading.gif', className: 'mr5'}))
            .insert(new Element('span').update(this.options.loadingMessage));

        new Ajax.Request('?call=monitoring_livestatus&ajax=1&func=load_livestatus_state', {
            method:     'post',
            parameters: {
                objID: objectID
            },
            onSuccess:  function (xhr) {
                var json = xhr.responseJSON;

                if (json.success)
                {
                    if (json.data)
                    {
                        $parent.update(display_monitoring_state(json.data));
                    }
                    else
                    {
                        $parent.addClassName('text-grey').update('-');
                    }
                }
                else
                {
                    $parent.addClassName('text-red').update(json.message || '-');
                }
            }.bind(this)
        });
    }
});