/**
 * Loads breadcrumb navigation for p_url
 * @param p_url String
 * @param suppressPushState bool
 */
function breadcrumb(p_url, suppressPushState)
{
    var l_url = p_url.replace(/#.*/g, ''), pushUrl;
    l_url += (l_url.match(/^.*[\?].*$/)) ? '&' : '?';

    pushUrl = l_url.parseQuery();

    delete pushUrl.call;

    new Ajax.Updater(
        'breadcrumb',
        l_url + 'request=breadcrumb&ajax=1',
        {
            method:     'get',
            onComplete: function ()
                        {
                            document.title =
                                'i-doit ' +
                                $('breadcrumb').innerHTML.replace(/\<li/g,
                                    ' > <li').stripTags().unescapeHTML().substr(6);

                            if (!suppressPushState)
                            {
                                window.pushState({url: l_url, content: 'main_content'}, document.title,
                                    window.www_dir + '?' + Object.toQueryString(pushUrl));
                            }
                        }
        }
    );
}

/**
 * New HTML5 History Api implementation
 * @todo Move and implement a specific javascript wrapper class
 */
(function ()
{
    "use strict";

    var ajaxOptions = {method: 'get', evalScripts: true};
    var options = ajaxOptions;

    /**
     * Popstate event handler
     *
     * Fired when user clicks back/forward buttons
     *
     * @param event
     */
    window.onpopstate = function (event)
    {
        if (event.state && event.state.url)
        {
            new Ajax.Updater(event.state.content,
                event.state.url + (event.state.url.match(/^.*[\?].*$/) ? '&' : '?') + 'ajax=1', ajaxOptions);
            breadcrumb(event.state.url, true);
        }
    };

    /**
     * Globalization of history.pushState.
     *
     * @param data
     * @param title
     * @param url
     */
    window.pushState = function (data, title, url)
    {
        if (window.history.pushState)
        {
            /**
             * Replace possible ajax=1 occurrences
             * @type {string}
             */
            url = url.replace('ajax=1', '');

            /**
             * Push current state to browser history
             */
            window.history.pushState(data, title, url);
        }
    };
})();

/* Global parameter array */
var g_params = [];

/**
 * Return key from global parameter array.
 * @param  key
 */
function glob(key)
{
    return g_params[key];
}

/**
 * Add param with val to the global parameter array.
 * @param  param
 * @param  val
 */
function globalize(param, val)
{
	if (val)
	{
		g_params[param] = val;
	}
}

/**
 * Changes p_key parameter to p_value from current form action.
 * @param  p_key
 * @param  p_value
 */
function change_action_parameter(p_key, p_value)
{
    "use strict";

    var l_new_action = '',
        key          = '',
        query        = $('isys_form').action.parseQuery();

    if ($('isys_form').action.search(p_key) != -1)
    {
        Object.values(query).each(function (val, i)
        {
            if (val)
            {
                key = Object.keys(query)[i];

                if (key == p_key)
                {
                    val = p_value;
                }
                l_new_action += key + '=' + val + '&';
            }
        });

        if (l_new_action != '')
        {
            $('isys_form').action = window.www_dir + '?' + l_new_action;
        }
    }
    else if ($('isys_form').action.search('\\?') !== -1)
    {
        $('isys_form').action += '&' + p_key + '=' + p_value;
    }
}

/**
 * Submits the current isys_form as a post and loads the results into el.
 * @param  el
 */
function save_via_ajax(el)
{
    el = $(el);

    if (el)
    {
        var btn = $('navbar_item_C__NAVMODE__SAVE');

        if (btn)
        {
            if (btn.hasClassName('navbar_item_inactive'))
            {
                return;
            }

            btn.addClassName('navbar_item_inactive');
        }

        $('contentArea').setOpacity(0.4);
        change_action_parameter('call', 'category_update');

        new Ajax.Updater(
            el.id,
            $('isys_form').action + ($('isys_form').action.match(/^.*[\?].*$/) ? '&ajax=1' : '?ajax=1'),
            {
                method:      'post',
                evalScripts: true,
                onComplete:  function (response)
                             {
                                 $('contentArea').setOpacity(1);
                                 $('navMode').value = '';
                                 reset_use_template_hidden_field();

                                 /* Trigger custom event form:saved */
                                 $('isys_form').fire('form:saved', {'response': response});

                                 if (btn)
                                 {
                                     btn.removeClassName('navbar_item_inactive');
                                 }
                             },
                parameters:  $('isys_form').serialize(true)
            }
        );
    }
}

/**
 * Do a form submit.
 * @param  p_url            form action
 * @param  p_method         'post' or 'get'
 * @param  p_content        Ajax result id, 'main_content' if null
 * @param  p_no_breadcrumb  Don't refresh breadcrumb
 * @param  p_data
 * @param  successCallback
 */
function form_submit(p_url, p_method, p_content, p_no_breadcrumb, successCallback, p_data)
{

    var addition,
        l_params = (!p_method || p_method == 'post') ? $('isys_form').serialize(true) : {};

    document.isys_form.target = '';

    /* Merge given additional data with isys_form */
    if (p_data)
    {
        for (var attrname in p_data)
        {
			if (p_data.hasOwnProperty(attrname))
			{
				l_params[attrname] = p_data[attrname];
			}
        }
    }

    if (!Object.isUndefined(p_url) && p_url)
    {
        $('isys_form').action = p_url;
    }
    else
    {
        change_action_parameter('call', 'category');
    }

    addition = (($('isys_form').action.search('\\?') != -1) ? '&ajax=1' : '?ajax=1');

    new Ajax.Updater((!Object.isUndefined(p_content) && p_content) ? p_content : 'main_content',
        $('isys_form').action + addition,
        {
            method:      p_method ? p_method : 'post',
            evalScripts: true,
            onComplete:  function (response)
             {
                 if (!p_no_breadcrumb)
                 {
                     breadcrumb($('isys_form').action);
                 }

                 if (successCallback)
                 {
                     setTimeout(function ()
                     {
                         eval(successCallback);
                     }, 500);
                 }

                 // Delay the firing of the event, because sometimes the UI is not completely parsed to attach to this event
                 delay(function()
                 {
                     Event.fire(document, 'form:submitted', {
                         'url':      p_url,
                         'method':   p_method,
                         'response': response,
                         'data':     p_data
                     });
                 }, 150);
             },
            parameters:  l_params
        });
}

/**
 * Loads the content/bottom/content for specified objectgroup.
 * @param  p_group_id
 * @param  p_viewMode
 * @param  successCallback
 */
function get_content_by_group(p_group_id, p_viewMode, successCallback)
{
    var l_link = '?call=object_list&' +
                 C__CMDB__GET__OBJECTGROUP +
                 '=' +
                 p_group_id +
                 '&' +
                 C__CMDB__GET__VIEWMODE +
                 '=' +
                 p_viewMode +
                 '&ajax=1';

    // remove object list otherwise the ids of the object list will be used which is wrong
    if (window.object_list)
    {
        delete window.object_list;
    }

    new Ajax.Updater('main_content', l_link, {
        method:      'get',
        evalScripts: true,
        onSuccess:   function ()
                     {
                         breadcrumb(l_link);

                         if (successCallback)
                         {
                             eval(successCallback);
                         }
                     }
    });
}

/**
 * Load tree by p_link.
 * @param  p_link
 */
function get_tree(p_link)
{
    close_mydoit();
    document.isys_form.target = '';

    globalize(C__CMDB__GET__OBJECT, $H(p_link.parseQuery()).get(C__CMDB__GET__OBJECT));

    new Ajax.Updater('tree_content',
        p_link + ((p_link.match(/^.*[\?].*$/)) ? '&ajax=1' : '?ajax=1'), {
            method:      'get',
            evalScripts: true,
            onComplete:  function (response)
                         {
                             $('tree_content').fire('cmdbObjectTree:retrieved', {"response": response});
                         }
        });
}

/**
 * Load object type tree be group id (software, infrastructure, other).
 * @param  p_group_id
 * @param  p_objecttype
 * @param  p_treeMode
 */
function get_tree_object_type(p_group_id, p_objecttype, p_treeMode)
{
    // Re-Initialize drops
    remove_drops();

    var l_link = window.www_dir + '?call=tree&' + C__CMDB__GET__OBJECTGROUP + '=' + p_group_id;

    if (p_treeMode)
    {
        l_link += '&' + C__CMDB__GET__TREEMODE + '=' + p_treeMode;
    }

    get_tree(l_link);

    if (p_objecttype)
    {
        tree_obj_type_click(p_objecttype);
    }
}

/**
 * Load category tree by object id and treemode.
 * @param  p_object_id
 * @param  p_treemode
 * @param  p_selected_catg
 * @param  p_selected_cats
 */
function get_tree_by_object(p_object_id, p_treemode, p_selected_catg, p_selected_cats)
{
    globalize(C__CMDB__GET__TREEMODE, p_treemode);
    globalize(C__CMDB__GET__OBJECT, p_object_id);

    var l_url = window.www_dir + '?call=tree_object&ajax=1&' +
                C__CMDB__GET__OBJECT + '=' + p_object_id + '&' +
                C__CMDB__GET__TREEMODE + '=' + p_treemode;

    if (p_selected_catg !== undefined && p_selected_catg !== null)
    {
        globalize('catgID', p_selected_catg);
        l_url += '&catgID=' + p_selected_catg;
    }
    else if (p_selected_cats !== undefined && p_selected_cats !== null)
    {
        globalize('catsID', p_selected_cats);
        l_url += '&catsID=' + p_selected_cats;
    }

    new Ajax.Updater('tree_content',
        l_url,
        {
            method:      'get',
            evalScripts: true
        }
    );

}

/**
 * Load dynamic category tree.
 * @param  p_object_id
 * @param  p_viewmode
 * @param  p_treemode
 */
function get_catd_by_object(p_object_id, p_viewmode, p_treemode)
{
    get_tree_by_object(p_object_id, p_treemode);
    get_content_by_object(p_object_id, p_viewmode);
}

/**
 * Load content by object.
 * @param  p_object_id        object id
 * @param  p_viewmode         viewmode to show (list or detail)
 * @param  p_category         id of the category to show
 * @param  p_category_type    type of the category (constant of global,specific)
 * @param  p_custom_category
 */
function get_content_by_object(p_object_id, p_viewmode, p_category, p_category_type, p_custom_category)
{
    // remove object list otherwise the ids of the object list will be used which is wrong
    if (window.object_list)
    {
        delete window.object_list;
    }

    change_action_parameter(C__CMDB__GET__VIEWMODE, p_viewmode);
    change_action_parameter(C__CMDB__GET__OBJECT, p_object_id);
    change_action_parameter('call', 'category');

    remove_action_parameter(['catgID', 'objTypeID', 'objGroupID', 'editMode', 'moduleID', 'viewMode', 'catsID', 'cateID', 'cat1ID', 'cat2ID', 'cat3ID', 'cat4ID', 'cat5ID', 'ifaceID', 'subcatID', 'navPageStart']);

    reset_use_template_hidden_field();

    if (p_category)
    {
        if (p_category_type)
        {
            change_action_parameter(p_category_type, p_category);
        }
        else
        {
            change_action_parameter('catgID', p_category);
        }
    }

    if (p_custom_category)
    {
        change_action_parameter('customID', p_custom_category);
    }

    if (g_params[C__CMDB__GET__TREEMODE] || g_params[C__CMDB__GET__TREEMODE] == 'undefined')
    {
        change_action_parameter(C__CMDB__GET__TREEMODE, g_params[C__CMDB__GET__TREEMODE]);
    }

    form_submit($('isys_form').action, 'get');
}

/**
 * Pagination (Changes the current list page to p_page).
 * @param  p_page
 * @param  p_url
 * @param  p_content
 * @param  p_method
 */
function change_page(p_page, p_url, p_content, p_method)
{
    if (Object.isUndefined(p_url))
    {
        p_url = false;
    }

    if (Object.isUndefined(p_method))
    {
        p_method = 'get';
    }

    change_action_parameter('navPageStart', p_page);

    form_submit(p_url, p_method, p_content);
}

/**
 * Shows an object list.
 * @param  p_objtype
 * @param  p_page
 * @param  p_status
 */
function tree_obj_type_click(p_objtype, p_page, p_status)
{
    var url = window.www_dir +
              '?' +
              C__CMDB__GET__VIEWMODE +
              '=1001&' +
              C__CMDB__GET__OBJECTTYPE +
              '=' +
              p_objtype +
              '&call=object_list';

    // Bugfix for #3518 - Der CMDB Explorer schreibt styles auf dieses Element, wodurch kein scrollen mehr möglich ist.
    $('contentArea').setStyle({'overflow': ''});
    $('navPageStart').setValue((p_page) ? p_page : 0);
    $('isys_form').action = url;

    reset_template_hidden_field();
    form_submit(url, 'get');
}

/**
 * Remove key parameter from current form action.
 * @param  key
 */
function remove_action_parameter(key)
{
    var i,
        $form      = $('isys_form'),
        formAction = $form.readAttribute('action').parseQuery();

    if (!Object.isArray(key))
    {
        key = [key];
    }

    for (i in key)
    {
        if (key.hasOwnProperty(i))
        {
            delete formAction[key[i]];
        }
    }

    $form.writeAttribute('action', window.www_dir + '?' + Object.toQueryString(formAction));
}

/**
 * Resets hidden input field template.
 */
function reset_template_hidden_field()
{
    "use strict";
    $N('template')[0].setValue('');
}

/**
 * Reset hidden input field indicator if default Template should be used or not.
 */
function reset_use_template_hidden_field()
{
    "use strict";
    $('useTemplate').setValue('');
}