/**
 * i-doit Browsers
 */

if (!Browser) var Browser = {};

// Browser Scripts.
Browser.objectList = Class.create(TableOrderer, {
	parameters:{
		form:'browser',
		minLen:2
	},

	initialize:function (element, params) {
		this.element = element;
		this.defaultSelection = new Hash({
			isys_obj__id:'__checkbox__',
			isys_obj__title:idoit.Translate.get('LC__CMDB__LOGBOOK__TITLE'),
			isys_obj_type__title:idoit.Translate.get('LC__CMDB__CATG__TYPE'),
			isys_obj__sysid:idoit.Translate.get('LC__CMDB__CATG__SYSID')
		});

		Object.extend(this.parameters, params || {
			cmdb_filter:''
		});

		if (!this.parameters.jsonClient) this.parameters.jsonClient = new idoit.JSONClient();

		this.options = Object.extend({
			data:[], // array of data
			url:false, // url to a JSON application containing the data
			allowMultiselect:false, // don't work yet
			unsortedColumn:[], // array of column you don't want to sort
			dateFormat:'d', // d|m ; d => dd/mm/yyyy; m => mm/dd/yyyy
			filter:'top', // show Filter Option. false | 'top' | 'bottom'
			pageCount:25, // Number of items by table
			paginate:'top', // show Paginate Option. false | 'top' | 'bottom'
			search:false, // show Gloabal Search . false | 'top' | 'bottom',
			objectSelectionCallback:'browserPreselection.add',
			firstSelection:false,
			secondSelection:false,
			secondSelectionExists:false,
			classPrefix:'prototools',
			instanceName:'objectList',
            useAuth: false
		}, this.parameters.listOptions || {});

		// saves tool state data for pagination, filtering and searching.
		this.tools = {
			page:1,
			pages:1,
			filterCol:'',
			filterData:'',
			searchData:''
		};

		// separates text messages out
		this.msgs = {
			loading:idoit.Translate.get('LC__CMDB__OBJECT_BROWSER__SCRIPT_JS__LOADING'),
			emptyResults:idoit.Translate.get('LC__CMDB__OBJECT_BROWSER__SCRIPT_JS__EMPTY_RESULTS'),
			errorURL:idoit.Translate.get('LC__CMDB__OBJECT_BROWSER__SCRIPT_JS__ERROR_URL'),
			errorData:idoit.Translate.get('LC__CMDB__OBJECT_BROWSER__SCRIPT_JS__ERROR_DATA'),
			searchLabel:idoit.Translate.get('LC__CMDB__OBJECT_BROWSER__SCRIPT_JS__SEARCH_LABEL'),
			filterLabel:idoit.Translate.get('LC__CMDB__OBJECT_BROWSER__SCRIPT_JS__FILTER_LABEL'),
			paginationOf:idoit.Translate.get('LC__CMDB__OBJECT_BROWSER__SCRIPT_JS__PAGINATEN_OF'),
			paginationPages:idoit.Translate.get('LC__CMDB__OBJECT_BROWSER__SCRIPT_JS__PAGINATEN_PAGES'),
			paginationFirst:'&laquo;',
			paginationPrev:'&lsaquo;',
			paginationNext:'&rsaquo;',
			paginationLast:'&raquo;',
			minLen:'Bitte mehr als {0} Zeichen eingeben.'
		};

		this.cache = [];
		this.isCached = false;
		this.container = $(this.element);
		this.orderField = false;
		this.order = 'asc';

		this.thClickbfx = this.thClick.bindAsEventListener(this);
		this.thOverbfx = this.thOver.bindAsEventListener(this);
		this.thOutbfx = this.thOut.bindAsEventListener(this);

        if (this.options.useAuth) {
            this.parameters.jsonClient.useAuth(this.options.useAuth);
        }
	},

	searchByType:function (objectTypeID) {
		// We need this to reset the second list, when we change the object-type drop down.
		if ($('portList')) {
			$('portList').update('');
		}

		if (objectTypeID > 0) this.parameters.jsonClient.getObjectsByType(objectTypeID, this.initList.bind(this), {
			select:this.defaultSelection,
			parameters:this.parameters
		});
	},

	searchByGroup:function (objID) {
		// We need this to reset the second list, when we change the object-type drop down.
		if ($('portList')) {
			$('portList').update('');
		}

		if (objID > 0) this.parameters.jsonClient.getObjectsByGroup(objID, this.initList.bind(this), {
			select:new Hash({
				connected_id:'__checkbox__',
				connected_title:idoit.Translate.get('LC__CMDB__LOGBOOK__TITLE'),
				connected_type:idoit.Translate.get('LC__CMDB__CATG__TYPE'),
				connected_sysid:idoit.Translate.get('LC__CMDB__CATG__SYSID')
			}),
			parameters:this.parameters
		});
	},

	searchByPersonGroup:function (objID) {
		// We need this to reset the second list, when we change the object-type drop down.
		if ($('portList')) {
			$('portList').update('');
		}

		if (objID > 0) {
			this.parameters.jsonClient.getObjectsByPersonGroup(
				objID,
				this.initList.bind(this),
				{
					//select:this.defaultSelection,
                    select:new Hash({
                        person_id:'__checkbox__',
                        person_title:idoit.Translate.get('LC__CMDB__LOGBOOK__TITLE'),
                        person_type:idoit.Translate.get('LC__CMDB__CATG__TYPE'),
                        person_sysid:idoit.Translate.get('LC__CMDB__CATG__SYSID')
                    }),
					parameters:{cmdb_filter:this.parameters.cmdb_filter}
				});
		}
	},

	searchByRelationType:function (objID) {
		// We need this to reset the second list, when we change the object-type drop down.
		if ($('portList')) {
			$('portList').update('');
		}

		if (objID > 0) {
			this.parameters.jsonClient.getObjectsByRelationType(
				objID,
				this.initList.bind(this),
				{
					select:this.defaultSelection,
					parameters:{cmdb_filter:this.parameters.cmdb_filter}
				});
		}
	},

	searchByTimeCondition:function (p_condition, p_from, p_to, p_params) {
		// We need this to reset the second list, when we change the object-type drop down.
		if ($('portList')) {
			$('portList').update('');
		}

		if (p_condition && p_condition != -1) {
			this.parameters.jsonClient.getObjectsByTimeCondition({
					condition:p_condition,
					from:p_from,
					to:p_to
				},
				this.initList.bind(this),
				{
					parameters:p_params,
					select:this.defaultSelection
				});
		}
	},

	searchByCategoryFilter:function (p_request, p_callFunction) {
		// We need this to reset the second list, when we change the object-type drop down.
		if ($('portList')) {
			$('portList').update('');
		}

		this.parameters.jsonClient.getObjectsByCustomBrowserRequest(
			new Hash({
				request:p_request,
				callFunction:p_callFunction
			}),
			this.initList.bind(this),
			{
				select:this.defaultSelection
			});
	},

	search:function (p_element, p_typeFilter, p_groupFilter) {
		if (p_element && p_element.value.length > this.parameters.minLen) {
			this.parameters.jsonClient.getObjectsByFilter(encodeURI(p_element.value), this.initList.bind(this), {
				select:this.defaultSelection,
				parameters:{
					typeFilter:p_typeFilter,
					groupFilter:p_groupFilter
				}
			});
		} else {
			this.showError(this.msgs.minLen.replace('{0}', this.parameters.minLen));
		}
	},

	showError:function (p_message) {
		$(this.element).update('<p class="emptyPageMessage"><img src="images/outlet.png" class="" /> ' + p_message + '</p>');
	},

	initList:function (r) {
		if (r.responseJSON) {
			this.data = r.responseJSON;

			if (this.data && this.data.length > 0) {
				this.perform();

				if (this.options.colgroup) this.table.insert({top:this.options.colgroup});
			} else {
				this.showError(this.msgs.emptyResults);
			}

			$$('.browserContent table tr td span.quickinfo').each(function (element) {
				var obj_id = element.readAttribute('data-obj-id');
				new Tip('obj-browser-quickinfo-' + obj_id, '', {
					"ajax": {"url": '?ajax=1&call=quick_info&' + idoit.Translate.get('C__CMDB__GET__OBJECT') + '=' + obj_id},
					"delay": 0,
//					"stem": 'topLeft',
					"style": 'default',
					"effect": 'appear',
					"className": 'objectinfo'
				});
			});
		}
	}
});

Browser.preselection = Class.create({
	options:{
		preselection:[],
		ajaxURL:'',
		afterFinish:Prototype.emptyFunction,
		instanceName:'preselection'
	},

	initialize:function (element, options) {
		this.element = $(element) || false;
		this.secondElement = ('secondElement' in options) ? $(options.secondElement) : false;
		this.secondList = ('secondList' in options) ? options.secondList : false;

		Object.extend(this.options, options || { });
	},

	log:function (string) {
		if (this.options.logElement) $(this.options.logElement).insert({top:new Date().toLocaleString() + ': ' + string + '\n'});
		if (this.options.latestLogElement) $(this.options.latestLogElement).update(string);
	},

	alreadyAdded:function (name) {
		this.log(('<span class="text-red">' + idoit.Translate.get('LC__CMDB__OBJECT_BROWSER__EXISTS_IN_SELECTION') + '</span>').replace('{0}', name));
		this.morphSelectionCount('#cc0000');
	},

	getData:function () {
		var tmpArray = [];
		if (this.options.preselection) {
			for (var i = 0, len = this.options.preselection.length; i < len; ++i) tmpArray.push(this.options.preselection[i][0]);
			return Object.toJSON(tmpArray);
		} else return '[]';
	},

	/*
	 * @todo find a much faster and more elegant way of checking the existence!!
	 */
	exists:function (objectId) {
		if (this.element) return (this.element.innerHTML.search('objectid="#' + objectId + '"') != -1);
		else return false;
	},

	remove_all:function () {

        /* Remove the whole selection from object view */
        for (var i = 0, len = this.options.preselection.length; i < len; ++i) {
            this.remove_selection(this.options.preselection[i][0]);
		}

		this.options.preselection = [];
		this.element.update(this.compute());
        this.updateSelectionCount('#cc0000');
	},
    remove_selection: function(objID) {
        /* Do we have to unselected the row in the selection list */

        /* Single */
        if ($$('input[onclick*="browserPreselection.add([\''+objID+'\',"]').length) {
            $$('input[onclick*="browserPreselection.add([\''+objID+'\',"]')[0].checked=false;

        /* Multi */
        } else if($$('a[onclick*="browserPreselection.remove('+objID+')"]').length) {
            var l_elem = $$('a[onclick*="browserPreselection.remove('+objID+')"]')[0];
            var l_onclick = l_elem.getAttribute("onclick");

            /* Replace call of remove() to prevent endless loop */
            l_onclick = l_onclick.substr(l_onclick.indexOf(";"));
            $$('a[onclick*="browserPreselection.remove('+objID+')"]')[0].setAttribute("onclick", l_onclick);

            /* Click the Button */
            l_elem.simulate("click");

        /* Second-List */
        } else if ($$('input[onclick*="browserPreselection.secondSelectionCall([\''+objID+'\',"]').length) {
            /* Deselect radios */
            if ($$('div#objectview :last-child input:checked').length) {
                $$('div#objectview :last-child input:checked').each(function(v, k) {
                    v.checked = false;
                });
            }
        }
    },
	remove:function (objID, skipRebuildingTable) {
		var tmpdata = [], index = -1;

        for (var i = 0, len = this.options.preselection.length; i < len; ++i) {
			if (objID == this.options.preselection[i][0]) {
				index = i;
				break;
			}
		}

		if (index === 0 && this.options.preselection.length == 1) this.options.preselection = [];
		else if (index >= 0) {
			tmpdata = this.options.preselection[index];

			if (this.options.preselection.splice(index, 1) && tmpdata[1] && tmpdata[2])
				this.log(('<span class="text-red">' + idoit.Translate.get('LC__CMDB__OBJECT_BROWSER__SCRIPT__0_FROM_TYPE_1_HAS_BEEN_REMOVED') + '</span>').replace('{0}', tmpdata[1]).replace('{1}', tmpdata[2]));
		}

		if (!skipRebuildingTable) this.element.update(this.compute());
		this.updateSelectionCount('#cc0000');

        /* Unselect row in object view */
        this.remove_selection(objID);

		return true;
	},

	addObject:function (objID, objTypeID, objTitle, objTypeTitle) {

		this.add([ objID, objTitle, objTypeTitle, '-' ]);

	},

	/**
	 * A new method for the second-selection call.
	 */
	secondSelectionCall:function (object) {
		if (typeof object[0] == 'undefined') {
			return false;
		}

		if (typeof object[1] != 'undefined') {
			window.secondSelection.objName = object[1];
		}

		if (!this.options.multiselection) {
			this.options.preselection = [object]
		}

		idoitJSON.call(
			this.options.ajaxURL + '&' + C__CMDB__GET__OBJECT + '=' + object[0],
			function (json) {
				if (this.secondList) {
					this.secondElement.update('');
					this.secondList.initList(json);
				}
			}.bind(this),
			{
				select:new Hash({
					isys_obj__id:'__checkbox__',
					isys_obj__port:idoit.Translate.get('LC__CMDB__CATG__CONNECTORS'),
					isys_obj__connected_with:idoit.Translate.get('LC__CMDB__CATG__UI_ASSIGNED_UI'),
					isys_obj__category:idoit.Translate.get('LC__CMDB__CATG__CATEGORY'),
					isys_obj__in_out:idoit.Translate.get('LC__CMDB__CATS__PRT_TYPE')
				})
			});
	},
	/**
	 * object dataformat is:
	 *  [ id, objectname, objecttype, sysid ];
	 */
	add:function (object, fromButton) {

		// This needs to be done for IE browsers (they have problems without "hasOwnProperty()").
		if (typeof object == 'object') {
			var result = [];

			for (var i in object) {
				if (object.hasOwnProperty(i)) {
					result.push(object[i]);
				}
			}
			object = result;
		}

		// Really really really go sure it's an integer.
		object[0] = parseInt(object[0]);

		if (object[0] > 0) {
			if (!this.exists(object[0])) {
				// Add object on multiselection, set object on single selection.
				if (!this.options.secondElement || fromButton) {
					if (!this.options.multiselection || this.options.multiselection == 'false') {
						this.options.preselection = [object]
					} else {
						this.options.preselection.push(object);
					}

					this.updateSelectionCount();
				}

				// Initialize table if element exists and this is the first object.
				if (this.element) {
					if (this.options.preselection.length == 1) {
						this.element.update(this.compute());
					} else {
						if (this.element.down()) {
							// @todo error, here!
							this.element.down().insert(this.addRow(object));
						}
					}
				}

				if ((this.options.multiselection && !this.secondElement) || fromButton)
					this.log(('<span class="text-green">' + idoit.Translate.get('LC__CMDB__OBJECT_BROWSER__SCRIPT__0_FROM_TYPE_1_HAS_BEEN_ADDED') + '</span>').replace('{0}', object[1]).replace('{1}', object[2]));
				else
					this.log(('<span class="text-green">' + idoit.Translate.get('LC__CMDB__OBJECT_BROWSER__SCRIPT__0_FROM_TYPE_1_SELECTED') + '</span>').replace('{0}', object[1]).replace('{1}', object[2]));

				return true;
			} else {
				this.alreadyAdded(object[1]);
			}
		}

		return false;
	},

	/**
	 * object dataformat is:
	 *  [ id, objectname, objecttype, sysid ];
	 */
    addRow: function (object, i) {
        if (!i) i = this.options.preselection.length;

        var onclick = "browserPreselection.remove('" + object[0] + "'); var remover = $('selection-obj-" + object[0] + "'); if(remover) remover.remove();";

        return new Element('tr', {className: ((i++ % 2 == 0) ? 'line0' : 'line1'), 'objectid': '#' + object[0], id: 'selection-obj-' + object[0]})
            .update(new Element('td').update(new Element('button', {className:'btn btn-small btn-block', type:'button', onclick:onclick})
                .update(new Element('img', {src:window.dir_images + 'icons/silk/delete.png', style:'margin-right:3px'}))
                .insert(new Element('span').update(idoit.Translate.get('LC__CMDB__OBJECT_BROWSER__SCRIPT__REMOVE')))))
            .insert(new Element('td').update(object[1]))
            .insert(new Element('td').update(object[2]));
    },

	morphSelectionCount:function (toColor) {
		if ($(this.options.objectCountElement).up()) {
			new Effect.Morph($(this.options.objectCountElement).up(), {
				style:'color:' + toColor + ';',
				duration:0.3,
				afterFinish:function () {
					new Effect.Morph($(this.options.objectCountElement).up(), {
						style:'color:#666666;',
						duration:1.8
					});
				}.bind(this)
			});
		}
	},

	updateSelectionCount:function (morphTo) {
		if (!morphTo) morphTo = '#00cc00';

		if ('objectCountElement' in this.options && $(this.options.objectCountElement)) {
			$(this.options.objectCountElement).update(this.options.preselection.length);
			this.morphSelectionCount(morphTo);
		}
	},

	count:function () {
		return this.options.preselection.length;
	},

	compute:function () {

		var content = '', i = 0;

		if (this.options.preselection && this.options.preselection.length > 0) {
			var remove_button = '';

            if (this.options.multiselection)
            {
                remove_button = '<button type="button" class="btn btn-small btn-block" id="remove_all" onclick="browserPreselection.remove_all();">' +
                                '<img src="' + window.dir_images + 'icons/silk/delete.png" style="margin-right:3px" />' +
                                '<span>' + idoit.Translate.get('LC__UNIVERSAL__ALL') + '</span>' +
                                '</button>';
            }

			content = '<table class="prototools-table">' +
				'<colgroup>' +
				'<col width="25" />' +
				'</colgroup>' +
				'<thead>' +
				'<tr>' +
				'<th>' + remove_button + '</th>' +
				'<th>' + idoit.Translate.get('LC__UNIVERSAL__OBJECT_TITLE') + '</th>' +
				'<th>' + idoit.Translate.get('LC__UNIVERSAL__OBJECT_TYPE') + '</th>' +
				'</tr>' +
				'</thead>' +
				'<tbody>';

			this.options.preselection.each(function (v, k) {
				content += this.addRow(v, i++).outerHTML;
			}.bind(this));

			content += '</tbody></table>';
		} else {
			this.log(idoit.Translate.get('LC__CMDB__OBJECT_BROWSER__SCRIPT__NO_PRESELECTION'));
		}

		//if (this.options['afterFinish']) this.options['afterFinish'](this);
		if (this.options.afterFinish) this.options.afterFinish(this);

		return content || '<p class="box-red p10 m10">' + idoit.Translate.get('LC__CMDB__OBJECT_BROWSER__SCRIPT__NO_PRESELECTION') + '</p>';
	}
});
