var BrowserSelectionSorter = Class.create({
	/**
	 * Constructor method, initializes everything necessary.
	 * @param  element
	 * @param  options
	 */
	initialize: function (element, options) {
		if ($(element + '_SELECTION_SORTER')) {
			this.destruct(element);

			return null;
		}

		this.element = $(element);
		this.hidden_element = this.element.next('input[type="hidden"]');

		if (options.hasOwnProperty('hidden')) {
			this.hidden_element = $(options.hidden);
		}

		this.options = Object.extend({
			values: [],
			ajax_url: '?ajax=1&call=json&action=load_object_data'
		}, options || {});

		// Saving some positioning values...
		this.positions = this.element.positionedOffset();
		this.positions.height = this.element.getHeight();
		this.positions.width = this.element.getWidth();

		this.element.insert({after:
			new Element('div', {id:element + '_SELECTION_SORTER', className:'browser-selection-sorter border'}).setStyle({
			position:'absolute',
			left:this.positions.left + 'px',
			top:(this.positions.top + this.positions.height - 1) + 'px',
			width:(this.positions.width - 2) + 'px',
			display:'none'})
		});

		this.sorting_div = this.element.next('#' + element + '_SELECTION_SORTER');

		// Now we'll load the selected data and display it.
		this.load_selection();
	},

	/**
	 * Method for loading and displaying all the selected objects.
	 * @return  BrowserSelectionSorter
	 */
	load_selection: function () {
		new Ajax.Request(this.options.ajax_url, {
			parameters:{
				objects:this.options.values
			},
			method:'post',
			onSuccess:function (transport) {
				var json = transport.responseJSON,
					objects = [],
					object;

				if (json.success) {
					for (i in json.data) {
						if (json.data.hasOwnProperty(i)) {
							object = json.data[i];

							objects.push(new Element('li', {'data-obj-id':object.id})
								.update(new Element('span', {className:'handle mr5'}).update('&nbsp;&nbsp;&nbsp;'))
								.insert(new Element('span').update(object.title + ' (' + object.type_title + ')')).outerHTML);
						}
					}

					this.sorting_div
						.update(new Element('ul', {id:this.element.id + '_SELECTION_SORTABLE'}).update(objects.join('')))
						.insert(new Element('div', {className:'controls'})
							.update(new Element('button', {className:'btn btn-small m5 accept', type:'button'}).update(new Element('img', {src:window.dir_images + 'icons/silk/tick.png'})).insert(' ').insert(new Element('span').update(idoit.Translate.get('LC_UNIVERSAL__ACCEPT'))))
							.insert(new Element('button', {className:'btn btn-small m5 abort', type:'button'}).update(new Element('img', {src:window.dir_images + 'icons/silk/cross.png'})).insert(' ').insert(new Element('span').update(idoit.Translate.get('LC_UNIVERSAL__ABORT')))));
				} else {
					this.sorting_div.update(new Element('div', {className:'p5 m5 box-red'}).update(json.message));
				}

				Effect.BlindDown(this.sorting_div, {duration:0.4});

				// This has to be called only once.
				this.set_observer();
			}.bind(this)
		});
	},

	/**
	 * Method for resetting all observers.
	 * @return  BrowserSelectionSorter
	 */
	set_observer: function () {
		Sortable.destroy(this.element.id + '_SELECTION_SORTABLE');

		Sortable.create(this.element.id + '_SELECTION_SORTABLE', {
			handle:'handle'
		});

		this.sorting_div.down('button.accept').on('click', function () {
			var view = [],
				hidden = [];

			this.sorting_div.select('li').each(function (el) {
				view.push(el.down('span', 1).innerHTML);
				hidden.push(parseInt(el.readAttribute('data-obj-id')));
			}.bind(this));

			this.element.setValue(view.join(', '));
			this.hidden_element.setValue(Object.toJSON(hidden));

			this.destruct(this.element.id);
		}.bind(this));

		this.sorting_div.down('button.abort').on('click', function () {
			this.destruct(this.element.id);
		}.bind(this));

		return this;
	},

	/**
	 * Destructor.
	 * @param   element
	 * @return  BrowserSelectionSorter
	 */
	destruct: function (element) {
		Effect.BlindUp(element + '_SELECTION_SORTER', {duration:0.4, afterFinish:function () {
			$(element + '_SELECTION_SORTER').remove();
		}});

		return this;
	}
});