//Event.observe(window, 'load', function() {
    /**
     * @author Ryan Johnson <http://syntacticx.com/>
     * @copyright 2008 PersonalGrid Corporation <http://personalgrid.com/>
     * @package LivePipe UI
     * @license MIT
     * @url http://livepipe.net/core
     * @require prototype.js
     */

    if (typeof(Control) == 'undefined')
        Control = {};

    var $proc = function(proc) {
        return typeof(proc) == 'function' ? proc : function() {
            return proc
        };
    };

    var $value = function(value) {
        return typeof(value) == 'function' ? value() : value;
    };

    Object.Event = {
        extend: function(object) {
            object._objectEventSetup = function(event_name) {
                this._observers = this._observers || {};
                this._observers[event_name] = this._observers[event_name] || [];
            };
            object.observe = function(event_name, observer) {
                if (typeof(event_name) == 'string' && typeof(observer) != 'undefined') {
                    this._objectEventSetup(event_name);
                    if (!this._observers[event_name].include(observer))
                        this._observers[event_name].push(observer);
                } else
                    for (var e in event_name)
                        this.observe(e, event_name[e]);
            };
            object.stopObserving = function(event_name, observer) {
                this._objectEventSetup(event_name);
                if (event_name && observer)
                    this._observers[event_name] = this._observers[event_name].without(observer);
                else if (event_name)
                    this._observers[event_name] = [];
                else
                    this._observers = {};
            };
            object.observeOnce = function(event_name, outer_observer) {
                var inner_observer = function() {
                    outer_observer.apply(this, arguments);
                    this.stopObserving(event_name, inner_observer);
                }.bind(this);
                this._objectEventSetup(event_name);
                this._observers[event_name].push(inner_observer);
            };
            object.notify = function(event_name) {
                this._objectEventSetup(event_name);
                var collected_return_values = [];
                var args = $A(arguments).slice(1);
                try {
                    for (var i = 0; i < this._observers[event_name].length; ++i)
                        collected_return_values.push(this._observers[event_name][i].apply(this, args) || null);
                } catch (e) {
                    if (e == $break)
                        return false;
                    else
                        throw e;
                }
                return collected_return_values;
            };
            if (object.prototype) {
                object.prototype._objectEventSetup = object._objectEventSetup;
                object.prototype.observe = object.observe;
                object.prototype.stopObserving = object.stopObserving;
                object.prototype.observeOnce = object.observeOnce;
                object.prototype.notify = function(event_name) {
                    if (object.notify) {
                        var args = $A(arguments).slice(1);
                        args.unshift(this);
                        args.unshift(event_name);
                        object.notify.apply(object, args);
                    }
                    this._objectEventSetup(event_name);
                    var args = $A(arguments).slice(1);
                    var collected_return_values = [];
                    try {
                        if (this.options && this.options[event_name] && typeof(this.options[event_name]) == 'function')
                            collected_return_values.push(this.options[event_name].apply(this, args) || null);
                        var callbacks_copy = this._observers[event_name]; // since original array will be modified after observeOnce calls
                        for (var i = 0; i < callbacks_copy.length; ++i)
                            collected_return_values.push(callbacks_copy[i].apply(this, args) || null);
                    } catch (e) {
                        if (e == $break)
                            return false;
                        else
                            throw e;
                    }
                    return collected_return_values;
                };
            }
        }
    };

    /* Begin Core Extensions */

//Element.observeOnce
    Element.addMethods({
        observeOnce: function(element, event_name, outer_callback) {
            var inner_callback = function() {
                outer_callback.apply(this, arguments);
                Element.stopObserving(element, event_name, inner_callback);
            };
            Element.observe(element, event_name, inner_callback);
        }
    });

//mouse:wheel
//    (function() {
//        function wheel(event) {
//            var delta, element, custom_event;
//            // normalize the delta
//            if (event.wheelDelta) { // IE & Opera
//                delta = event.wheelDelta / 120;
//            } else if (event.detail) { // W3C
//                delta = -event.detail / 3;
//            }
//            if (!delta) {
//                return;
//            }
//            element = Event.extend(event).target;
//            element = Element.extend(element.nodeType === Node.TEXT_NODE ? element.parentNode : element);
//            custom_event = element.fire('mouse:wheel', {delta: delta});
//            if (custom_event.stopped) {
//                Event.stop(event);
//                return false;
//            }
//        }
//        document.observe('mousewheel', wheel);
//        document.observe('DOMMouseScroll', wheel);
//    })();

    /* End Core Extensions */

//from PrototypeUI
    var IframeShim = Class.create({
        initialize: function() {
            this.element = new Element('iframe', {
                style: 'position:absolute;filter:progid:DXImageTransform.Microsoft.Alpha(opacity=0);display:none',
                src: 'javascript:void(0);',
                frameborder: 0
            });
            $(document.body).insert(this.element);
        },
        hide: function() {
            this.element.hide();
            return this;
        },
        show: function() {
            this.element.show();
            return this;
        },
        positionUnder: function(element) {
            var element = $(element);
            var offset = element.cumulativeOffset();
            var dimensions = element.getDimensions();
            this.element.setStyle({
                left: offset[0] + 'px',
                top: offset[1] + 'px',
                width: dimensions.width + 'px',
                height: dimensions.height + 'px',
                zIndex: element.getStyle('zIndex') - 1
            }).show();
            return this;
        },
        setBounds: function(bounds) {
            for (prop in bounds)
                bounds[prop] += 'px';
            this.element.setStyle(bounds);
            return this;
        },
        destroy: function() {
            if (this.element)
                this.element.remove();
            return this;
        }
    });

    /**
     * @author Ryan Johnson <http://syntacticx.com/>
     * @copyright 2008 PersonalGrid Corporation <http://personalgrid.com/>
     * @package LivePipe UI
     * @license MIT
     * @url http://livepipe.net/control/rating
     * @require prototype.js, livepipe.js
     */

    /*global Prototype, Class, Option, $, $A, Control, $break,  */

    if (typeof(Prototype) == "undefined") {
        throw "Control.SelectMultiple requires Prototype to be loaded.";
    }
    if (typeof(Object.Event) == "undefined") {
        throw "Control.SelectMultiple requires Object.Event to be loaded.";
    }

    Control.SelectMultiple = Class.create({
        select: false,
        container: false,
        numberOfCheckedBoxes: 0,
        checkboxes: [],
        hasExtraOption: false,
        initialize: function(select, container, options) {
            this.options = {
                checkboxSelector: 'input[type=checkbox]',
                nameSelector: 'span.name',
                labelSeparator: ', ',
                valueSeparator: ',',
                afterChange: Prototype.emptyFunction,
                overflowString: function(str) {
                    return str.truncate();
                },
                overflowLength: 30
            };
            Object.extend(this.options, options || {});
            this.select = $(select);
            this.container = $(container);
            this.checkboxes = (typeof(this.options.checkboxSelector) == 'function') ?
                    this.options.checkboxSelector.bind(this)() :
                    this.container.getElementsBySelector(this.options.checkboxSelector);
            var value_was_set = false;
            if (this.options.value) {
                value_was_set = true;
                this.setValue(this.options.value);
                delete this.options.value;
            }
            this.hasExtraOption = false;
            this.checkboxes.each(function(checkbox) {
                checkbox.observe('click', this.checkboxOnClick.bind(this, checkbox));
            }.bind(this));
            this.select.observe('change', this.selectOnChange.bind(this));
            this.countAndCheckCheckBoxes();
            if (!value_was_set) {
                this.scanCheckBoxes();
            }
            this.notify('afterChange', this.select.options[this.select.options.selectedIndex].value);
        },
        countAndCheckCheckBoxes: function() {
            this.numberOfCheckedBoxes = this.checkboxes.inject(0, function(number, checkbox) {
                checkbox.checked = (this.select.options[this.select.options.selectedIndex].value == checkbox.value);
                var value_string = this.select.options[this.select.options.selectedIndex].value;
                var value_collection = $A(value_string.split ? value_string.split(this.options.valueSeparator) : value_string);
                var should_check = value_collection.any(function(value) {
                    if (!should_check && checkbox.value == value) {
                        return true;
                    }
                }.bind(this));
                checkbox.checked = should_check;
                if (checkbox.checked) {
                    ++number;
                }
                return number;
            }.bind(this));
        },
        setValue: function(value_string) {
            this.numberOfCheckedBoxes = 0;
            var value_collection = $A(value_string.split ? value_string.split(this.options.valueSeparator) : value_string);
            this.checkboxes.each(function(checkbox) {
                checkbox.checked = false;
                value_collection.each(function(value) {
                    if (checkbox.value == value) {
                        ++this.numberOfCheckedBoxes;
                        checkbox.checked = true;
                    }
                }.bind(this));
            }.bind(this));
            this.scanCheckBoxes();
        },
        selectOnChange: function() {
            this.removeExtraOption();
            this.countAndCheckCheckBoxes();
            this.notify('afterChange', this.select.options[this.select.options.selectedIndex].value);
        },
        checkboxOnClick: function(checkbox) {
            this.numberOfCheckedBoxes = this.checkboxes.findAll(function(c) {
                return c.checked;
            }).length;
            this.scanCheckBoxes();
            this.notify('afterChange', this.numberOfCheckedBoxes === 0 ? "" :
                    this.select.options[this.select.options.selectedIndex].value);
        },
        scanCheckBoxes: function() {
            switch (this.numberOfCheckedBoxes) {
                case 1:
                    this.checkboxes.each(function(checkbox) {
                        if (checkbox.checked) {
                            $A(this.select.options).each(function(option, i) {
                                if (option.value == checkbox.value) {
                                    this.select.options.selectedIndex = i;
                                    throw $break;
                                }
                            }.bind(this));
                            throw $break;
                        }
                    }.bind(this));
                    break;
                case 0:
                    this.removeExtraOption();
                    break;
                default:
                    this.addExtraOption();
                    break;
            }
        },
        getLabelForExtraOption: function() {
            var label = (typeof(this.options.nameSelector) == 'function' ?
                    this.options.nameSelector.bind(this)() :
                    this.container.getElementsBySelector(this.options.nameSelector).inject([], function(labels, name_element, i) {
                if (this.checkboxes[i].checked) {
                    labels.push(name_element.innerHTML);
                }
                return labels;
            }.bind(this))
                    ).join(this.options.labelSeparator);
            return (label.length >= this.options.overflowLength && this.options.overflowLength > 0) ?
                    (typeof(this.options.overflowString) == 'function' ? this.options.overflowString(label) : this.options.overflowString) :
                    label;
        },
        getValueForExtraOption: function() {
            return this.checkboxes.inject([], function(values, checkbox) {
                if (checkbox.checked) {
                    values.push(checkbox.value);
                }
                return values;
            }).join(this.options.valueSeparator);
        },
        addExtraOption: function() {
            this.removeExtraOption();
            this.hasExtraOption = true;
            this.select.options[this.select.options.length] = new Option(this.getLabelForExtraOption(), this.getValueForExtraOption());
            this.select.options.selectedIndex = this.select.options.length - 1;
        },
        removeExtraOption: function() {
            if (this.hasExtraOption) {
                this.select.remove(this.select.options.length - 1);
                this.hasExtraOption = false;
            }
        }
    });
    Object.Event.extend(Control.SelectMultiple);
//});
