/**
  * i-doit javascript components
  * @author dennis stuecken <dstuecken@i-doit.org>
  */
if (!idoit) var idoit = {};

idoit.Translate = new Hash();

/**
 * i-doit CallbackManager - please try to avoid this class and use the JS native event handling with custom events.
 *
 * @deprecated  Please try to use native JS events (Element.observe(), Element.fire()) for this.
 * @type        {{callbacks: {}, registerCallback: Function, triggerCallback: Function}}
 */
idoit.callbackManager = {
	callbacks: {},
	registerCallback: function (name, callback, context) {
        if (Object.isFunction(callback)) {
		    this.callbacks[name] = [callback, context || null];
        }

		return this;
	},
	triggerCallback: function (name) {
		if (this.callbacks.hasOwnProperty(name)) {
            try {
			    this.callbacks[name][0].apply(this.callbacks[name][1], Array.prototype.slice.call(arguments, 1));
            } catch (e) {
                idoit.Notify.error(e.message);
            }
		}

		return this;
	}
};

idoit.Suggest = Class.create({
    initialize: function (method, el, el_hidden, p_options) {

        if (document.getElementsByName(el)[0] || $(el))
        {
            //Position.includeScrollOffsets = true;

            var element = ($(el)) ? $(el) : document.getElementsByName(el)[0];
            var rnd        = Math.floor(Math.random() * 9999),
                el_choices = 'l_choices' + rnd,
                url        = '?ajax=1&call=suggest&method=' + method;

            element.up().insert(
                '<div id="' + el_choices + '" class="autocomplete"></div>' +
                '<span id="indicator' + rnd + '" style="display:none;" class="indicator"><img src="' + window.dir_images + 'ajax-loading.gif" alt="Working..." /></span>'
            );

            var options = {
                paramName:          'search',
                minChars:           2,
                indicator:          'indicator' + rnd,
                afterUpdateElement: function (text, li) {
                    if (el_hidden && document.getElementsByName(el_hidden)[0])
                    {
                        document.getElementsByName(el_hidden)[0].value = li.id;
                    }

                    if (typeof p_options.selectCallback == 'function')
                    {
                        p_options.selectCallback(text, li);
                    }
                    else if (p_options.selectCallback)
                    {
                        eval(p_options.selectCallback);
                    }
                }
            };
            Object.extend(options, p_options || {});

            /**
             * @todo As mentioned in the wiki of scriptaculous, the parameters value unfortunately has to be converted to a query string:
             *       http://madrobby.github.com/scriptaculous/ajax-autocompleter/
             *       Using it as a hash may work in the upcoming versions of script.aculo.us
             */
            options.parameters = $H(options.parameters).toQueryString();

            new Ajax.Autocompleter(element, el_choices, url, options);

        }
        else
        {
            throw new Error('Element ' + el + ' not found.')
        }

    }
});

idoit.JSONClient = Class.create({
    jsonGetString: '?call=json',

    initialize: function (parameters) {
        this.parameters = {};
        Object.extend(this.parameters, parameters || {
                useAuth: false
            });

        this.useAuth(this.parameters.useAuth);
    },

    useAuth: function (auth) {
        this.parameters.useAuth = !!auth;

        this.jsonGetString = (this.parameters.useAuth ? '?call=json&useAuth=1' : '?call=json');
    },

	post: function(url, parameters, callback) {

		return this.call(url, callback, {
			parameters: parameters,
			method: 'post'
		});

	},

	call: function(url, callback, options) {
		if (!callback) callback = Prototype.emptyFunction;

		Object.extend(this.parameters,	options.parameters || { });

		this.options = {
			method: 'get',
			evalJSON: true,
			parameters: this.parameters,
			requestHeaders: {
				Accept: 'application/json'
			},
			onSuccess: callback
		};
		Object.extend(this.options,		options || { });

		if (options && 'select' in options) {
			this.options.parameters.select = Object.toJSON(options.select);
			delete options.select;
		}

		new Ajax.Call(url, this.options);

		return true;
	},

	getObjectByID: function(objectID, callback, options) {
		return this.call(this.jsonGetString + '&'+C__CMDB__GET__OBJECT+'=' + objectID, callback, options);
	},

	getObjectsByFilter: function(filter, callback, options) {
		return this.call(this.jsonGetString + '&action=filter&filter=' + filter, callback, options);
	},

	getObjectsByType: function(objTypeID, callback, options) {
		return this.call(this.jsonGetString + '&objTypeID=' + objTypeID, callback, options);
	},

	getObjectsByGroup: function(groupObjID, callback, options) {
		return this.call(this.jsonGetString + '&action=getByGroup&objID=' + groupObjID, callback, options);
	},

	getObjectsByPersonGroup: function(groupObjID, callback, options) {
		return this.call(this.jsonGetString + '&action=getByPersonGroup&objID=' + groupObjID, callback, options);
	},

	getObjectsByTimeCondition: function(condition, callback, options) {
		return this.call(this.jsonGetString + '&action=getByTimeCondition&condition=' + condition.condition + '&from=' + condition.from + '&to=' + condition.to, callback, options);
	},

	getObjectsByRelationType: function(relationTypeID, callback, options) {
		return this.call(this.jsonGetString + '&action=getByRelationType&type=' + relationTypeID, callback, options);
	},

	getObjectsByCustomBrowserRequest: function(request, callback, options) {
		return this.call(this.jsonGetString + '&action=getObjectsByCustomBrowserRequest&request=' + Object.toJSON(request), callback, options);
	},

	getObjectsByReport: function(request, callback, options) {
		return this.call(this.jsonGetString + '&action=getObjectsByReport&request=' + Object.toJSON(request), callback, options);
	},

	getRelationsByObjectId: function(request, callback, options) {
		return this.call(this.jsonGetString + '&action=getRelationsByObjectId&request=' + request, callback, options);
	},

	createObject: function(p_objectTitle, p_objectTypeID, p_callback) {
		return this.post(this.jsonGetString + '&action=createObject', {
			objectTitle: p_objectTitle,
			objectTypeID: p_objectTypeID
		}, p_callback);
	},

	createObjectGroup: function(p_objectGroupTitle, p_objectIDs, p_forceOverwrite, p_callback) {
		return this.post(this.jsonGetString + '&action=createObjectGroup', {
			objectTitle: p_objectGroupTitle,
			forceOverwrite: p_forceOverwrite,
			objects: p_objectIDs
		}, p_callback);
	},

	insertNewEntry: function(p_entryTitle, p_entryTable, p_callback) {
		return this.post(this.jsonGetString + '&action=insertNewEntry', {
			entryTitle: p_entryTitle,
			entryTable: p_entryTable
		}, p_callback);
	},

    getRightsByObjectTypeId: function(p_objectTypeID, p_right, p_callback) {
        return this.post(this.jsonGetString + '&action=hasEditRightsByObjectType', {
            objTypeID: p_objectTypeID,
            right:p_right
        }, p_callback);
    }
});

idoit.Sanitizer = Class.create({

    initialize: function(parameters) {
        this.parameters = {};
        Object.extend(this.parameters, parameters || { });
    },

    stripTags: function(str, allow) {
        // making sure the allow arg is a string containing only tags in lowercase (<a><b><c>)
        allow = (((allow || "") + "").toLowerCase().match(/<[a-z][a-z0-9]*>/g) || []).join('');

        var tags = /<\/?([a-z][a-z0-9]*)\b[^>]*>/gi;
        var commentsAndPhpTags = /<!--[\s\S]*?-->|<\?(?:php)?[\s\S]*?\?>/gi;
        return str.replace(commentsAndPhpTags, '').replace(tags, function ($0, $1) {
            return allow.indexOf('<' + $1.toLowerCase() + '>') > -1 ? $0 : '';
        });
    },

    sanitizeText: function(p_value) {
        return this.sanitizeCall('text', p_value);
    },

    sanitizeFloat: function(p_value) {
        return this.sanitizeCall('float', p_value);
    },

    sanitizeInt: function(p_value) {
        return this.sanitizeCall('int', p_value);
    },

    sanitizeCall: function(p_type, p_value) {
        var new_val = '';
        new Ajax.Request('?ajax=1&call=sanitize_field', {
            parameters:{
                type: p_type,
                field_value: p_value
            },
            method:'post',
            asynchronous:  false,
            onSuccess:function (transport) {
                new_val = transport.responseJSON;
            }
        });
        return new_val;
    }
});

window.Sanitizer = new idoit.Sanitizer({});

// Initialize i-doit JSON Client.
window.idoitJSON = new idoit.JSONClient({
	raw: true
});

// First, checks if it isn't implemented yet.
if (!String.prototype.format) {
  String.prototype.format = function() {
    var args = arguments;
    return this.replace(/{(\d+)}/g, function(match, number) {
      return typeof args[number] != 'undefined' ? args[number] : match
      ;
    });
  };
}

// These two observer will be used to determine if the user holds a button while clicking (shift + click for selecting multiple checkboxes).
document.observe('keydown', function (ev) {
    window.keyPressed = ev.keyCode;

    if (ev.keyCode === 16)
    {
        window.keyPressedShift = true;
    }
    else if (ev.keyCode === 13)
    {
        window.keyPressedEnter = true;
    }
});

document.observe('keyup', function (ev) {
    window.keyPressed = null;

    if (ev.keyCode === 16)
    {
        window.keyPressedShift = false;
    }
    else if (ev.keyCode === 13)
    {
        window.keyPressedEnter = false;
    }
});

document.observe('mousemove', function (ev) {
    window.mouseX = ev.pageX;
    window.mouseY = ev.pageY;
});