/* taborder i-doit implementation */

// addTableObserver -- binds event handlers to the events associated with the table generated.
TableOrderer.prototype.addTableObserver = function() {
	var that = this,
        tid = this.table.id;

	// We don't want to set observer to the "add all" button.
    this.table.select('th')
		.invoke('observe','click',this.thClickbfx)
		.invoke('observe','mouseover',this.thOverbfx)
		.invoke('observe','mouseout',this.thOutbfx);

	this.table.select('tr.data')
		.invoke('observe','click',this.trClick.bindAsEventListener(this))
		.invoke('observe','mouseover',this.trOver.bindAsEventListener(this))
		.invoke('observe','mouseout',this.trOut.bindAsEventListener(this));

	// This part is new, we want to add a new observer to the "add-all" button.
    if ($(tid + '-add_all'))
    {
        $(tid + '-add_all').on('click', function (ev) {
            // This is used to stop the "sorting" logic when clicking the "all" button.
            ev.stopPropagation();
            ev.preventDefault();

            that.addAll();
        });
    }

    if ($(tid + '-add_page')) {
        $(tid + '-add_page').on('click', function(ev) {
            // This is used to stop the "sorting" logic when clicking the "page" button.
            ev.stopPropagation();
            ev.preventDefault();

            that.addPage();
        });
    }
};

TableOrderer.prototype.addAll = function () {
    var i,
        $img = $(this.table.id + '-add_all').down('img'),
        data = this.cache || this.data;

    // @todo  The HTML gets replaced by another callback, so the "ajax loading" will not be shown :(
    // $img.writeAttribute('src', window.dir_images + 'ajax-loading.gif');

    if (data)
    {
        for (i in data)
        {
            if (!data.hasOwnProperty(i)) continue;

            browserPreselection.add(Object.values(data[i]));
        }

        this.updateTable();
    }

    // $img.writeAttribute('src', window.dir_images + 'icons/silk/add.png');
};

TableOrderer.prototype.addPage = function() {
    var $buttons = this.table.select('a[onclick*="browserPreselection.add"]'),
        $img = $(this.table.id + '-add_page').down('img');

    // @todo  The HTML gets replaced by another callback, so the "ajax loading" will not be shown :(
    // $img.writeAttribute('src', window.dir_images + 'ajax-loading.gif');

    if ($buttons.length) {
        $buttons.invoke('simulate', 'click');
	}

	this.updateTable();

    // $img.writeAttribute('src', window.dir_images + 'icons/silk/add.png');
};

TableOrderer.prototype.orderRule = function (s) {
	// Updated Regex - The points are new delimiters.
	var dateRE = /^(\d{2})[\/\.\- ](\d{2})[\/\.\- ](\d{4}|\d{2})/;
	var exp=new RegExp(dateRE);
	if (exp.test(s)) {
		s = this.options.dateFormat == 'd' ? s.replace(dateRE,"$3$2$1") : s.replace(dateRE,"$3$1$2");
	}
	return s;
};

TableOrderer.prototype.createPager = function() {
	/* Class overwritings */
	if ($(this.table.id + '-pager')) $(this.table.id + '-pager');

	$(this.table.id+'-pager')
		.insert({bottom : new Element('a',{'id' : this.table.id+'-page-first', 'class' : 'btn btn-small first-page-button'}).update(new Element('span', {style:'margin:0 3px;'}).update(this.msgs.paginationFirst))})
		.insert({bottom : new Element('a',{'id' : this.table.id+'-page-prev', 'class' : 'btn btn-small prev-page-button'}).update(new Element('span', {style:'margin:0 3px;'}).update(this.msgs.paginationPrev))})
		.insert({bottom : '<span id="' + this.table.id+'-page-current' + '" class="currentpage">' + this.tools.page + '</span>'})
		.insert({bottom : new Element('a',{'id' : this.table.id+'-page-next', 'class' : 'btn btn-small next-page-button'}).update(new Element('span', {style:'margin:0 3px;'}).update(this.msgs.paginationNext))})
		.insert({bottom : new Element('a',{'id' : this.table.id+'-page-last', 'class' : 'btn btn-small last-page-button'}).update(new Element('span', {style:'margin:0 3px;'}).update(this.msgs.paginationLast))})
		.insert({bottom : this.msgs.paginationOf + '<span id="' + this.table.id + '-page-total' + '" class="totalpages">' + this.tools.pages + '</span>' + this.msgs.paginationPages});
};

TableOrderer.prototype.trClick = function(e) {};

TableOrderer.prototype.createFirstRow = function(obj) {
	var row = '<thead><tr>';

	this.tableColumnsName.each(function(i) {
		// Set the first element to a specific width, to avoid a ugly bug in webkit-browsers.
		if (i == '__checkbox__') {
			var all_button = '';

			// Only display the button on multiselect browsers and browsers without secondSelection.
            if (this.options.multiselection && !this.options.secondSelection && !this.options.secondSelectionExists)
            {
                all_button = '<a href="#" id="' + this.table.id + '-add_all" class="btn btn-small mr5" title="' + idoit.Translate.get('LC__CMDB__OBJECT_BROWSER__ADD_ALL_BY_FILTER') + '">' +
                             '<img src="' + window.dir_images + 'icons/silk/add.png" style="margin-right:3px;" /><span>' + idoit.Translate.get('LC__UNIVERSAL__ALL') + '</span>' +
                             '</a>' +
                             '<a href="#" id="' + this.table.id + '-add_page" class="btn btn-small" title="' + idoit.Translate.get('LC__CMDB__OBJECT_BROWSER__ADD_ALL_ON_PAGE') + '">' +
                             '<img src="' + window.dir_images + 'icons/silk/add.png" style="margin-right:3px;" /><span>' + idoit.Translate.get('LC__UNIVERSAL__PAGE') + '</span>' +
                             '</a>';
            }

			row += '<th id="'+this.table.id+'-id" style="width:80px; text-align:center;">' + all_button + '</th>';
		} else {
			row += '<th id="'+this.table.id+'-'+i+'">'+i+'</th>';
		}
	}.bind(this));

	row += '</tr></thead>';
	return row;
};

TableOrderer.prototype.updateTable = function() {
	this.table = $('data-grid-'+this.element);
	$(this.table.id).update();
	this.createRows();
	this.addTableObserver();
	this.makeUnsort();

	if (this.options.colgroup) this.table.insert({top:this.options.colgroup});
};

TableOrderer.prototype.radioButton = function(values, checked) {
	return '<input type="radio" name="listSelection" '+(checked?'checked="checked" ':' ')+'onclick="'+this.options.objectSelectionCallback+'([\''+values[0]+'\', \'' + (values[1]).replace(/['"]/g, "") + '\', \''+values[2]+'\', \''+values[3]+'\']);" />';
};

TableOrderer.prototype.addButton = function(values, view) {
        if ((this.options.multiselection && !this.options.secondSelectionExists) || (this.options.multiselection && this.options.secondSelectionExists && view == 'r')) {
            var func_string = "browserPreselection.add([" + values[0] + ", '" + (values[1]).replace(/['"]/g, "") + "', '" + values[2] + "', '" + values[3] + "'], true);" +
			"this.up().update(window.browserList.removeButton([" + values[0] + ", '" + (values[1]).replace(/['"]/g, "") + "', '" + values[2] + "', '" + values[3] + "'], '" + view + "'));";

            return '<button type="button" class="btn btn-small btn-block" onclick="' + func_string + '">' +
                   '<img src="' + window.dir_images + 'icons/silk/add.png" style="margin-right:3px;" />' +
                   '<span>' + idoit.Translate.get('LC__CMDB__OBJECT_BROWSER__SCRIPT__ADD') + '</span>' +
                   '</button>';
        } else {
            return this.radioButton(values, false);
        }
};

TableOrderer.prototype.removeButton = function (values, view) {
    if ((this.options.multiselection && !this.options.secondSelectionExists) || (this.options.multiselection && this.options.secondSelectionExists && view == 'r'))
    {
        var func_string = "browserPreselection.remove(" + values[0] + ");" + "if (this.up()){this.up().update(window.browserList.addButton([" + values[0] + ", '" + (values[1]).replace(/['"]/g, "") + "', '" + values[2] + "', '" + values[3] + "'], '" + view + "'))}";

        return new Element('button', {type:'button', className:'btn btn-small btn-block', onclick:func_string})
            .update(new Element('img', {src:window.dir_images + 'icons/silk/delete.png', style:'margin-right:3px'}))
            .insert(new Element('span').update(idoit.Translate.get('LC__CMDB__OBJECT_BROWSER__SCRIPT__REMOVE')));
    }
    else
    {
        return this.radioButton(values, true);
    }
};

TableOrderer.prototype.createRow = function(obj, index) {
	var values = Object.values(obj), tmpClassName, tmpContent;
	var tr = new Element('tr', {'class': 'data line'+(index % 2), id: this.table.id+'-'+index}).writeAttribute('data-objectid', values[0]);

	this.tableColumnsName.each(function(s, index) {
		if (s == '__checkbox__') {
			if (!browserPreselection.exists(values[0])) {
				tmpContent = this.addButton(values, 'l');
			} else {
				tmpContent = this.removeButton(values, 'l');
			}

			tmpClassName = this.table.id+'-column-checkbox center';
		} else {
			tmpContent = values[index];
			if (index === 1) {
				/* Changes for ID-1110 */
				tmpContent += '<span id="obj-browser-quickinfo-' + values[0] + '" class="fr quickinfo" data-obj-id="' + values[0] + '"><img src="' + window.dir_images + 'icons/infoicon/info.png" class="vam" /></span>';
			}
			tmpClassName = this.table.id+'-column-'+s;
		}

		tr.insert(new Element('td', {
			className: tmpClassName
		}).update(tmpContent));
	}.bind(this));

	if (Prototype.Browser.IE && tr.outerHTML) return tr.outerHTML;
	else return tr;
};

TableOrderer.prototype.createFilter = function() {
	var option = '';
	this.tableColumnsName.each(function(i) {
		if (i != '__checkbox__') {
			option += '\t<option value="'+i+'">'+i.replace(/(_)/g,' ')+'</option>\n';
		}
	});

	$(this.table.id+'-options')
		.insert({bottom : this.msgs.filterLabel})
		.insert({bottom : '<select id="'+this.table.id+'-filter-column" class="input input-mini">'+option+'</select>'})
		.insert({bottom : new Element('input',{id: this.table.id+'-filter-data', className:'input input-mini'})});

	this.tools.filterCol = $F(this.table.id + '-filter-column');
	this.tools.filterData = $F(this.table.id + '-filter-data');
};

TableOrderer.prototype.thClick = function(e) {

	// Don't sort, if we click on the first/ID field.
	if ((this.table.id + '-id') == (e.element().id)) {
		return;
	}

	this.defineOrderField(e);
	this.defineOrder();
	this.orderData(this.orderField);

	if (this.order == 'desc') {this.cache = this.cache.reverse();}

	this.updateTable();
};