/* Copyright: InteRiders <http://interiders.com/> - Distributed under MIT - Keep this message! */
/* i-doit extension copyright: i-doit team <http://www.i-doit.com> */

/*
  Credits:
  - Caret position method: Diego Perini <http://javascript.nwbox.com/cursor_position/cursor.js>
  - Guillermo Rauch: Original MooTools script
  - Ran Grushkowsky/InteRiders Inc. : Porting into Prototype and further development

  Changelog:
  - 0.1: translation of MooTools script
  - 0.2: renamed from Proto!TextboxList to Proto!MultiSelect, added new features/bug fixes
         added feature: support to fetch list on-the-fly using AJAX    Credit: Cheeseroll
         added feature: support for value/caption
         added feature: maximum results to display, when greater displays a scrollbar   Credit: Marcel
         added feature: filter by the beginning of word only or everywhere in the word   Credit: Kiliman
         added feature: shows hand cursor when going over options
         bug fix: the click event stopped working
         bug fix: the cursor does not 'travel' when going up/down the list   Credit: Marcel
  - 0.3: some bug fixes
         prototype 1.7 support
         i-doit addons
  - 0.4: adding values via comma now possible

 */
Object.extend(Event, {
    KEY_COMMA: 188
});

var ResizableTextbox = Class.create({

    options: $H({
	min: 5,
	max: 500,
	step: 7
    }),

    initialize: function(element, options) {
	var that = this;
	this.options.update(options);
	this.el = $(element);
	this.width = this.el.offsetWidth;
	this.el.observe(
	    'keyup', function() {
		var newsize = that.options.get('step') * $F(this).length;
		if(newsize <= that.options.get('min')) newsize = that.width;
		if(! ($F(this).length == this.retrieveData('rt-value') || newsize <= that.options.min || newsize >= that.options.max))
		    this.setStyle({
			'width': newsize
		    });
	    }).observe('keydown', function() {
	    this.cacheData('rt-value', $F(this).length);
	});
    }
});

var TextboxList = Class.create({

    options: $H({
	resizable: {},
	className: 'bit',
	separator: ',',
	extrainputs: true,
	startinput: true,
	hideempty: true,
	results: 10,
	wordMatch: false,
	editmode: 0
    }),

    initialize: function(element, options) {
		this.options.update(options);

		if ($(element)) {
		    this.element = $(element).hide();
		    this.bits = new Hash();
		    this.events = new Hash();
		    this.count = 0;
		    this.current = false;
		    if (!options.editmode) return;

		    this.maininput = this.createInput({
				'class': 'maininput',
				'id': options.inputId
		    });
		    this.holder = new Element('ul', {
				'class': 'holder'
		    }).insert(this.maininput);

		    //this.holder.insert(this.createAddButton());

		    this.element.insert({
				'before':this.holder
		    });

		    this.holder.observe('click', function(event){
				event.stop();
				if(this.maininput != this.current) this.focus(this.maininput);
		    }.bind(this));

		    this.makeResizable(this.maininput);
		    this.setEvents();
		}
    },

    createAddButton: function() {
		var that = this;

		var button = new Element('img', {
		    'src':		'images/icons/plus-green.gif',
		    'className':'vam',
		    'height':	12
		}).observe('click', function(e) {
		    var title = this.previous().value;
		    if(title.length == 0) return;
		    var found = 0;

		    that.data.each(function(e){
				if(e.evalJSON(true).caption == title) found = e;
		    });

		    if(found == 0) {
				idoitJSON.insertNewEntry(title, that.table, function(transport) {
				    that.autoAdd({
						caption: title,
						value: transport.responseText
				    });
				});
		    } else if(found > 0){
				that.autoAdd(found.evalJSON(true));
		    }

		});

		return button;
    },

    setEvents: function() {
		document.observe(Prototype.Browser.IE ? 'keydown' : 'keypress', function(e) {
		    if(! this.current) return;
		    if(this.current.retrieveData('type') == 'box' && e.keyCode == Event.KEY_BACKSPACE) e.stop();
		}.bind(this));

		document.observe('keyup', function(e) {
			e.stop();
			if(! this.current) return;
			switch(e.keyCode){
			    case Event.KEY_LEFT:
					return this.move('left');
					break;
			    case Event.KEY_RIGHT:
					return this.move('right');
					break;
			    case Event.KEY_DELETE:
			    case Event.KEY_BACKSPACE:
					return this.moveDispose();
					break;
			}
	    }.bind(this)).observe('click', function() {
			document.fire('blur');
		}.bindAsEventListener(this));
    },

    update: function() {
		this.element.value = this.bits.values().join(this.options.get('separator'));
		return this;
    },

    add: function(text, html) {
		var id = this.options.get('className') + '-' + this.count++;
		var el = this.createBox($pick(html, text), {
		    'id': id
		});

		(this.current || this.maininput).insert({
		    'before':el
		});

		el.observe('click', function(e) {
		    e.stop();
		    this.focus(el);
		}.bind(this));

		this.bits.set(id, text.value);
		this.update();

		if(this.options.get('extrainputs') && (this.options.get('startinput') || el.previous())) this.addSmallInput(el,'before');

		return el;
    },

    addSmallInput: function(el, where) {
		var input = this.createInput({
		    'class': 'smallinput'
		});
		el.insert({}
		    [where] = input);
		input.cacheData('small', true);
		this.makeResizable(input);
		if(this.options.get('hideempty')) input.hide();
		return input;
    },

    dispose: function(el) {
		this.bits.unset(el.id);
		if(el.previous() && el.previous().retrieveData('small')) el.previous().remove();
		if(this.current == el) this.focus(el.next());
		if(el.retrieveData('type') == 'box') el.onBoxDispose(this);
		el.remove();
		this.update();
		return this;
    },

    focus: function(el, nofocus) {
		if(! this.current) el.fire('focus');
		else if(this.current == el) return this;
		this.blur();
		el.addClassName(this.options.get('className') + '-' + el.retrieveData('type') + '-focus');
		if(el.retrieveData('small')) el.setStyle({
		    'display': 'block'
		});
		if(el.retrieveData('type') == 'input') {
		    el.onInputFocus(this);
		    if(! nofocus) this.callEvent(el.retrieveData('input'), 'focus');
		}
		else el.fire('onBoxFocus');
		this.current = el;
		return this;
    },

    blur: function(noblur) {
		if(! this.current) return this;
		if(this.current.retrieveData('type') == 'input') {
		    var input = this.current.retrieveData('input');
		    if(! noblur) this.callEvent(input, 'blur');
		    input.onInputBlur(this);
		}
		else this.current.fire('onBoxBlur');
		if(this.current.retrieveData('small') && ! input.get('value') && this.options.get('hideempty'))
		    this.current.hide();
		this.current.removeClassName(this.options.get('className') + '-' + this.current.retrieveData('type') + '-focus');
		this.current = false;
		return this;
    },

    createBox: function(text, options) {
		return new Element('li', options).addClassName(this.options.get('className') + '-box').update(text.caption).cacheData('type', 'box');
    },

    createInput: function(options) {
		var li = new Element('li', {
		    'class': this.options.get('className') + '-input'
		});
		var el = new Element('input', Object.extend(options,{
		    'type': 'text'
		}));
		el.observe('click', function(e) {
		    e.stop();
		}).observe('focus', function(e) {
		    if(! this.isSelfEvent('focus')) this.focus(li, true);
		}.bind(this)).observe('blur', function() {
		    if(! this.isSelfEvent('blur')) this.blur(true);
		}.bind(this)).observe('keydown', function(e) {
		    this.cacheData('lastvalue', this.value).cacheData('lastcaret', this.getCaretPosition());
		});
		var tmp = li.cacheData('type', 'input').cacheData('input', el).insert(el);
		return tmp;
    },

    callEvent: function(el, type) {
		this.events.set(type, el);
		el[type]();
    },

    isSelfEvent: function(type) {
		return (this.events.get(type)) ? !! this.events.unset(type) : false;
    },

    makeResizable: function(li) {
		var el = li.retrieveData('input');
		el.cacheData('resizable', new ResizableTextbox(el, Object.extend(this.options.get('resizable'),{
		    min: el.offsetWidth,
		    max: (this.element.getWidth()?this.element.getWidth():0)
		})));
		return this;
    },

    checkInput: function() {
		var input = this.current.retrieveData('input');
		return (! input.retrieveData('lastvalue') || (input.getCaretPosition() === 0 && input.retrieveData('lastcaret') === 0));
    },

    move: function(direction) {
		var el = this.current[(direction == 'left' ? 'previous' : 'next')]();
		if(el && (! this.current.retrieveData('input') || ((this.checkInput() || direction == 'right')))) this.focus(el);
		return this;
    },

    moveDispose: function() {
		if(this.current.retrieveData('type') == 'box') return this.dispose(this.current);
		if(this.checkInput() && this.bits.keys().length && this.current.previous()) return this.focus(this.current.previous());
    }

});

//helper functions
Element.addMethods({
    getCaretPosition: function() {
	if (this.createTextRange) {
	    var r = document.selection.createRange().duplicate();
	    r.moveEnd('character', this.value.length);
	    if (r.text === '') return this.value.length;
	    return this.value.lastIndexOf(r.text);
	} else return this.selectionStart;
    },
    cacheData: function(element, key, value) {
	if (Object.isUndefined(this[$(element).identify()]) || !Object.isHash(this[$(element).identify()]))
	    this[$(element).identify()] = $H();
	this[$(element).identify()].set(key,value);
	return element;
    },
    retrieveData: function(element,key) {
	return this[$(element).identify()].get(key);
    }
});

function $pick(){
    for(var B=0,A=arguments.length;B<A;B++){
	if(!Object.isUndefined(arguments[B])){
	    return arguments[B];
	}
    }
    return null;
}


/*
 * Class for mulitselect box
 * Example:
 *
 * HTML Part:
 * <ul style="list-style-type:none;">
 *      <li id="list" class="input-text" >
 *         <div id="auto">
 *         		<div class="default" style='display:none;'></div>
 *           	<ul class="feed">
 *           	</ul>
 *         </div>
 * 		</li>
 * </ul>
 *
 * Initialization:
 * tlist3 = new inputListAutoComplete('demo', 'auto',{ajaxURL:'[{$ajaxURL}]', table:'isys_net_dns_domain', editmode:[{$is_editmode}], mainData:[{$json_data}], container:'C__CATS__NET__DNS_DOMAIN'});
 *
 */
if (!idoit) var idoit = {};
idoit.multiSelect = Class.create(TextboxList, {

    loptions: $H({
		autocomplete: {
		    'opacity': 0.8,
		    'maxresults': 15,
		    'minchars': 1
		}
    }),

    initialize: function($super, element, autoholder, options, func) {
		var that = this;
		var dataString = '';

		$super(element, options);

		this.data = [];
		this.inputFields = options.inputFields;
		this.table = options.table;
		this.editmode = options.editmode;

		if (options.data != undefined && options.data != '' && options.data != 'null') {
			options.data.evalJSON(true).each(function(t) {
				this.autoFeed(t);
			}.bind(this));

			// Set data
			if(options.mainData != undefined) {
				this.mainData = options.mainData;

				if(this.editmode) {
					this.mainData.each(function(e){
						that.autoAdd(e);
					});
				} else{
					dataString = '<ul class="holder">';
					this.mainData.each(function(e){
						dataString += '<li class="bit-box" style="padding-right:5px;">'+e.caption+'</li>';
					});
					$(autoholder).up().insert(dataString+'</ul>');
				}
			}
		}
		else if (options.ajaxURL != undefined) {
		    this.ajaxURL = options.ajaxURL;

		    /* Load the whole list on initialization and just filter on client side */
		    if (!Object.isUndefined(this.options.get('ajaxURL'))) {
			    new Ajax.Request(this.options.get('ajaxURL'), {
					    method: 'get',
					    onSuccess: function (transport) {
						    var json = transport.responseText.evalJSON(true);

						    if (json.hasOwnProperty('success') && json.hasOwnProperty('data')) {
							    if (json.success) {
								    json.data.each(function (t) {
									    this.autoFeed(t);
								    }.bind(this));
							    } else {
								    alert(json.message);
							    }
						    } else {
						        json.each(function (t) {
								    this.autoFeed(t);
							    }.bind(this));
						    }

						    // Set data
						    if (options.mainData != undefined) {
							    this.mainData = options.mainData;

							    if (this.editmode) {
								    this.mainData.each(function (e) {
									    that.autoAdd(e);
								    });
							    } else {
								    dataString = '<ul class="holder">';
								    this.mainData.each(function (e) {
									    dataString += '<li class="bit-box" style="padding-right:5px;">' + e.caption + '</li>';
								    });
								    $(autoholder).up().insert(dataString + '</ul>');
							    }
						    }
					    }.bind(this)
				    }
			    );
		    }
		}

		this.autoholder = $(autoholder).setOpacity(this.loptions.get('autocomplete').opacity).setStyle({zIndex: 1500});
		this.autoholder.observe('mouseover',function() {
		    this.curOn = true;
		}.bind(this)).observe('mouseout',function() {
		    this.curOn = false;
		}.bind(this));
		this.autoresults = this.autoholder.select('ul').first();
    },

    autoShow: function(search) {
		this.autoholder.setStyle({
		    'display': 'block'
		});
		this.autoholder.descendants().each(function(e) {
		    e.hide()
		});
		this.autocurrent = false;

		if (!search) search = '';
		search = search.strip();

		if(search.length < this.loptions.get('autocomplete').minchars)
		{
		    this.autoholder.select('.default').first().setStyle({
				'display': 'block'
		    });
		    this.resultsshown = false;
		} else {
		    this.resultsshown = true;
		    this.autoresults.setStyle({
				'display': 'block'
		    }).update('');

		    if (this.options.get('wordMatch')) var regexp = new RegExp("(^|\\s)"+search,'i');
		    else var regexp = new RegExp(search,'i');

		    var count = 0;
		    this.data.filter(
				function(str) {
				    return str ? regexp.test(str.evalJSON(true).caption) : false;
				}
			).each(
				function(result, ti) {
				    count++;
				    if(ti >= this.loptions.get('autocomplete').maxresults) return;
				    var that = this;
				    var el = new Element('li').observe('click',function(e) {
					e.stop();
					that.autoAdd(this);
				    }
				    ).observe('mouseover', function() {
					that.autoFocus(this);
				    } ).update(
					this.autoHighlight(result.evalJSON(true).caption, search)
					);
				    this.autoresults.insert(el);
				    el.cacheData('result', result.evalJSON(true));
				    if(ti == 0) this.autoFocus(el);
				},
				this
			);
		}
		if (count > this.options.get('results'))
		    this.autoresults.setStyle({
				'height': (this.options.get('results')*22)+'px'
		    });
		else
		    this.autoresults.setStyle({
				'height': (count?(count*22):0)+'px'
		    });

		return this;
	},

	autoHighlight: function(html, highlight) {
		return html.gsub(new RegExp(highlight,'i'), function(match) {
		    if (match[0]) return '<em>' + match[0] + '</em>';
		    else return '';
		});
    },

    autoHide: function() {
		this.resultsshown = false;
		this.autoholder.hide();
		return this;
    },

    autoFocus: function(el) {
		if(! el) return;
		if(this.autocurrent) this.autocurrent.removeClassName('auto-focus');
		this.autocurrent = el.addClassName('auto-focus');

		return this;
    },

    autoMove: function(direction) {
		if(!this.resultsshown) return;
		this.autoFocus(this.autocurrent[(direction == 'up' ? 'previous' : 'next')]());
		this.autoresults.scrollTop = this.autocurrent.positionedOffset()[1]-this.autocurrent.getHeight();
		return this;
    },

    autoFeed: function(text) {
		if (this.data.indexOf(Object.toJSON(text)) == -1)
			this.data.push(Object.toJSON(text));
		return this;
    },

    autoAdd: function(el) {
		if(el.caption != undefined) {
            if(el.caption.replace(/^\s+|\s+$/g, '') != '') {
                this.add(el);
                delete this.data[this.data.indexOf(Object.toJSON(el))];
                if(this.lastinput != undefined) this.lastinput.clear().focus();
            }
		} else {
		    if(!el || !el.retrieveData('result')) return;

		    this.add(el.retrieveData('result'));
		    delete this.data[this.data.indexOf(Object.toJSON(el.retrieveData('result')))];

		    this.autoHide();
		    var input = this.lastinput || this.current.retrieveData('input');
		    input.clear().focus();
		}
        this.handleMultiEdit();

		return this;
    },

    createInput: function($super,options) {
		var li = $super(options);
		var that = this;
		var input = li.retrieveData('input');

		input.observe('keydown', function(e) {
		    this.dosearch = false;
		    switch(e.keyCode) {
				case Event.KEY_UP:
				    e.stop();
				    return this.autoMove('up');
				break;
				case Event.KEY_DOWN:
				    e.stop();
				    return this.autoMove('down');
				break;
				case Event.KEY_COMMA:
				case Event.KEY_RETURN:

				    e.stop();

				    if(this.autocurrent) {
				    	/* Element exists, so add it to the list */
					    this.autoAdd(this.autocurrent);
				    } else {
				    	/* Element does not exist, so add it to the database and then to the list */
					    if(input && input.value.length > 0 && input.value.replace(/^\s+|\s+$/g, '') != '') {
							idoitJSON.insertNewEntry(input.value, that.table, function(transport) {
								this.autoAdd({
									caption: input.value,
									value: transport.responseText
							    });
							    input.value = '';
							}.bind(this));
					    }
				    }

                    // Trigger change event
                    this.element.simulate('change');

					this.autocurrent = false;
					this.autoenter = true;

				break;
				case Event.KEY_ESC:
				    this.autoHide();
				    if(this.current && this.current.retrieveData('input'))
					this.current.retrieveData('input').clear();
				break;
				default:
				    this.dosearch = true;
				break;
		    }
		}.bind(this))
		.observe('keyup',function(e) {
			switch(e.keyCode) {
				case Event.KEY_UP:
				case Event.KEY_DOWN:
				case Event.KEY_RETURN:
				case Event.KEY_COMMA:
				case Event.KEY_ESC:
                    e.stop();
				break;
				default:
				    if(this.searchTimeout) clearTimeout(this.searchTimeout);
				    this.searchTimeout = setTimeout(function(){
						if(this.dosearch) this.autoShow(input.value);
					}.bind(this), 250);
				break;
			}

		}.bind(this))
		.observe(Prototype.Browser.IE ? 'keydown' : 'keypress', function(e) {
		    if(this.autoenter) e.stop();
		    this.autoenter = false;
		}.bind(this));

		return li;
    },

    createBox: function($super,text, options) {
		var li = $super(text, options);
		li.observe('mouseover',function() {
		    this.addClassName('bit-hover');
		}).observe('mouseout',function() {
		    this.removeClassName('bit-hover')
		});
		var a = new Element('a', {
		    'href': '#',
		    'class': 'closebutton'
		}
		);
		a.observe('click',function(e) {
		    e.stop();
		    if(! this.current) this.focus(this.maininput);
		    this.dispose(li);

            this.handleMultiEdit();

            // Trigger change event
            this.element.simulate('change');
		}.bind(this));
		li.insert(a).cacheData('text', Object.toJSON(text));
		return li;
    },

    handleMultiEdit: function() {
        // MultiEdit-Handling
        if (this.inputFields.indexOf('[skip]') != -1) {
            // Walk through each multiselect field
            $$('.' + this.element.classList[0]).each(function (item) {
                if (item.id.indexOf('[skip]') == -1) {
                    // Get multiselect object
                    var l_multiselect_object = window["mselect_" + item.id.replace(/[-\[\]]/g, "").replace("__HIDDEN", "")];

                    if (l_multiselect_object) {
                        // Retrieve bits from skip-field
                        var l_bits = this.holder.clone(true).select('.bit-box');

                        // Remove existing bits first
                        l_multiselect_object.holder.select('.bit-box').each(function (item) {
                            item.remove();
                        });

                        // Reinit required parts
                        l_multiselect_object.bits = new Hash();
                        l_multiselect_object.events = new Hash();
                        l_multiselect_object.count = 0;
                        l_multiselect_object.current = false;
                        l_multiselect_object.data = this.data.clone(true);

                        // Add bits from skip-field
                        l_bits.each(function (item, index) {
                            l_multiselect_object.autoAdd({
                                caption: item.innerHTML,
                                value: this.bits._object[item.id]
                            })
                        }.bind(this));
                    }
                }
            }.bind(this));
        }
    }
});

Element.addMethods({
    onBoxDispose: function(item,obj) {
		obj.autoFeed(item.retrieveData('text').evalJSON(true));
	},
	onInputFocus: function(el,obj) {
		obj.autoShow(el.retrieveData('input').value);
	},
	onInputBlur: function(el,obj) {
        if (el.value.replace(/^\s+|\s+$/g, '') != '') {
            idoitJSON.insertNewEntry(el.value, obj.table, function(transport) {
								this.autoAdd({
									caption: el.value,
									value: transport.responseText
							    });
							    el.value = '';
							}.bind(obj));
        }

        obj.lastinput = el;
		if(!obj.curOn) {
		    obj.blurhide = obj.autoHide.bind(obj).delay(0.1);
		}
	    //el.value = '';
	},
	filter:function(D,E){
		var C=[];
		for(var B=0,A=this.length;B<A;B++) {
		    if(D.call(E,this[B],B,this)) {
				C.push(this[B]);
		    }
		}
		return C;
    }
});