<?php
/**
 * i-doit core classes.
 *
 * @package     i-doit
 * @subpackage  Core
 * @author      Dennis Stücken <dstuecken@synetics.de>
 * @copyright   synetics GmbH
 * @license     http://www.gnu.org/licenses/agpl-3.0.html GNU AGPLv3
 */
class isys_tenantsettings extends isys_settings
{
	/**
	 * To identify changes.
	 * @var  boolean
	 */
	private static $m_changed = false;

	/**
	 * Settings storage
	 * @var  array
	 */
	protected static $m_settings = [];

	/**
	 * Cache file.
	 * @var  string
	 */
	protected static $m_cachefile = 'settings.cache';

	/**
	 * Database component.
	 * @var  isys_component_dao_tenant_settings
	 */
	protected static $m_dao;

	/**
	 * This will cache the cache-directory string.
	 * @var  string
	 */
	protected static $m_cache_dir = NULL;

    /**
     * Settings initialized?
     *
     * @var bool
     */
    protected static $m_initialized = false;

	/**
	 * Settings register.
	 * Constant C__TREE__TITLE__MAXLEN is not used
	 *
	 * @var array
	 */
	protected static $m_definition = array(
		/* These settings do exist, but shall not be displayed in "administration > system settings".
		'Registry' => [
			'cmdb.registry.show_full_lists' => [
				'title' => ' ### show full list',
				'type' => 'select',
				'options' => [
					'0' => 'LC__UNIVERSAL__NO',
					'1' => 'LC__UNIVERSAL__YES'
				]
			],
			'cmdb.registry.quicksave' => [
				'title' => ' ### quicksave',
				'type' => 'select',
				'options' => [
					'0' => 'LC__UNIVERSAL__NO',
					'1' => 'LC__UNIVERSAL__YES'
				]
			],
			'cmdb.registry.sysid_readonly' => [
				'title' => ' ### SYS-ID readonly',
				'type' => 'select',
				'options' => [
					'0' => 'LC__UNIVERSAL__NO',
					'1' => 'LC__UNIVERSAL__YES'
				]
			],
			'cmdb.registry.object_dragndrop' => [
				'title' => ' ### Object drag\'n\'drop',
				'type' => 'select',
				'options' => [
					'0' => 'LC__UNIVERSAL__NO',
					'1' => 'LC__UNIVERSAL__YES'
				]
			],
		],
		*/

		'Display Limits' => array(
			'cmdb.limits.obj-browser.objects-in-viewmode' => array(
				'title' => 'LC__SETTINGS__CMDB__OBJ_BROWSER__OBJECTS_IN_VIEWMODE',
				'default' => 8,
				'type' => 'text',
				'description' => 'LC__SETTINGS__CMDB__OBJ_BROWSER__OBJECTS_IN_VIEWMODE_DESCRIPTION'
			),
			'gui.lists.preload-pages' => array(
				'title' => 'LC__SYSTEM__SETTINGS__TENANT__PRELOAD_OBJECT_PAGES_TITLE',
				'type' => 'text',
				'placeholder' => '30',
				'default' => '30'
			),
			'cmdb.object-browser.max-objects' => array(
				'title' => 'LC__SYSTEM_SETTINGS__OBJECT_BROWSER_RESULT_LIMIT',
				'type' => 'text',
				'placeholder' => '1500'
			),
			'cmdb.limits.port-lists-vlans' => array(
				'title' => 'LC__SETTINGS__CMDB__VLAN_LIMIT_IN_PORT_LISTS',
				'default' => 5,
				'type' => 'text'
			),
			'cmdb.limits.port-lists-layer2' => array(
				'title' => 'LC__SETTINGS__CMDB__LAYER2_LIMIT_IN_LOGICAL_PORT_LISTS',
				'default' => 5,
				'type' => 'text'
			),
			'cmdb.limits.port-overview-default-vlan-only' => array(
				'title' => 'LC__SETTINGS__CMDB__PORT_OVERVIEW_DEFAULT_VLAN_ONLY',
				'type' => 'select',
				'options' => array(
					'0' => 'LC__UNIVERSAL__NO',
					'1' => 'LC__UNIVERSAL__YES'
				)
			),
			'cmdb.limits.connector-lists-assigned_connectors' => array(
				'title' => 'LC__SETTINGS__CMDB__ASSIGNED_CONNECTOR_LIMIT_IN_CONNECTOR_LISTS',
				'default' => 5,
				'type' => 'text'
			),
			'cmdb.limits.ip-lists' => array(
				'title' => 'LC__SETTINGS__CMDB__IP_LISTS_LIMIT',
				'default' => 5,
				'type' => 'text'
			),
		),
		'LC__SYSTEM_SETTINGS__TENANT__IP_LIST' => array(
			'cmdb.ip-list.cache-lifetime' => array(
				'title' => 'LC__SYSTEM_SETTINGS__TENANT__IP_LIST__CACHE_LIFETIME',
				'default' => 86400,
				'placeholder' => 86400,
				'type' => 'text'
			),
			'cmdb.ip-list.ping-method' => array(
				'title' => 'LC__SYSTEM_SETTINGS__TENANT__IP_LIST__PING_METHOD',
				'type' => 'select',
				'options' => array(
					'nmap' => 'Ping via NMAP',
					'fping' => 'Ping via FPING'
				)
			),
			'cmdb.ip-list.nmap-parameter' => array(
				'title' => 'LC__SYSTEM_SETTINGS__TENANT__IP_LIST__NMAP_PARAMETER',
				'type' => 'select',
				'options' => array(
					'PE' => 'PE/PP/PM: ICMP echo, timestamp, and netmask request discovery probes',
					'sP' => 'sP: Ping Scan - go no further than determining if host is online'
				)
			)
		),
		'Unique checks' => array(
			'cmdb.unique.object-title' => array(
				'title' => 'LC__UNIVERSAL__OBJECT_TITLE',
				'type' => 'select',
				'options' => array(
					'0' => 'LC__UNIVERSAL__NO',
					'1' => 'LC__UNIVERSAL__YES'
				),
			),
			'cmdb.unique.layer-2-net' => array(
				'title' => 'LC__REPORT__VIEW__LAYER2_NETS__TITLE',
				'type' => 'select',
				'options' => array(
					'0' => 'LC__UNIVERSAL__NO',
					'1' => 'LC__UNIVERSAL__YES'
				),
			),
			'cmdb.unique.ip-address' => array(
				'title' => 'LC__REPORT__VIEW__LAYER2_NETS__IP_ADDRESSES',
				'type' => 'select',
				'options' => array(
					'0' => 'LC__UNIVERSAL__NO',
					'1' => 'LC__UNIVERSAL__YES'
				),
			),
			'cmdb.unique.hostname' => array(
				'title' => 'Hostname',
				'type' => 'select',
				'options' => array(
					'0' => 'LC__UNIVERSAL__NO',
					'1' => 'LC__UNIVERSAL__YES'
				),
			)
		),
		'Barcodes' => array(
			// C__BARCODE_TYPE
			'barcode.enabled' => array(
				'title' => 'LC__SYSTEM_SETTINGS__TENANT__BARCODE_ENABLED',
				'type' => 'select',
				'options' => array(
					'0' => 'LC__UNIVERSAL__NO',
					'1' => 'LC__UNIVERSAL__YES'
				),
				'default' => '1'
			),
			'barcode.type' => array(
				'title' => 'LC__SYSTEM_SETTINGS__TENANT__BARCODE_FORM',
				'type' => 'select',
				'options' => array(
					'qr' => 'QR-Code',
					'code39' => 'Code39'
				),
				'default' => 'qr'
			)
		),
		'LC__SYSTEM_SETTINGS__TENANT__GUI' => array(
			// C__GUI_VALUE__NA
			'gui.empty_value' => array(
				'title' => 'LC__SYSTEM_SETTINGS__TENANT__GUI__EMPTY_VALUES',
				'type' => 'text',
				'placeholder' => '-',
				'default' => '-'
			),
			// C_CMDB_LOCATION_SEPARATOR
			'gui.separator.location' => array(
				'title' => 'LC__SYSTEM_SETTINGS__TENANT__GUI__LOCATION_SEPARATOR',
				'type' => 'text',
				'placeholder' => ' > ',
				'default' => ' > '
			),
			// C_CMDB_CONNECTOR_SEPARATOR
			'gui.separator.connector' => array(
				'title' => 'LC__SYSTEM_SETTINGS__TENANT__GUI__CONNECTOR_SEPARATOR',
				'type' => 'text',
				'placeholder' => ' > ',
				'default' => ' > '
			),
		),
		'LC__SYSTEM_SETTINGS__TENANT__MAXLENGTH' => array(
			// C__DIALOG_PLUS__MAXLENGTH
			'maxlength.dialog_plus' => array(
				'title' => 'Dialog-Plus',
				'type' => 'text',
				'placeholder' => '110',
				'default' => '110'
			),
			// C__LIST__TITLE__MAXLEN
			'maxlength.object.lists' => array(
				'title' => 'LC__SYSTEM__SETTINGS__TENANT__MAXLENGTH_OBJECT_TITLE',
				'type' => 'text',
				'placeholder' => '55',
				'default' => '55'
			),
			// C__LIST__LOCATION__OBJLEN
			'maxlength.location.objects' => array(
				'title' => 'LC__SYSTEM__SETTINGS__TENANT__MAXLENGTH_OBJECTS_IN_TREE',
				'type' => 'text',
				'placeholder' => '16',
				'default' => '16'
			),
			// C__LIST__LOCATION__MAXLEN
			'maxlength.location.path' => array(
				'title' => 'LC__SYSTEM__SETTINGS__TENANT__MAXLENGTH_LOCATION_PATH',
				'type' => 'text',
				'placeholder' => '40',
				'default' => '40'
			),
			// C__FILTER__LENGTH
			'maxlength.search.filter' => array(
				'title' => 'LC__SYSTEM__SETTINGS__TENANT__MAXLENGTH_SEARCH_FILTER',
				'type' => 'text',
				'placeholder' => '3',
				'default' => '3'
			),
		),
		'LC__SYSTEM_SETTINGS__TENANT__LOGBOOK' => array(
			// C__SAVE_DETAILED_CMDB_CHANGES
			'logbook.changes' => array(
				'title' => 'LC__SYSTEM_SETTINGS__TENANT__LOGBOOK__LOGGING',
				'type' => 'select',
				'options' => array(
					'1' => 'LC__UNIVERSAL__YES',
					'0' => 'LC__UNIVERSAL__NO'
				),
				'default' => '1'
			),
		),
		'LC__SYSTEM_SETTINGS__TENANT__SECURITY' => array(
			'minlength.login.password' => array(
				'title' => 'LC__SYSTEM_SETTINGS__TENANT__SECURITY__PASSWORD_MINLENGTH',
				'type' => 'text',
				'placeholder' => '4',
				'default' => 4
			)
		),
		'Logging' => array(
			'logging.system.exceptions' => array(
				'title' => 'Exception Log',
				'type' => 'select',
				'options' => array(
					'0' => 'LC__UNIVERSAL__NO',
					'1' => 'LC__UNIVERSAL__YES'
				),
				'description' => 'LC__SYSTEM_SETTINGS__SYSTEM__LOGGING_ENABLED'
			),
		),
		'Quickinfo (Link mouseover)' => array(
			'cache.quickinfo.expiration' => array(
				'title' => 'LC__SYSTEM_SETTINGS__TENANT__QUICKINFO_EXPIRATION',
				'type' => 'select',
				'default' => isys_convert::DAY,
				'options' => array(
					isys_convert::MINUTE => 'LC__UNIVERSAL__MINUTE',
					isys_convert::HOUR => 'LC__UNIVERSAL__HOUR',
					isys_convert::DAY => 'LC__UNIVERSAL__DAY'
				),
				'description' => 'LC__SYSTEM_SETTINGS__TENANT__QUICKINFO_EXPIRATION__DESCRIPTION'
			)
		)
	);


	/**
	 * Load cache.
	 *
	 * @static
	 * @param   isys_component_database  $p_database
	 * @param   integer                  $p_tenant
	 * @param   boolean                  $p_reset_dao
	 * @return  void
	 */
	public static function initialize (isys_component_database $p_database, $p_tenant = null, $p_reset_dao = false)
	{
		isys_component_signalcollection::get_instance()->connect('system.shutdown', array('isys_tenantsettings', 'shutdown'));

		if (!is_object(self::$m_dao) || $p_reset_dao)
		{
			self::$m_dao = new isys_component_dao_tenant_settings($p_database, $p_tenant);
		} // if

		$l_cache_dir = static::get_cache_dir();

		// Generate cache and load settings.
		if ($l_cache_dir)
		{
			try
			{
				if (!file_exists($l_cache_dir . static::$m_cachefile))
				{
					static::regenerate();
				}
				else
				{
					static::load_cache($l_cache_dir);
				} // if
			}
			catch (Exception $e)
			{
                if (isys_application::instance()->container['logger'])
                {
                    isys_application::instance()->container['logger']->addError($e->getMessage());
                }

				// Load settings from database instead of cache.
				if (!static::$m_settings)
				{
					static::$m_settings = static::$m_dao->get_settings();
				} // if
			} // try
		} // if

		static::$m_initialized = true;
	} // function


	/**
	 * Set a setting value.
	 *
	 * @param  string   $p_key
	 * @param  mixed    $p_value
	 */
	public static function set ($p_key, $p_value)
	{
		self::$m_changed = true;

		parent::set($p_key, $p_value);
	} // function


    /**
     * @param $p_settings
     */
    public static function extend($p_settings)
    {
        self::$m_definition += $p_settings;
    }

	/**
	 * Method for retrieving the cache directory.
	 *
	 * @static
	 * @throws  Exception
	 * @return  string
	 * @author  Leonard Fischer <lfischer@i-doit.com>
	 */
	protected static function get_cache_dir()
	{
		if (static::$m_cache_dir !== NULL)
		{
			return static::$m_cache_dir;
		} // if

		global $g_absdir, $g_mandator_info;

        if (is_object(isys_application::instance()->session))
        {
            if (isys_application::instance()->session->is_logged_in())
            {
                if (!isset($g_mandator_info["isys_mandator__dir_cache"]) && !$g_mandator_info["isys_mandator__dir_cache"])
                {
                    throw new Exception('Error: Cache directory in $g_mandator_info not set.');
                } // if

                return static::$m_cache_dir = $g_absdir . DS . 'temp' . DS . $g_mandator_info["isys_mandator__dir_cache"] . DS;
            }
            else
            {
                throw new Exception('Tenantsettings are only available after logging in.');
            } // if
        } // if

        return false;
	} // function


	/**
	 * Before destructing the usersettings we want to save the data.
	 *
	 * @author  Leonard Fischer <lfischer@i-doit.com>
	 */
	public static function shutdown()
	{
		if (self::$m_changed)
		{
			self::force_save();
		} // if
	} // function
} // class