<?php
namespace idoit\Module\Cmdb\Search\QueryExtension;

use idoit\Module\Cmdb\Model\CiType;
use idoit\Module\Cmdb\Model\CiTypeCache;
use idoit\Module\Search\Query\AbstractQueryResultItem;
use idoit\Module\Search\Query\Condition;
use idoit\Module\Search\Query\Protocol\QueryResultItem as QueryResultItemProtocol;

/**
 * i-doit
 *
 * Default query result item
 *
 * @package     i-doit
 * @subpackage  Modules
 * @author      Dennis Stücken <dstuecken@i-doit.com>
 * @version     1.7
 * @copyright   synetics GmbH
 * @license     http://www.gnu.org/licenses/agpl-3.0.html GNU AGPLv3
 */
class QueryResultItem
    extends AbstractQueryResultItem
    implements QueryResultItemProtocol, \JsonSerializable
{
    /**
     * @var string
     */
    protected $type = 'cmdb';

    /**
     * @var array
     */
    private $keys;

    /**
     * @var int
     */
    private $categoryId;

    /**
     * @var CiType[]
     */
    private static $ciTypeCache = NULL;

    /**
     * @return string
     */
    public function getKey()
    {
        if (!self::$ciTypeCache)
        {
            self::$ciTypeCache = CiTypeCache::instance(\isys_application::instance()->database)->getCiTypes();
        }

        return ucwords(
            (is_numeric($this->keys[0]) && isset(self::$ciTypeCache[$this->keys[0]]) ? self::$ciTypeCache[$this->keys[0]] . ' > ' : '') .
            str_replace('_', ' ', $this->keys[1])
        ) .
        ' > ' .
        str_replace(
            '_', ' ', ucwords($this->keys[3])
        );
    }

    /**
     * @return string
     */
    public function getLink()
    {
        $categoryAddition = '';
        $class            = 'isys_cmdb_dao_category_g_' . $this->keys[1];

        if (!class_exists($class))
        {
            $class = 'isys_cmdb_dao_category_s_' . $this->keys[1];
            if (!class_exists($class))
            {
                $class = false;
            }
        }
        if ($class)
        {
            /**
             * @var $instance \isys_cmdb_dao_category
             */
            $instance = call_user_func([$class, 'instance'], \isys_application::instance()->database);
            $categoryAddition = '&'.$instance->get_category_type_abbr().'ID=' . $instance->get_category_id() . '&cateID=' . $this->categoryId;
        }

        if (!$categoryAddition)
        {
            if (defined('C__CATG__' . strtoupper($this->keys[1])))
            {
                if ($this->keys[1] == 'custom_fields')
                {
                    $categoryAddition = '&catgID=' . constant('C__CATG__' . strtoupper($this->keys[1])) . '&cateID=' . $this->categoryId;
                }
                else
                {
                    $categoryAddition = '&catgID=' . constant('C__CATG__' . strtoupper($this->keys[1])) . '&cateID=' . $this->categoryId;
                }
            }
            elseif (defined('C__CATS__' . strtoupper($this->keys[1])))
            {
                $categoryAddition = '&catsID=' . constant('C__CATS__' . strtoupper($this->keys[1])) . '&cateID=' . $this->categoryId;
            }
            elseif ($this->keys[1] === 'network_port')
            {
                $categoryAddition = '&catgID=' . C__CMDB__SUBCAT__NETWORK_PORT . '&cateID=' . $this->categoryId;
            }
            elseif ($this->keys[1] === 'storage_device')
            {
                $categoryAddition = '&catgID=' . C__CMDB__SUBCAT__STORAGE__DEVICE . '&cateID=' . $this->categoryId;
            }
            else $categoryAddition = '';
        }

        if (isset($this->conditions[0]))
        {
            $categoryAddition .= '&highlight=' . urlencode($this->getValue());
        }

        return rtrim(
            \isys_application::instance()->www_path,
            '/'
        ) . '/?objID=' . $this->getDocumentId() . $categoryAddition;
    }

    /**
     * JsonSerializable Interface
     *
     * @return array
     */
    public function jsonSerialize()
    {
        return [
            'documentId' => $this->getDocumentId(),
            'key'        => $this->getKey(),
            'value'      => $this->getValue(),
            'type'       => $this->getType(),
            'link'       => $this->getLink(),
            'score'      => $this->getScore()
        ];
    }

    /**
     * QueryResultItem constructor.
     *
     * @param int         $documentId
     * @param string      $key
     * @param string      $value
     * @param double      $score
     * @param Condition[] $conditions
     */
    public function __construct($documentId, $key, $value, $score, array $conditions)
    {
        parent::__construct($documentId, $key, $value, $score, $conditions);

        $this->keys       = explode('.', $this->key);
        $this->categoryId = $this->keys[2];
    }

}