<?php
namespace idoit\Module\Search\Index\Observer;

use idoit\Module\Cmdb\Model\Ci;
use idoit\Module\Search\Index\Protocol\Observer;
use idoit\Module\Search\Index\Protocol\Document;
use idoit\Module\Search\Index\Protocol\ObservableIndexManager;
use isys_application as Application;

/**
 * i-doit
 *
 * Json Observer
 *
 * Retrieves all indexed cis and writes them into a json file
 *
 * @package     i-doit
 * @subpackage  Modules
 * @author      Dennis Stücken <dstuecken@i-doit.com>
 * @version     1.7
 * @copyright   synetics GmbH
 * @license     http://www.gnu.org/licenses/agpl-3.0.html GNU AGPLv3
 */
class JsonDump implements Observer
{
    /**
     * The filename the json is written to
     *
     * @var string
     */
    private $filename;

    /**
     * The index
     *
     * @var array
     */
    private $index = [];

    /**
     * @param Document               $document
     * @param ObservableIndexManager $indexManager
     */
    public function update(Document $document, ObservableIndexManager $indexManager)
    {
        $this->index[] = $document;
    }

    /**
     * Json constructor.
     *
     * @param string $filename
     */
    public function __construct($filename = 'temp/objects.json')
    {
        if ($filename[0] !== '/')
        {
            $filename = Application::instance()->app_path . '/' . $filename;
        }

        $this->filename = $filename;
    }

    /**
     * Write file on destruction
     */
    public function __destruct()
    {
        file_put_contents($this->filename, json_encode($this->index));
    }

}