<?php
namespace idoit\Module\Search\Index\Traits;

use idoit\Module\Search\Index\Protocol\Observer;
use idoit\Module\Search\Index\Protocol\Document;
use idoit\Module\Search\Index\Protocol\ObservableIndexManager;

/**
 * i-doit
 *
 * Search index manager
 *
 * Uses observable pattern to inform several handles about the indexed ci's.
 *
 * @package     i-doit
 * @subpackage  Modules
 * @author      Dennis Stücken <dstuecken@i-doit.com>
 * @version     1.7
 * @copyright   synetics GmbH
 * @license     http://www.gnu.org/licenses/agpl-3.0.html GNU AGPLv3
 */
trait IndexObserverTrait
{
    /**
     * @var Observer[]
     */
    private $observer = [];

    /**
     * Amount of Cis to be indexed
     *
     * @var int
     */
    private $documentsToBeIndexed = 0;

    /**
     * Notify all observers
     *
     * @param Document $document
     */
    public function notify(Document $document, ObservableIndexManager $indexManager)
    {
        foreach ($this->observer as $observerItem)
        {
            $observerItem->update($document, $indexManager);
        }
    }

    /**
     * @return Observer[]
     */
    public function getObserver()
    {
        return $this->observer;
    }

    /**
     * @return int
     */
    public function getDocumentsToBeIndexed()
    {
        return $this->documentsToBeIndexed;
    }

    /**
     * @param Observer $observer
     *
     * @return $this
     */
    public function attach(Observer $observer)
    {
        $this->observer[] = $observer;

        return $this;
    }

    /**
     * @param Observer $observer
     *
     * @return $this
     */
    public function detach(Observer $observer)
    {
        foreach ($this->observer as $key => $observerItem)
        {
            if ($observerItem === $observer)
            {
                unset($this->observer[$key]);
            }
        }

        return $this;
    }
}