<?php
/**
 * i-doit - Updates
 *
 * This migration creates description field entries for custom categories, if they're not available.
 *
 * @package     i-doit
 * @subpackage  Update
 * @copyright   synetics GmbH
 * @license     http://www.gnu.org/licenses/agpl-3.0.html GNU AGPLv3
 * @author      Leonard Fischer <lfischer@i-doit.com>
 */

// Initialization
global $g_comp_database;

$g_migration_identifier = 'migration_custom_description';

if ($this->is_migration_done($g_migration_identifier))
{
	$g_migration_log[] = '<span class="bold">Adding description field entries for custom categories.</span>';
}
else
{
	try
	{
		$g_migration_log[] = '<span class="bold">Starting description field migration for custom categories.</span>';
		$l_dao = new isys_cmdb_dao_category_g_custom_fields($g_comp_database);

		// In this SQL we fetch all custom category data, that has no "commentary" field.
		$l_sql = 'SELECT DISTINCT isys_catg_custom_fields_list__isys_obj__id, isys_catg_custom_fields_list__isysgui_catg_custom__id, isys_catg_custom_fields_list__data__id
			FROM isys_catg_custom_fields_list
			WHERE
				CONCAT_WS("-", isys_catg_custom_fields_list__isys_obj__id, isys_catg_custom_fields_list__isysgui_catg_custom__id, isys_catg_custom_fields_list__data__id)
				NOT IN (
					SELECT DISTINCT CONCAT_WS("-", isys_catg_custom_fields_list__isys_obj__id, isys_catg_custom_fields_list__isysgui_catg_custom__id, isys_catg_custom_fields_list__data__id)
					FROM isys_catg_custom_fields_list
					WHERE isys_catg_custom_fields_list__field_key LIKE "C__CMDB__CAT__COMMENTARY_%");';

		$l_res = $l_dao->retrieve($l_sql);

		if (count($l_res))
		{
			$g_migration_log[] = '<span class="bold grey">Found ' . count($l_res) . ' entries with missing description fields.</span>';

			while ($l_row = $l_res->get_row())
			{
				$l_result = $l_dao->create(
					$l_row['isys_catg_custom_fields_list__isys_obj__id'], // Object ID.
					$l_row['isys_catg_custom_fields_list__isysgui_catg_custom__id'], // Custom category ID.
					'C__CMDB__CAT__COMMENTARY_' . C__CMDB__CATEGORY__TYPE_CUSTOM . $l_row['isys_catg_custom_fields_list__isysgui_catg_custom__id'], // Key.
					'', // Value.
					'commentary', // Type.
					C__RECORD_STATUS__NORMAL,
					'', // Description.
					$l_row['isys_catg_custom_fields_list__data__id']);

				if ($l_result === false)
				{
					$g_migration_log[] = '<span class="indent red">Failure for object ID #' . $l_row['isys_catg_custom_fields_list__isys_obj__id'] .
						', custom category ID #' . $l_row['isys_catg_custom_fields_list__isysgui_catg_custom__id'] .
						', data ID #' . $l_row['isys_catg_custom_fields_list__data__id'] . '</span>';
				} // if
			} // while
		}
		else
		{
			$g_migration_log[] = '<span class="indent">No missing description fields found.</span>';
		} // if

		// Set flag in db.
		$this->migration_done($g_migration_identifier);

		$g_migration_log[] = '<span class="bold">Migration successful.</span>';
	}
	catch (Exception $e)
	{
		$g_migration_log[] = '<span class="bold red">' . $e->getMessage() . '</span>';
	} // try
} // if