<?php
/**
 * i-doit - Updates
 * This migration rebuilds the missing relations for the category location
 *
 * @package     i-doit
 * @subpackage  Update
 * @copyright   synetics GmbH
 * @license     http://www.gnu.org/licenses/agpl-3.0.html GNU AGPLv3
 * @author      Van Quyen Hoang <qhoang@i-doit.com>
 */

// Initialization
global $g_comp_database, $g_comp_database_system;

$g_migration_identifier = "migration_rebuild_location_relations";

if ($this->is_migration_done($g_migration_identifier))
{
	$g_migration_log[] = "<span class=\"bold grey indent\">Rebuilding location relation Migration has already been executed.</span>";
}
else
{
	try
	{
		$l_dao = new isys_cmdb_dao_location($g_comp_database);

		$l_sql = 'SELECT * FROM isys_catg_location_list
				WHERE isys_catg_location_list__isys_catg_relation_list__id IS NULL AND
				isys_catg_location_list__parentid IS NOT NULL AND isys_catg_location_list__isys_obj__id IS NOT NULL';
		$l_res = $l_dao->retrieve($l_sql);

		if ($l_res->num_rows() > 0)
		{
			$g_migration_log[] = '<span class="bold">Start rebuilding location relations of ' . count($l_res) . ' entries ...</span>';
			if(!$l_dao->regenerate_missing_relation_objects())
			{
				$g_migration_log[] = "<span class=\"bold red\">Migration failed.</span>";
				$g_migration_log[] = "<span class=\"bold red\">Please manually trigger the \"Correction of locations\" in Administration -> System tools -> Cache/Database.</span>";
			} // if

			$l_dao->apply_update();
		}
		else
		{
			$g_migration_log[] = '<span class="bold">No rows to migrate :)</span>';
		} // if

		// Set flag in db.
		$this->migration_done($g_migration_identifier);

		$g_migration_log[] = "<span class=\"bold\">Migration successful.</span>";
	}
	catch (Exception $e)
	{
		$g_migration_log[] = "<span class=\"bold red\">" . $e->getMessage() . "</span>";
	} // try
} // if