<?php
/**
 * i-doit - Updates
 * This migration rebuilds the missing relations for the category location
 *
 * @package     i-doit
 * @subpackage  Update
 * @copyright   synetics GmbH
 * @license     http://www.gnu.org/licenses/agpl-3.0.html GNU AGPLv3
 * @author      Van Quyen Hoang <qhoang@i-doit.com>
 */

// Initialization
global $g_comp_database_system, $g_comp_database;

$g_migration_identifier = "migration_registry";

if ($this->is_migration_done($g_migration_identifier))
{
    $g_migration_log[] = '<span class="bold grey indent">Registry migration has already been executed.</span>';
}
else
{
    try
    {
        if ($g_comp_database_system->num_rows($g_comp_database_system->query('SHOW TABLES LIKE "isys_registry";')) > 0)
        {
            $g_migration_log[] = '<span class="bold">Starting registry migration.</span>';

            // First we need to get the tenant ID.
            $l_row = $g_comp_database_system->fetch_array(
                $g_comp_database_system->query(
                    'SELECT isys_mandator__id FROM isys_mandator WHERE isys_mandator__db_name = "' . $g_comp_database->get_db_name() . '";'
                )
            );

            if ($l_row['isys_mandator__id'] > 0)
            {
                isys_tenantsettings::initialize($g_comp_database_system, $l_row['isys_mandator__id'], true);

                $l_comp_registry = new isys_component_dao_registry(
                    $g_comp_database_system,
                    "isys_registry",
                    "isys_registry__id",
                    "isys_registry__parentid",
                    "isys_registry__key",
                    "isys_registry__value",
                    "isys_registry__deletable",
                    "isys_registry__editable"
                );

                $l_mapping = [
                    'int_mytask_entries'          => 'cmdb.registry.mytask_entries',
                    'b_always_display_full_lists' => 'cmdb.registry.show_full_lists',
                    'b_showQuickSaveButton'       => 'cmdb.registry.quicksave',
                    'SYSID__READONLY'             => 'cmdb.registry.sysid_readonly',
                    'OBJECT_DRAGNDROP'            => 'cmdb.registry.object_dragndrop',
                    'C__OBJTYPE__SORTING'         => 'cmdb.registry.object_type_sorting',
                    'C__CMDB__SANITIZING'         => 'cmdb.registry.sanitize_input_data',
                    'LOCK__DATASETS'              => 'cmdb.registry.lock_dataset',
                    'LOCK__TIMEOUT'               => 'cmdb.registry.lock_timeout',
                    'API__STATUS'                 => 'api.status',
                    'int_guestID'                 => 'session.guest_id',
                    'int_notify'                  => 'workflow.notify',
                    'TEMPLATE__COLORS'            => 'cmdb.template.colors',
                    'TEMPLATE__COLOR_VALUE'       => 'cmdb.template.color_value',
                    'TEMPLATE__STATUS'            => 'cmdb.template.status',
                    'TEMPLATE__SHOW_ASSIGNMENTS'  => 'cmdb.template.show_assignments',
                    'i_defaultInventoryStatus'    => 'import.hinventory.default_status',
                    'i_showStatusFilter'          => 'system.mydoit.show_filter',
                    'INFOBOX__LENGTH'             => 'gui.infobox.length',
                    'INFOBOX__DATEFORMAT'         => 'gui.infobox.dateformat',
                    'OCS__DEFAULT_OBJTYPE'        => 'ocs.default.objtype',
                    'OCS__SERVER_PREFIX'          => 'ocs.prefix.server',
                    'OCS__CLIENT_PREFIX'          => 'ocs.prefix.client',
                    'OCS__APPLICATION'            => 'ocs.application',
                    'OCS__ROUTER_PREFIX'          => 'ocs.prefix.router',
                    'OCS__SWITCH_PREFIX'          => 'ocs.prefix.switch',
                    'OCS__PRINTER_PREFIX'         => 'ocs.prefix.printer',
                    'OCS__APPLICATION_ASSIGNMENT' => 'ocs.application.assignment',
                    'OCS__LOGBOOK_ACTIVE'         => 'ocs.logbook.active',
                    'OCS__DEFAULT_DB'             => 'ocs.default.db',
                ];

                foreach ($l_mapping as $l_registry => $l_tenantsetting)
                {
                    isys_tenantsettings::set(
                        $l_tenantsetting,
                        $l_comp_registry->get_by_key($l_registry)
                                        ->get_row_value('isys_registry__value')
                    );
                    $g_migration_log[] = '<span class="indent">Migrating "' . $l_registry . '" to "' . $l_tenantsetting . '".</span>';
                } // foreach

                isys_tenantsettings::force_save();
            }
        }
        else
        {
            $g_migration_log[] = '<span class="bold">Migration not necessary...</span>';
        } // if

        // Set flag in db.
        $this->migration_done($g_migration_identifier);

        $g_migration_log[] = '<span class="bold">Migration successful.</span>';
    }
    catch (Exception $e)
    {
        $g_migration_log[] = '<span class="bold red">' . $e->getMessage() . '</span>';
    } // try
} // if