<?php
/**
 * i-doit - Updates
 *
 * Quick search indexing
 *
 * @package     i-doit
 * @subpackage  Update
 * @copyright   synetics GmbH
 * @license     http://www.gnu.org/licenses/agpl-3.0.html GNU AGPLv3
 * @author      Dennis Stücken <dstuecken@i-doit.com,>
 */

/**
 * @var $g_comp_database isys_component_database
 */
global $g_comp_database;

$g_migration_identifier = "migration_search_index";

if ($this->is_migration_done($g_migration_identifier))
{
    $g_migration_log[] = "<span class=\"bold grey indent\">Search index already created.</span>";
}
else
{
    $g_migration_log[] = '<span class="bold">Creating a search index for a full-text search experience</span>';

    require_once isys_application::instance()->app_path . '/src/classes/modules/search/init.php';

    $onelineIndexer = new \idoit\Module\Cmdb\Search\IndexExtension\OnelineIndexer($g_comp_database);
    $dao            = new isys_cmdb_dao($g_comp_database);

    $l_condition = "AND isys_obj__isys_obj_type__id NOT IN (" .
     "(SELECT isys_obj_type__id FROM isys_obj_type WHERE isys_obj_type__const = 'C__OBJTYPE__RELATION'), " .
     "(SELECT isys_obj_type__id FROM isys_obj_type WHERE isys_obj_type__const = 'C__OBJTYPE__PARALLEL_RELATION'), " .
     "(SELECT isys_obj_type__id FROM isys_obj_type WHERE isys_obj_type__const = 'C__OBJTYPE__NAGIOS_SERVICE'), " .
     "(SELECT isys_obj_type__id FROM isys_obj_type WHERE isys_obj_type__const = 'C__OBJTYPE__NAGIOS_HOST_TPL'), " .
     "(SELECT isys_obj_type__id FROM isys_obj_type WHERE isys_obj_type__const = 'C__OBJTYPE__NAGIOS_SERVICE_TPL') " .
     ")";

    if ($dao->count_objects(NULL, NULL, false, $l_condition) > 200000)
    {
        // Indexing some basic properties via SQL only
        $onelineIndexer->indexObjectTitles()
                       ->indexInventoryNumber()
                       ->indexInvoiceNumber()
                       ->indexIpAddresses([])
                       ->indexMacAddress()
                       ->indexOrderNumber()
                       ->indexPortTitle()
                       ->indexSerialNumbers();

        $g_migration_log[] = '<span class="bold indent">Index successfully created.</span>';
    }
    else
    {
        $onelineIndexer->index();

        isys_application::instance()->logger->addInfo('Index successfully created: ' . $onelineIndexer->getIndexedItems() . ' attributes within ' . $onelineIndexer->getIndexedDocuments() . ' categories indexed');
        $g_migration_log[] = '<span class="bold indent">Index successfully created: ' . $onelineIndexer->getIndexedItems() . ' attributes within ' . $onelineIndexer->getIndexedDocuments() . ' categories indexed</span>';
    }

    // Set flag in DB.
    $this->migration_done($g_migration_identifier);

}