<?php
/**
 * i-doit - Updates
 * This migration will translate the old CSV identificators to the new ones!
 *
 * @package     i-doit
 * @subpackage  Update
 * @copyright   synetics GmbH
 * @license     http://www.gnu.org/licenses/agpl-3.0.html GNU AGPLv3
 * @author      Leonard Fischer <lfischer@i-doit.com>
 */

// Initialization
use idoit\Module\Cmdb\Model\Ci\Table\Config;
use idoit\Module\Cmdb\Model\Ci\Table\Property;

global $g_comp_database, $g_migration_log;

$g_migration_identifier = "object_type_lists";

if (!function_exists('convertOldConfig'))
{
    function convertOldConfig($jsonString, $rowClickable, $sortByProp, $sortDirection)
    {
        global $g_comp_database, $g_migration_log;

        if (!isys_format_json::is_json_array($jsonString))
        {
            return null;
        } // if

        $jsonConfig = [];
        $oldConfig  = isys_format_json::decode($jsonString);

        $tableConfig = (new Config())->setGroupingType(isys_cmdb_dao_category_property_ng::C__GROUPING__LIST)
            ->setRowClickable(!!$rowClickable);

        if ($sortByProp > 0)
        {
            $property = isys_cmdb_dao_category_property::factory($g_comp_database)
                ->retrieve_properties($sortByProp)
                ->get_row();

            $tableConfig->setSortingProperty($property['class'] . '__' . $property['key'])
                ->setSortingDirection($sortDirection ?: 'ASC');
        } // if

        foreach ($oldConfig as $oldProperty)
        {
            $propKey = ltrim($oldProperty[1], '_');

            list ($catClass, $propertyMethod) = explode('::', $oldProperty[4]);

            if (class_exists($catClass))
            {
                /**
                 * @var  isys_cmdb_dao_category $dao
                 */
                $dao = isys_factory::get_instance($catClass, $g_comp_database);

                $customCatID = null;

                if ($catClass == 'isys_cmdb_dao_category_g_custom_fields')
                {
                    $customCatID = explode('###', $oldProperty[2])[1];
                    $dao->set_catg_custom_id($customCatID);
                }

                $property = $dao->get_property_by_key($propKey);

                if (!$property)
                {
                    $g_migration_log[] = '<span class="indent text-grey">Property "' . _L($oldProperty[6]) . ' >> ' . _L($oldProperty[3]) . '" does not exist anymore.</span>';
                    continue;
                } // if

                if (!$property[C__PROPERTY__PROVIDES][C__PROPERTY__PROVIDES__LIST])
                {
                    $g_migration_log[] = '<span class="indent text-grey">Property "' . _L($oldProperty[6]) . ' >> ' . _L($oldProperty[3]) .
                        '" is not allowed to be displayed in the list.</span>';
                    continue;
                }

                // Basically we save the JSON 1:1 but change all dynamic properties to static.
                $jsonConfig[] = [
                    1,
                    $propKey,
                    $oldProperty[2],
                    $oldProperty[3],
                    $catClass . '::get_properties',
                    $oldProperty[5],
                    $oldProperty[6],
                    $oldProperty[7]
                ];

                $tableConfig->addProperty(
                    new Property(
                        $catClass,
                        $propKey,
                        $oldProperty[6],
                        $oldProperty[3],
                        $property[C__PROPERTY__DATA][C__PROPERTY__DATA__INDEX],
                        $customCatID,
                        $property[C__PROPERTY__INFO][C__PROPERTY__INFO__TYPE]
                    )
                );
            }
        } // foreach

        return [
            'config'       => $jsonConfig,
            'table_config' => $tableConfig
        ];
    }
} // if

if ($this->is_migration_done($g_migration_identifier))
{
    $g_migration_log[] = '<span class="bold grey indent">Object type lists have already been migrated.</span>';
}
else
{
    try
    {
        $baseConfig = [
            'C__CMDB__OBJTYPE__CABLE_TRAY' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"created","isys_obj__created","LC__TASK__DETAIL__WORKORDER__CREATION_DATE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__created"],[1,"changed","isys_obj__updated","LC__CMDB__LAST_CHANGE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__changed"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__CMDB__OBJTYPE__CONDUIT' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"created","isys_obj__created","LC__TASK__DETAIL__WORKORDER__CREATION_DATE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__created"],[1,"changed","isys_obj__updated","LC__CMDB__LAST_CHANGE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__changed"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJECT_TYPE__GROUP' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__ACCESS_POINT' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"ssid","isys_cats_access_point_list__ssid","LC__CMDB__CATS__ACCESS_POINT_SSID","isys_cmdb_dao_category_s_access_point::get_properties",false,"LC__CMDB__CATS__ACCESS_POINT",null,"isys_cmdb_dao_category_s_access_point__ssid"],[1,"manufacturer","isys_model_manufacturer__title","LC__CATG__STORAGE_MANUFACTURER","isys_cmdb_dao_category_g_model::get_properties",false,"LC__CMDB__CATG__MODEL",null,"isys_cmdb_dao_category_g_model__manufacturer"],[1,"title","isys_model_title__title","LC__CMDB__CATG__MODEL","isys_cmdb_dao_category_g_model::get_properties",false,"LC__CMDB__CATG__MODEL",null,"isys_cmdb_dao_category_g_model__title"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__AIRCRAFT' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"created","isys_obj__created","LC__TASK__DETAIL__WORKORDER__CREATION_DATE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__created"],[1,"changed","isys_obj__updated","LC__CMDB__LAST_CHANGE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__changed"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__AIR_CONDITION_SYSTEM' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"manufacturer","isys_model_manufacturer__title","LC__CATG__STORAGE_MANUFACTURER","isys_cmdb_dao_category_g_model::get_properties",false,"LC__CMDB__CATG__MODEL",null,"isys_cmdb_dao_category_g_model__manufacturer"],[1,"title","isys_model_title__title","LC__CMDB__CATG__MODEL","isys_cmdb_dao_category_g_model::get_properties",false,"LC__CMDB__CATG__MODEL",null,"isys_cmdb_dao_category_g_model__title"],[1,"type","isys_ac_type__title","LC__CATS__AC_TYPE","isys_cmdb_dao_category_s_ac::get_properties",false,"LC__CMDB__CATS__AC",null,"isys_cmdb_dao_category_s_ac__type"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__AMPLIFIER' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__APPLIANCE' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"all_ips","isys_catg_ip_list__isys_cats_net_ip_addresses_list__id","LC__CMDB__CATG__CLUSTER_SERVICE__HOST_ADDRESSES","isys_cmdb_dao_category_g_ip::get_properties",false,"LC__CATG__IP_ADDRESS",null,"isys_cmdb_dao_category_g_ip__all_ips"],[1,"primary_hostname","isys_catg_ip_list__hostname","LC__CATP__IP__PRIMARY_HOSTNAME","isys_cmdb_dao_category_g_ip::get_properties",false,"LC__CATG__IP_ADDRESS",null,"isys_cmdb_dao_category_g_ip__primary_hostname"],[1,"manufacturer","isys_model_manufacturer__title","LC__CATG__STORAGE_MANUFACTURER","isys_cmdb_dao_category_g_model::get_properties",false,"LC__CMDB__CATG__MODEL",null,"isys_cmdb_dao_category_g_model__manufacturer"],[1,"title","isys_model_title__title","LC__CMDB__CATG__MODEL","isys_cmdb_dao_category_g_model::get_properties",false,"LC__CMDB__CATG__MODEL",null,"isys_cmdb_dao_category_g_model__title"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__APPLICATION' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"manufacturer","isys_application_manufacturer__title","LC__CMDB__CATG__MANUFACTURE","isys_cmdb_dao_category_s_application::get_properties",false,"LC__CMDB__CATS__APPLICATION",null,"isys_cmdb_dao_category_s_application__manufacturer"],[1,"installation_count",null,"LC__CMDB__CATS__APPLICATION_INSTALLATION_COUNT","isys_cmdb_dao_category_s_application::get_properties",false,"LC__CMDB__CATS__APPLICATION",null,"isys_cmdb_dao_category_s_application__installation_count"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__BLADE_CHASSIS' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"title","isys_cats_chassis_slot_list__title","LC__CMDB__CATG__GLOBAL_TITLE","isys_cmdb_dao_category_s_chassis_slot::get_properties",false,"LC__CMDB__CATS__CHASSIS_SLOTS",null,"isys_cmdb_dao_category_s_chassis_slot__title"],[1,"assigned_devices","isys_cats_chassis_slot_list__id","LC__CMDB__CATS__CHASSIS__ASSIGNED_DEVICES","isys_cmdb_dao_category_s_chassis_slot::get_properties",false,"LC__CMDB__CATS__CHASSIS_SLOTS",null,"isys_cmdb_dao_category_s_chassis_slot__assigned_devices"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__BLADE_SERVER' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"manufacturer","isys_model_manufacturer__title","LC__CATG__STORAGE_MANUFACTURER","isys_cmdb_dao_category_g_model::get_properties",false,"LC__CMDB__CATG__MODEL",null,"isys_cmdb_dao_category_g_model__manufacturer"],[1,"title","isys_model_title__title","LC__CMDB__CATG__MODEL","isys_cmdb_dao_category_g_model::get_properties",false,"LC__CMDB__CATG__MODEL",null,"isys_cmdb_dao_category_g_model__title"],[1,"primary_hostaddress","isys_cats_net_ip_addresses_list__title","LC__CATG__IP__PRIMARY_ADDRESS","isys_cmdb_dao_category_g_ip::get_properties",false,"LC__CATG__IP_ADDRESS",null,"isys_cmdb_dao_category_g_ip__primary_hostaddress"],[1,"primary_hostname","isys_catg_ip_list__hostname","LC__CATP__IP__PRIMARY_HOSTNAME","isys_cmdb_dao_category_g_ip::get_properties",false,"LC__CATG__IP_ADDRESS",null,"isys_cmdb_dao_category_g_ip__primary_hostname"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__BUILDING' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"street","isys_catg_address_list__street","LC__CONTACT__ORGANISATION_STREET","isys_cmdb_dao_category_g_address::get_properties",false,"LC__CATG__ADDRESS",null,"isys_cmdb_dao_category_g_address__street"],[1,"postcode","isys_catg_address_list__postalcode","LC__CONTACT__ORGANISATION_POSTAL_CODE","isys_cmdb_dao_category_g_address::get_properties",false,"LC__CATG__ADDRESS",null,"isys_cmdb_dao_category_g_address__postcode"],[1,"city","isys_catg_address_list__city","LC__CONTACT__ORGANISATION_CITY","isys_cmdb_dao_category_g_address::get_properties",false,"LC__CATG__ADDRESS",null,"isys_cmdb_dao_category_g_address__city"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__CABLE' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"connection","isys_catg_cable_list__id__isys_obj__id","LC__CATS__CABLE__CONNECTION","isys_cmdb_dao_category_g_cable::get_properties",false,"LC__CATG__CABLE",null,"isys_cmdb_dao_category_g_cable__connection"],[1,"cable_type","isys_cable_type__title","LC__CMDB__CATS__CABLE__TYPE","isys_cmdb_dao_category_g_cable::get_properties",false,"LC__CATG__CABLE",null,"isys_cmdb_dao_category_g_cable__cable_type"],[1,"length","isys_catg_cable_list__length","LC__CMDB__CATS__CABLE__LENGTH","isys_cmdb_dao_category_g_cable::get_properties",false,"LC__CATG__CABLE",null,"isys_cmdb_dao_category_g_cable__length"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__CELL_PHONE_CONTRACT' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"manufacturer","isys_model_manufacturer__title","LC__CATG__STORAGE_MANUFACTURER","isys_cmdb_dao_category_g_model::get_properties",false,"LC__CMDB__CATG__MODEL",null,"isys_cmdb_dao_category_g_model__manufacturer"],[1,"title","isys_model_title__title","LC__CMDB__CATG__MODEL","isys_cmdb_dao_category_g_model::get_properties",false,"LC__CMDB__CATG__MODEL",null,"isys_cmdb_dao_category_g_model__title"],[1,"serial","isys_catg_model_list__serial","LC__CMDB__CATG__SERIAL","isys_cmdb_dao_category_g_model::get_properties",false,"LC__CMDB__CATG__MODEL",null,"isys_cmdb_dao_category_g_model__serial"],[1,"imei_number","isys_cats_mobile_phone_list__imei_number","LC__CMDB__CATS__SIM_CARD__IMEI_NUMBER","isys_cmdb_dao_category_s_cp_contract::get_properties",false,"LC__CMDB__CATS__CELL_PHONE_CONTRACT",null,"isys_cmdb_dao_category_s_cp_contract__imei_number"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__CITY' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"created","isys_obj__created","LC__TASK__DETAIL__WORKORDER__CREATION_DATE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__created"],[1,"changed","isys_obj__updated","LC__CMDB__LAST_CHANGE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__changed"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__CLIENT' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"primary_contact","isys_connection__title","LC__CATG__CONTACT_PRIMARY","isys_cmdb_dao_category_g_contact::get_properties",false,"LC__CMDB__CATG__CONTACT",null,"isys_cmdb_dao_category_g_contact__primary_contact"],[1,"type","isys_client_type__title","LC__CMDB__CATS__CLIENT_TYPE","isys_cmdb_dao_category_s_client::get_properties",false,"LC__CMDB__CATS__CLIENT",null,"isys_cmdb_dao_category_s_client__type"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__CLUSTER' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"cluster_members","isys_catg_cluster_members_list__isys_obj__id","LC__CMDB__CATG__CLUSTER_MEMBERS","isys_cmdb_dao_category_g_cluster::get_properties",false,"LC__CMDB__CATG__CLUSTER",null,"isys_cmdb_dao_category_g_cluster__cluster_members"],[1,"cluster_service","isys_catg_cluster_service_list__isys_obj__id","LC__CMDB__CATG__CLUSTER_SERVICES","isys_cmdb_dao_category_g_cluster::get_properties",false,"LC__CMDB__CATG__CLUSTER",null,"isys_cmdb_dao_category_g_cluster__cluster_service"],[1,"administration_service","isys_catg_cluster_adm_service_list__isys_obj__id","LC__CMDB__CATG__CLUSTER__ADMINISTRATION_SERVICE","isys_cmdb_dao_category_g_cluster::get_properties",false,"LC__CMDB__CATG__CLUSTER",null,"isys_cmdb_dao_category_g_cluster__administration_service"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__CLUSTER_SERVICE' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"cluster","isys_catg_cluster_service_list__isys_obj__id","LC__CMDB__CATS__CLUSTER_SERVICE__ASSIGNED_CLUSTER","isys_cmdb_dao_category_s_cluster_service::get_properties",false,"LC__CMDB__CATS__CLUSTER_SERVICE__ASSIGNED_CLUSTER",null,"isys_cmdb_dao_category_s_cluster_service__cluster"],[1,"runs_on","isys_catg_cluster_service_list__id","LC__CMDB__CATG__CLUSTER_SERVICE__RUNS_ON","isys_cmdb_dao_category_s_cluster_service::get_properties",false,"LC__CMDB__CATS__CLUSTER_SERVICE__ASSIGNED_CLUSTER",null,"isys_cmdb_dao_category_s_cluster_service__runs_on"],[1,"type","isys_cluster_type__title","LC__CMDB__CATG__CLUSTER_SERVICE__TYPE","isys_cmdb_dao_category_s_cluster_service::get_properties",false,"LC__CMDB__CATS__CLUSTER_SERVICE__ASSIGNED_CLUSTER",null,"isys_cmdb_dao_category_s_cluster_service__type"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__CLUSTER_VRRP_HSRP' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"created","isys_obj__created","LC__TASK__DETAIL__WORKORDER__CREATION_DATE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__created"],[1,"changed","isys_obj__updated","LC__CMDB__LAST_CHANGE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__changed"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__CONVERTER' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__COUNTRY' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"created","isys_obj__created","LC__TASK__DETAIL__WORKORDER__CREATION_DATE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__created"],[1,"changed","isys_obj__updated","LC__CMDB__LAST_CHANGE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__changed"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__DATABASE_INSTANCE' => '[[1,"title","isys_obj__title","Name (Allgemein)","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"title","isys_cats_database_instance_list__title","Name (Instanz \/ Oracle Datenbank)","isys_cmdb_dao_category_s_database_instance::get_properties",false,"LC__CMDB__TREE__DATABASE_INSTANCE",null,"isys_cmdb_dao_category_s_database_instance__title"],[1,"listener","isys_cats_database_instance_list__listener","Listener","isys_cmdb_dao_category_s_database_instance::get_properties",false,"LC__CMDB__TREE__DATABASE_INSTANCE",null,"isys_cmdb_dao_category_s_database_instance__listener"],[1,"instance","isys_connection__title","DBMS","isys_cmdb_dao_category_s_database_instance::get_properties",false,"LC__CMDB__TREE__DATABASE_INSTANCE",null,"isys_cmdb_dao_category_s_database_instance__instance"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__DATABASE_SCHEMA' => '[[1,"title","isys_obj__title","Name (Allgemein)","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"storage_engine","isys_cats_database_schema_list__storage_engine","Storage-Engine","isys_cmdb_dao_category_s_database_schema::get_properties",false,"LC__OBJTYPE__DATABASE_SCHEMA",null,"isys_cmdb_dao_category_s_database_schema__storage_engine"],[1,"title","isys_cats_database_objects_list__title","Name (Datenbankobjekte)","isys_cmdb_dao_category_s_database_objects::get_properties",false,"LC__CMDB__TREE__DATABASE_OBJECTS",null,"isys_cmdb_dao_category_s_database_objects__title"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__DBMS' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"manufacturer","isys_application_manufacturer__title","LC__CMDB__CATG__MANUFACTURE","isys_cmdb_dao_category_s_application::get_properties",false,"LC__CMDB__CATS__APPLICATION",null,"isys_cmdb_dao_category_s_application__manufacturer"],[1,"installation_count",null,"LC__CMDB__CATS__APPLICATION_INSTALLATION_COUNT","isys_cmdb_dao_category_s_application::get_properties",false,"LC__CMDB__CATS__APPLICATION",null,"isys_cmdb_dao_category_s_application__installation_count"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__DISTRIBUTION_BOX' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__EMERGENCY_PLAN' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__ENCLOSURE' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"rackunits","isys_catg_formfactor_list__rackunits","LC__CMDB__CATG__RACKUNITS","isys_cmdb_dao_category_g_formfactor::get_properties",false,"LC__CMDB__CATG__FORMFACTOR",null,"isys_cmdb_dao_category_g_formfactor__rackunits"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"vertical_slots_front","isys_cats_enclosure_list__vertical_slots_front","LC__CMDB__CATS__ENCLOSURE__VERTICAL_SLOTS_FRONTSIDE","isys_cmdb_dao_category_s_enclosure::get_properties",false,"LC__CMDB__CATS__ENCLOSURE",null,"isys_cmdb_dao_category_s_enclosure__vertical_slots_front"],[1,"vertical_slots_rear","isys_cats_enclosure_list__vertical_slots_rear","LC__CMDB__CATS__ENCLOSURE__VERTICAL_SLOTS_BACKSIDE","isys_cmdb_dao_category_s_enclosure::get_properties",false,"LC__CMDB__CATS__ENCLOSURE",null,"isys_cmdb_dao_category_s_enclosure__vertical_slots_rear"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__EPS' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"manufacturer","isys_model_manufacturer__title","LC__CATG__STORAGE_MANUFACTURER","isys_cmdb_dao_category_g_model::get_properties",false,"LC__CMDB__CATG__MODEL",null,"isys_cmdb_dao_category_g_model__manufacturer"],[1,"title","isys_model_title__title","LC__CMDB__CATG__MODEL","isys_cmdb_dao_category_g_model::get_properties",false,"LC__CMDB__CATG__MODEL",null,"isys_cmdb_dao_category_g_model__title"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__ESC' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"volt","isys_catg_power_supplier_list__volt","LC__CMDB__CATG__POWER_SUPPLIER__VOLT","isys_cmdb_dao_category_g_power_supplier::get_properties",false,"LC__CMDB__CATG__POWER_SUPPLIER",null,"isys_cmdb_dao_category_g_power_supplier__volt"],[1,"watt","isys_catg_power_supplier_list__watt","LC__CMDB__CATG__POWER_SUPPLIER__WATT","isys_cmdb_dao_category_g_power_supplier::get_properties",false,"LC__CMDB__CATG__POWER_SUPPLIER",null,"isys_cmdb_dao_category_g_power_supplier__watt"],[1,"ampere","isys_catg_power_supplier_list__ampere","LC__CMDB__CATG__POWER_SUPPLIER__AMPERE","isys_cmdb_dao_category_g_power_supplier::get_properties",false,"LC__CMDB__CATG__POWER_SUPPLIER",null,"isys_cmdb_dao_category_g_power_supplier__ampere"],[1,"connector","con_connector","LC__CATG__CONNECTOR__ASSIGNED_CONNECTOR","isys_cmdb_dao_category_g_power_supplier::get_properties",false,"LC__CMDB__CATG__POWER_SUPPLIER",null,"isys_cmdb_dao_category_g_power_supplier__connector"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__FC_SWITCH' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"target","isys_catg_fc_port_list__isys_catg_connector_list__id","LC__CMDB__CATG__NETWORK__TARGET_OBJECT","isys_cmdb_dao_category_g_controller_fcport::get_properties",false,"LC__STORAGE_FCPORT",null,"isys_cmdb_dao_category_g_controller_fcport__target"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__FILE' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__HOST' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"all_ips","isys_catg_ip_list__isys_cats_net_ip_addresses_list__id","LC__CMDB__CATG__CLUSTER_SERVICE__HOST_ADDRESSES","isys_cmdb_dao_category_g_ip::get_properties",false,"LC__CATG__IP_ADDRESS",null,"isys_cmdb_dao_category_g_ip__all_ips"],[1,"primary_hostname","isys_catg_ip_list__hostname","LC__CATP__IP__PRIMARY_HOSTNAME","isys_cmdb_dao_category_g_ip::get_properties",false,"LC__CATG__IP_ADDRESS",null,"isys_cmdb_dao_category_g_ip__primary_hostname"],[1,"manufacturer","isys_model_manufacturer__title","LC__CATG__STORAGE_MANUFACTURER","isys_cmdb_dao_category_g_model::get_properties",false,"LC__CMDB__CATG__MODEL",null,"isys_cmdb_dao_category_g_model__manufacturer"],[1,"title","isys_model_title__title","LC__CMDB__CATG__MODEL","isys_cmdb_dao_category_g_model::get_properties",false,"LC__CMDB__CATG__MODEL",null,"isys_cmdb_dao_category_g_model__title"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__IT_SERVICE' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"its_type","isys_its_type__title","LC__CMDB__CATG__TYPE","isys_cmdb_dao_category_g_its_type::get_properties",false,"LC__CMDB__CATG__ITS_TYPE",null,"isys_cmdb_dao_category_g_its_type__its_type"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__KRYPTO_CARD' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"date_of_issue","isys_cats_krypto_card_list__date_of_issue","LC__CMDB__CATS__KRYPTO_CARD__DATE_OF_ISSUE","isys_cmdb_dao_category_s_krypto_card::get_properties",false,"LC__CMDB__CATS__KRYPTO_CARD",null,"isys_cmdb_dao_category_s_krypto_card__date_of_issue"],[1,"certgate_card_number","isys_cats_krypto_card_list__certgate_card_number","LC__CMDB__CATS__KRYPTO_CARD__CERTGATE_CARD_NUMBER","isys_cmdb_dao_category_s_krypto_card::get_properties",false,"LC__CMDB__CATS__KRYPTO_CARD",null,"isys_cmdb_dao_category_s_krypto_card__certgate_card_number"],[1,"imei_number","isys_cats_krypto_card_list__imei_number","LC__CMDB__CATS__KRYPTO_CARD__IMEI_NUMBER","isys_cmdb_dao_category_s_krypto_card::get_properties",false,"LC__CMDB__CATS__KRYPTO_CARD",null,"isys_cmdb_dao_category_s_krypto_card__imei_number"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__KVM_SWITCH' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"manufacturer","isys_model_manufacturer__title","LC__CATG__STORAGE_MANUFACTURER","isys_cmdb_dao_category_g_model::get_properties",false,"LC__CMDB__CATG__MODEL",null,"isys_cmdb_dao_category_g_model__manufacturer"],[1,"title","isys_model_title__title","LC__CMDB__CATG__MODEL","isys_cmdb_dao_category_g_model::get_properties",false,"LC__CMDB__CATG__MODEL",null,"isys_cmdb_dao_category_g_model__title"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__LAYER2_NET' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"vlan_id","isys_cats_layer2_net_list__ident","LC__CMDB__CATS__LAYER2_ID","isys_cmdb_dao_category_s_layer2_net::get_properties",false,"LC__CMDB__CATS__LAYER2_NET",null,"isys_cmdb_dao_category_s_layer2_net__vlan_id"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__LAYER3_NET' => '[[1,"ip_address_link","isys_cats_net_ip_addresses_link__isys_obj__id","LC__CMDB__IP__URL_TITLE","isys_cmdb_dao_category_s_net_ip_addresses::get_properties",false,"LC__CMDB__CATS__NET_IP_ADDRESSES",null,"isys_cmdb_dao_category_s_net_ip_addresses__ip_address_link"],[1,"type","isys_net_type__title","LC__CMDB__CATS__NET__TYPE","isys_cmdb_dao_category_s_net::get_properties",false,"LC__CMDB__CATS__NET",null,"isys_cmdb_dao_category_s_net__type"],[1,"address_with_suffix","isys_obj__id","LC__CMDB__CATS__NET__ADDRESS_WITH_SUFFIX","isys_cmdb_dao_category_s_net::get_properties",false,"LC__CMDB__CATS__NET",null,"isys_cmdb_dao_category_s_net__address_with_suffix"],[1,"address_range","isys_obj__id","LC__CMDB__CATS__NET__ADDRESS_RANGE","isys_cmdb_dao_category_s_net::get_properties",false,"LC__CMDB__CATS__NET",null,"isys_cmdb_dao_category_s_net__address_range"],[1,"netmask","isys_cats_net_list__mask","LC__CMDB__CATS__NET__MASK","isys_cmdb_dao_category_s_net::get_properties",false,"LC__CMDB__CATS__NET",null,"isys_cmdb_dao_category_s_net__netmask"],[1,"free_addresses","isys_obj__id","LC__CMDB__CATG__NETWORK__ASS_IP","isys_cmdb_dao_category_s_net::get_properties",false,"LC__CMDB__CATS__NET",null,"isys_cmdb_dao_category_s_net__free_addresses"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__LICENCE' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"amount","isys_cats_lic_list__amount","LC__CMDB__CATS__LICENCE_AMOUNT","isys_cmdb_dao_category_s_lic::get_properties",false,"LC__CMDB__CATS__LICENCE_LIST",null,"isys_cmdb_dao_category_s_lic__amount"],[1,"lic_not_in_use","isys_cats_lic_list__id","LC__CMDB__CATS__LICENCE_FREE","isys_cmdb_dao_category_s_lic::get_properties",false,"LC__CMDB__CATS__LICENCE_LIST",null,"isys_cmdb_dao_category_s_lic__lic_not_in_use"],[1,"type","isys_cats_lic_list__type","LC__CMDB__CATS__LICENCE_TYPE","isys_cmdb_dao_category_s_lic::get_properties",false,"LC__CMDB__CATS__LICENCE_LIST",null,"isys_cmdb_dao_category_s_lic__type"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__MAINTENANCE' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"type","isys_contract_type__title","LC__CMDB__CATS__CONTRACT__TYPE","isys_cmdb_dao_category_s_contract::get_properties",false,"LC__CMDB__CATS__CONTRACT",null,"isys_cmdb_dao_category_s_contract__type"],[1,"contract_status","isys_contract_status__title","LC__CMDB__CATS__CONTRACT__STATUS","isys_cmdb_dao_category_s_contract::get_properties",false,"LC__CMDB__CATS__CONTRACT",null,"isys_cmdb_dao_category_s_contract__contract_status"],[1,"run_time","isys_cats_contract_list__runtime","LC__CMDB__CATG__LEASING__RUNTIME","isys_cmdb_dao_category_s_contract::get_properties",false,"LC__CMDB__CATS__CONTRACT",null,"isys_cmdb_dao_category_s_contract__run_time"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__MIDDLEWARE' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"manufacturer","isys_application_manufacturer__title","LC__CMDB__CATG__MANUFACTURE","isys_cmdb_dao_category_s_application::get_properties",false,"LC__CMDB__CATS__APPLICATION",null,"isys_cmdb_dao_category_s_application__manufacturer"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__MIGRATION_OBJECT' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__MONITOR' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"manufacturer","isys_model_manufacturer__title","LC__CATG__STORAGE_MANUFACTURER","isys_cmdb_dao_category_g_model::get_properties",false,"LC__CMDB__CATG__MODEL",null,"isys_cmdb_dao_category_g_model__manufacturer"],[1,"title","isys_model_title__title","LC__CMDB__CATG__MODEL","isys_cmdb_dao_category_g_model::get_properties",false,"LC__CMDB__CATG__MODEL",null,"isys_cmdb_dao_category_g_model__title"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__NAGIOS_HOST_TPL' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"created","isys_obj__created","LC__TASK__DETAIL__WORKORDER__CREATION_DATE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__created"],[1,"changed","isys_obj__updated","LC__CMDB__LAST_CHANGE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__changed"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__NAGIOS_SERVICE' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"created","isys_obj__created","LC__TASK__DETAIL__WORKORDER__CREATION_DATE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__created"],[1,"changed","isys_obj__updated","LC__CMDB__LAST_CHANGE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__changed"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__NAGIOS_SERVICE_TPL' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"created","isys_obj__created","LC__TASK__DETAIL__WORKORDER__CREATION_DATE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__created"],[1,"changed","isys_obj__updated","LC__CMDB__LAST_CHANGE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__changed"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__OPERATING_SYSTEM' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"manufacturer","isys_application_manufacturer__title","LC__CMDB__CATG__MANUFACTURE","isys_cmdb_dao_category_s_application::get_properties",false,"LC__CMDB__CATS__APPLICATION",null,"isys_cmdb_dao_category_s_application__manufacturer"],[1,"installation_count",null,"LC__CMDB__CATS__APPLICATION_INSTALLATION_COUNT","isys_cmdb_dao_category_s_application::get_properties",false,"LC__CMDB__CATS__APPLICATION",null,"isys_cmdb_dao_category_s_application__installation_count"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__ORGANIZATION' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"street","isys_catg_address_list__street","LC__CONTACT__ORGANISATION_STREET","isys_cmdb_dao_category_g_address::get_properties",false,"LC__CATG__ADDRESS",null,"isys_cmdb_dao_category_g_address__street"],[1,"postcode","isys_catg_address_list__postalcode","LC__CONTACT__ORGANISATION_POSTAL_CODE","isys_cmdb_dao_category_g_address::get_properties",false,"LC__CATG__ADDRESS",null,"isys_cmdb_dao_category_g_address__postcode"],[1,"city","isys_catg_address_list__city","LC__CONTACT__ORGANISATION_CITY","isys_cmdb_dao_category_g_address::get_properties",false,"LC__CATG__ADDRESS",null,"isys_cmdb_dao_category_g_address__city"],[1,"telephone","isys_cats_organization_list__telephone","LC__CONTACT__ORGANISATION_PHONE","isys_cmdb_dao_category_s_organization_master::get_properties",false,"LC__CMDB__CATS__ORGANIZATION_MASTER_DATA",null,"isys_cmdb_dao_category_s_organization_master__telephone"]]',
            'C__OBJTYPE__PATCH_PANEL' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__PDU' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"title","isys_catg_connector_list__title","LC__CMDB__CATG__TITLE","isys_cmdb_dao_category_g_connector::get_properties",false,"LC__CMDB__CATG__CONNECTORS",null,"isys_cmdb_dao_category_g_connector__title"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__PERSON' => '[[1,"last_name","isys_cats_person_list__last_name","LC__CONTACT__PERSON_LAST_NAME","isys_cmdb_dao_category_s_person_master::get_properties",false,"LC__CONTACT__TREE__MASTER_DATA",null,"isys_cmdb_dao_category_s_person_master__last_name"],[1,"first_name","isys_cats_person_list__first_name","LC__CONTACT__PERSON_FIRST_NAME","isys_cmdb_dao_category_s_person_master::get_properties",false,"LC__CONTACT__TREE__MASTER_DATA",null,"isys_cmdb_dao_category_s_person_master__first_name"],[1,"department","isys_cats_person_list__department","LC__CONTACT__PERSON_DEPARTMENT","isys_cmdb_dao_category_s_person_master::get_properties",false,"LC__CONTACT__TREE__MASTER_DATA",null,"isys_cmdb_dao_category_s_person_master__department"],[1,"phone_company","isys_cats_person_list__phone_company","LC__CONTACT__PERSON_TELEPHONE_COMPANY","isys_cmdb_dao_category_s_person_master::get_properties",false,"LC__CONTACT__TREE__MASTER_DATA",null,"isys_cmdb_dao_category_s_person_master__phone_company"],[1,"mail","isys_catg_mail_addresses_list__title","LC__CONTACT__PERSON_MAIL_ADDRESS","isys_cmdb_dao_category_s_person_master::get_properties",false,"LC__CONTACT__TREE__MASTER_DATA",null,"isys_cmdb_dao_category_s_person_master__mail"],[1,"title","isys_cats_person_list__title","LC__CONTACT__PERSON_USER_NAME","isys_cmdb_dao_category_s_person_login::get_properties",false,"LC__UNIVERSAL__LOGIN",null,"isys_cmdb_dao_category_s_person_login__title"]]',
            'C__OBJTYPE__PERSON_GROUP' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"email_address","isys_catg_mail_addresses_list__title","LC__CONTACT__GROUP_EMAIL_ADDRESS","isys_cmdb_dao_category_s_person_group_master::get_properties",false,"LC__CONTACT__TREE__MASTER_DATA",null,"isys_cmdb_dao_category_s_person_group_master__email_address"],[1,"phone","isys_cats_person_group_list__phone","LC__CONTACT__GROUP_PHONE","isys_cmdb_dao_category_s_person_group_master::get_properties",false,"LC__CONTACT__TREE__MASTER_DATA",null,"isys_cmdb_dao_category_s_person_group_master__phone"],[1,"ldap_group","isys_cats_person_group_list__ldap_group","LC__CMDB__CATS__PERSON_GROUPS__LDAP_MAPPING","isys_cmdb_dao_category_s_person_group_master::get_properties",false,"LC__CONTACT__TREE__MASTER_DATA",null,"isys_cmdb_dao_category_s_person_group_master__ldap_group"]]',
            'C__OBJTYPE__PHONE' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"extension","isys_catg_telephone_fax_list__extension","LC__CMDB__CATG__TELEPHONE_FAX__EXTENSION","isys_cmdb_dao_category_g_telephone_fax::get_properties",false,"LC__CMDB__CATG__TELEPHONE_FAX",null,"isys_cmdb_dao_category_g_telephone_fax__extension"],[1,"telephone_number","isys_catg_telephone_fax_list__telephone_number","LC__CMDB__CATG__TELEPHONE_FAX__TELEPHONE_NUMBER","isys_cmdb_dao_category_g_telephone_fax::get_properties",false,"LC__CMDB__CATG__TELEPHONE_FAX",null,"isys_cmdb_dao_category_g_telephone_fax__telephone_number"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__PRINTBOX' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"manufacturer","isys_model_manufacturer__title","LC__CATG__STORAGE_MANUFACTURER","isys_cmdb_dao_category_g_model::get_properties",false,"LC__CMDB__CATG__MODEL",null,"isys_cmdb_dao_category_g_model__manufacturer"],[1,"title","isys_model_title__title","LC__CMDB__CATG__MODEL","isys_cmdb_dao_category_g_model::get_properties",false,"LC__CMDB__CATG__MODEL",null,"isys_cmdb_dao_category_g_model__title"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__PRINTER' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"manufacturer","isys_model_manufacturer__title","LC__CATG__STORAGE_MANUFACTURER","isys_cmdb_dao_category_g_model::get_properties",false,"LC__CMDB__CATG__MODEL",null,"isys_cmdb_dao_category_g_model__manufacturer"],[1,"title","isys_model_title__title","LC__CMDB__CATG__MODEL","isys_cmdb_dao_category_g_model::get_properties",false,"LC__CMDB__CATG__MODEL",null,"isys_cmdb_dao_category_g_model__title"],[1,"type","isys_cats_prt_type__title","LC__CMDB__CATS__PRT_TYPE","isys_cmdb_dao_category_s_prt::get_properties",false,"LC__CMDB__CATS__PRT",null,"isys_cmdb_dao_category_s_prt__type"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__REPLICATION' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"replication_mechanism","isys_replication_mechanism__title","LC__CMDB__CATS__REPLICATION__MECHANISM","isys_cmdb_dao_category_s_replication::get_properties",false,"LC__CATS__REPLICATION",null,"isys_cmdb_dao_category_s_replication__replication_mechanism"],[1,"replication_partner","isys_connection__title","LC__CATS__REPLICATION_PARTNER","isys_cmdb_dao_category_s_replication_partner::get_properties",false,"LC__CATS__REPLICATION_PARTNER",null,"isys_cmdb_dao_category_s_replication_partner__replication_partner"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__RM_CONTROLLER' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"created","isys_obj__created","LC__TASK__DETAIL__WORKORDER__CREATION_DATE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__created"],[1,"changed","isys_obj__updated","LC__CMDB__LAST_CHANGE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__changed"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__ROOM' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"number","isys_cats_room_list__number","LC__CMDB__CATS__ROOM_NUMBER","isys_cmdb_dao_category_s_room::get_properties",false,"LC__CMDB__CATS__ROOM",null,"isys_cmdb_dao_category_s_room__number"],[1,"floor","isys_cats_room_list__floor","LC__CMDB__CATS__ROOM_FLOOR","isys_cmdb_dao_category_s_room::get_properties",false,"LC__CMDB__CATS__ROOM",null,"isys_cmdb_dao_category_s_room__floor"],[1,"type","isys_room_type__title","LC__CMDB__CATG__TYPE","isys_cmdb_dao_category_s_room::get_properties",false,"LC__CMDB__CATS__ROOM",null,"isys_cmdb_dao_category_s_room__type"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__ROUTER' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"primary_hostaddress","isys_cats_net_ip_addresses_list__title","LC__CATG__IP__PRIMARY_ADDRESS","isys_cmdb_dao_category_g_ip::get_properties",false,"LC__CATG__IP_ADDRESS",null,"isys_cmdb_dao_category_g_ip__primary_hostaddress"],[1,"primary_hostname","isys_catg_ip_list__hostname","LC__CATP__IP__PRIMARY_HOSTNAME","isys_cmdb_dao_category_g_ip::get_properties",false,"LC__CATG__IP_ADDRESS",null,"isys_cmdb_dao_category_g_ip__primary_hostname"],[1,"all_ips","isys_catg_ip_list__isys_cats_net_ip_addresses_list__id","LC__CMDB__CATG__CLUSTER_SERVICE__HOST_ADDRESSES","isys_cmdb_dao_category_g_ip::get_properties",false,"LC__CATG__IP_ADDRESS",null,"isys_cmdb_dao_category_g_ip__all_ips"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__SAN' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"description","isys_catg_sanpool_list__description","LC__CMDB__LOGBOOK__DESCRIPTION","isys_cmdb_dao_category_g_sanpool::get_properties",false,"LC__CMDB__CATG__LDEV_SERVER",null,"isys_cmdb_dao_category_g_sanpool__description"],[1,"lun","isys_catg_sanpool_list__lun","LUN","isys_cmdb_dao_category_g_sanpool::get_properties",false,"LC__CMDB__CATG__LDEV_SERVER",null,"isys_cmdb_dao_category_g_sanpool__lun"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__SAN_ZONING' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"members","isys_cats_san_zoning_list__id","LC__CMDB__CATS__SAN_ZONING__MEMBERS","isys_cmdb_dao_category_s_san_zoning::get_properties",false,"LC__CMDB__CATS__SAN_ZONING",null,"isys_cmdb_dao_category_s_san_zoning__members"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__SERVER' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"primary_hostaddress","isys_cats_net_ip_addresses_list__title","LC__CATG__IP__PRIMARY_ADDRESS","isys_cmdb_dao_category_g_ip::get_properties",false,"LC__CATG__IP_ADDRESS",null,"isys_cmdb_dao_category_g_ip__primary_hostaddress"],[1,"primary_hostname","isys_catg_ip_list__hostname","LC__CATP__IP__PRIMARY_HOSTNAME","isys_cmdb_dao_category_g_ip::get_properties",false,"LC__CATG__IP_ADDRESS",null,"isys_cmdb_dao_category_g_ip__primary_hostname"],[1,"manufacturer","isys_model_manufacturer__title","LC__CATG__STORAGE_MANUFACTURER","isys_cmdb_dao_category_g_model::get_properties",false,"LC__CMDB__CATG__MODEL",null,"isys_cmdb_dao_category_g_model__manufacturer"],[1,"title","isys_model_title__title","LC__CMDB__CATG__MODEL","isys_cmdb_dao_category_g_model::get_properties",false,"LC__CMDB__CATG__MODEL",null,"isys_cmdb_dao_category_g_model__title"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__SERVICE' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"manufacturer","isys_application_manufacturer__title","LC__CMDB__CATG__MANUFACTURE","isys_cmdb_dao_category_s_service::get_properties",false,"LC__CMDB__CATS__SERVICE",null,"isys_cmdb_dao_category_s_service__manufacturer"],[1,"installation_count",null,"LC__CMDB__CATS__APPLICATION_INSTALLATION_COUNT","isys_cmdb_dao_category_s_service::get_properties",false,"LC__CMDB__CATS__SERVICE",null,"isys_cmdb_dao_category_s_service__installation_count"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__SIM_CARD' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"type","isys_cp_contract_type__title","LC__CMDB__CATG__TYPE","isys_cmdb_dao_category_g_sim_card::get_properties",false,"LC__CMDB__CATS__SIM_CARD",null,"isys_cmdb_dao_category_g_sim_card__type"],[1,"assigned_mobile","isys_catg_assigned_cards_list__isys_obj__id","LC__CMDB__CATS__SIM_CARD__ASSIGNED_MOBILE_PHONE","isys_cmdb_dao_category_g_sim_card::get_properties",false,"LC__CMDB__CATS__SIM_CARD",null,"isys_cmdb_dao_category_g_sim_card__assigned_mobile"],[1,"network_provider","isys_network_provider__title","LC__CMDB__CATS_CP_CONTRACT__NETWORK_PROVIDER","isys_cmdb_dao_category_g_sim_card::get_properties",false,"LC__CMDB__CATS__SIM_CARD",null,"isys_cmdb_dao_category_g_sim_card__network_provider"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__STACKING' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"created","isys_obj__created","LC__TASK__DETAIL__WORKORDER__CREATION_DATE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__created"],[1,"changed","isys_obj__updated","LC__CMDB__LAST_CHANGE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__changed"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__SUPERNET' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"type","isys_net_type__title","LC__CMDB__CATS__NET__TYPE","isys_cmdb_dao_category_s_net::get_properties",false,"LC__CMDB__CATS__NET",null,"isys_cmdb_dao_category_s_net__type"],[1,"address_with_suffix","isys_obj__id","LC__CMDB__CATS__NET__ADDRESS_WITH_SUFFIX","isys_cmdb_dao_category_s_net::get_properties",false,"LC__CMDB__CATS__NET",null,"isys_cmdb_dao_category_s_net__address_with_suffix"],[1,"address_range","isys_obj__id","LC__CMDB__CATS__NET__ADDRESS_RANGE","isys_cmdb_dao_category_s_net::get_properties",false,"LC__CMDB__CATS__NET",null,"isys_cmdb_dao_category_s_net__address_range"],[1,"netmask","isys_cats_net_list__mask","LC__CMDB__CATS__NET__MASK","isys_cmdb_dao_category_s_net::get_properties",false,"LC__CMDB__CATS__NET",null,"isys_cmdb_dao_category_s_net__netmask"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__SWITCH' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"primary_hostaddress","isys_cats_net_ip_addresses_list__title","LC__CATG__IP__PRIMARY_ADDRESS","isys_cmdb_dao_category_g_ip::get_properties",false,"LC__CATG__IP_ADDRESS",null,"isys_cmdb_dao_category_g_ip__primary_hostaddress"],[1,"primary_hostname","isys_catg_ip_list__hostname","LC__CATP__IP__PRIMARY_HOSTNAME","isys_cmdb_dao_category_g_ip::get_properties",false,"LC__CATG__IP_ADDRESS",null,"isys_cmdb_dao_category_g_ip__primary_hostname"],[1,"manufacturer","isys_model_manufacturer__title","LC__CATG__STORAGE_MANUFACTURER","isys_cmdb_dao_category_g_model::get_properties",false,"LC__CMDB__CATG__MODEL",null,"isys_cmdb_dao_category_g_model__manufacturer"],[1,"title","isys_model_title__title","LC__CMDB__CATG__MODEL","isys_cmdb_dao_category_g_model::get_properties",false,"LC__CMDB__CATG__MODEL",null,"isys_cmdb_dao_category_g_model__title"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__SWITCH_CHASSIS' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"title","isys_cats_chassis_slot_list__title","LC__CMDB__CATG__GLOBAL_TITLE","isys_cmdb_dao_category_s_chassis_slot::get_properties",false,"LC__CMDB__CATS__CHASSIS_SLOTS",null,"isys_cmdb_dao_category_s_chassis_slot__title"],[1,"assigned_devices","isys_cats_chassis_slot_list__id","LC__CMDB__CATS__CHASSIS__ASSIGNED_DEVICES","isys_cmdb_dao_category_s_chassis_slot::get_properties",false,"LC__CMDB__CATS__CHASSIS_SLOTS",null,"isys_cmdb_dao_category_s_chassis_slot__assigned_devices"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__TELEPHONE_SYSTEM' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"manufacturer","isys_model_manufacturer__title","LC__CATG__STORAGE_MANUFACTURER","isys_cmdb_dao_category_g_model::get_properties",false,"LC__CMDB__CATG__MODEL",null,"isys_cmdb_dao_category_g_model__manufacturer"],[1,"title","isys_model_title__title","LC__CMDB__CATG__MODEL","isys_cmdb_dao_category_g_model::get_properties",false,"LC__CMDB__CATG__MODEL",null,"isys_cmdb_dao_category_g_model__title"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__UPS' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"manufacturer","isys_model_manufacturer__title","LC__CATG__STORAGE_MANUFACTURER","isys_cmdb_dao_category_g_model::get_properties",false,"LC__CMDB__CATG__MODEL",null,"isys_cmdb_dao_category_g_model__manufacturer"],[1,"title","isys_model_title__title","LC__CMDB__CATG__MODEL","isys_cmdb_dao_category_g_model::get_properties",false,"LC__CMDB__CATG__MODEL",null,"isys_cmdb_dao_category_g_model__title"],[1,"type","isys_ups_type__title","LC__CMDB__CATS__UPS__TYPE","isys_cmdb_dao_category_s_ups::get_properties",false,"LC__CMDB__OBJTYPE__UPS",null,"isys_cmdb_dao_category_s_ups__type"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__VEHICLE' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"created","isys_obj__created","LC__TASK__DETAIL__WORKORDER__CREATION_DATE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__created"],[1,"changed","isys_obj__updated","LC__CMDB__LAST_CHANGE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__changed"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__VIRTUAL_CLIENT' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"primary_contact","isys_connection__title","LC__CATG__CONTACT_PRIMARY","isys_cmdb_dao_category_g_contact::get_properties",false,"LC__CMDB__CATG__CONTACT",null,"isys_cmdb_dao_category_g_contact__primary_contact"],[1,"hosts","vm_obj","LC__CMDB__CATG__VM__RUNNING_ON_HOST","isys_cmdb_dao_category_g_virtual_machine::get_properties",false,"LC__CMDB__CATG__VIRTUAL_MACHINE",null,"isys_cmdb_dao_category_g_virtual_machine__hosts"],[1,"primary","isys_catg_virtual_machine_list__primary","LC__CMDB__HOST_IN_CLUSTER","isys_cmdb_dao_category_g_virtual_machine::get_properties",false,"LC__CMDB__CATG__VIRTUAL_MACHINE",null,"isys_cmdb_dao_category_g_virtual_machine__primary"],[1,"system","isys_vm_type__title","LC__CMDB__CATG__VIRTUALIZATION_SYSTEM","isys_cmdb_dao_category_g_virtual_machine::get_properties",false,"LC__CMDB__CATG__VIRTUAL_MACHINE",null,"isys_cmdb_dao_category_g_virtual_machine__system"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__VIRTUAL_HOST' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"primary_hostaddress","isys_cats_net_ip_addresses_list__title","LC__CATG__IP__PRIMARY_ADDRESS","isys_cmdb_dao_category_g_ip::get_properties",false,"LC__CATG__IP_ADDRESS",null,"isys_cmdb_dao_category_g_ip__primary_hostaddress"],[1,"primary_hostname","isys_catg_ip_list__hostname","LC__CATP__IP__PRIMARY_HOSTNAME","isys_cmdb_dao_category_g_ip::get_properties",false,"LC__CATG__IP_ADDRESS",null,"isys_cmdb_dao_category_g_ip__primary_hostname"],[1,"connected_object","isys_catg_virtual_machine_list__isys_obj__id","LC__CMDB__CATG__GUEST_SYSTEMS","isys_cmdb_dao_category_g_guest_systems::get_properties",false,"LC__CMDB__CATG__GUEST_SYSTEMS",null,"isys_cmdb_dao_category_g_guest_systems__connected_object"],[1,"manufacturer","isys_model_manufacturer__title","LC__CATG__STORAGE_MANUFACTURER","isys_cmdb_dao_category_g_model::get_properties",false,"LC__CMDB__CATG__MODEL",null,"isys_cmdb_dao_category_g_model__manufacturer"],[1,"title","isys_model_title__title","LC__CMDB__CATG__MODEL","isys_cmdb_dao_category_g_model::get_properties",false,"LC__CMDB__CATG__MODEL",null,"isys_cmdb_dao_category_g_model__title"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__VIRTUAL_SERVER' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"primary_hostaddress","isys_cats_net_ip_addresses_list__title","LC__CATG__IP__PRIMARY_ADDRESS","isys_cmdb_dao_category_g_ip::get_properties",false,"LC__CATG__IP_ADDRESS",null,"isys_cmdb_dao_category_g_ip__primary_hostaddress"],[1,"primary_hostname","isys_catg_ip_list__hostname","LC__CATP__IP__PRIMARY_HOSTNAME","isys_cmdb_dao_category_g_ip::get_properties",false,"LC__CATG__IP_ADDRESS",null,"isys_cmdb_dao_category_g_ip__primary_hostname"],[1,"primary","isys_catg_virtual_machine_list__primary","LC__CMDB__HOST_IN_CLUSTER","isys_cmdb_dao_category_g_virtual_machine::get_properties",false,"LC__CMDB__CATG__VIRTUAL_MACHINE",null,"isys_cmdb_dao_category_g_virtual_machine__primary"],[1,"hosts","vm_obj","LC__CMDB__CATG__VM__RUNNING_ON_HOST","isys_cmdb_dao_category_g_virtual_machine::get_properties",false,"LC__CMDB__CATG__VIRTUAL_MACHINE",null,"isys_cmdb_dao_category_g_virtual_machine__hosts"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__VOIP_PHONE' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__VRRP' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"created","isys_obj__created","LC__TASK__DETAIL__WORKORDER__CREATION_DATE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__created"],[1,"changed","isys_obj__updated","LC__CMDB__LAST_CHANGE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__changed"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__WAN' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"role","isys_wan_role__title","LC__CATG__WAN__ROLE","isys_cmdb_dao_category_g_wan::get_properties",false,"LC__CATG__WAN",null,"isys_cmdb_dao_category_g_wan__role"],[1,"type","isys_wan_type__title","LC__CATG__WAN__TYPE","isys_cmdb_dao_category_g_wan::get_properties",false,"LC__CATG__WAN",null,"isys_cmdb_dao_category_g_wan__type"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__WIRING_SYSTEM' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"title","isys_net_type_title__title","LC__CMDB__CATS__WS_NET_TYPE","isys_cmdb_dao_category_s_ws_net_type::get_properties",false,"LC__CMDB__CATS__WS_NET_TYPE",null,"isys_cmdb_dao_category_s_ws_net_type__title"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
            'C__OBJTYPE__WORKSTATION' => '[[1,"title","isys_obj__title","LC__UNIVERSAL__TITLE","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__title"],[1,"parent","isys_catg_logical_unit_list__isys_obj__id__parent","LC__CMDB__CATG__LOGICAL_UNIT__PARENT","isys_cmdb_dao_category_g_logical_unit::get_properties",false,"LC__CMDB__CATG__LOGICAL_UNIT",null,"isys_cmdb_dao_category_g_logical_unit__parent"],[1,"assigned_object","isys_obj__title","LC_UNIVERSAL__OBJECT","isys_cmdb_dao_category_g_assigned_logical_unit::get_properties",false,"LC__CMDB__CATG__ASSIGNED_LOGICAL_UNITS",null,"isys_cmdb_dao_category_g_assigned_logical_unit__assigned_object"],[1,"location_path","isys_catg_location_list__parentid","LC__CMDB__CATG__LOCATION_PATH","isys_cmdb_dao_category_g_location::get_properties",false,"LC__CMDB__CATG__LOCATION",null,"isys_cmdb_dao_category_g_location__location_path"],[1,"cmdb_status","isys_cmdb_status__title","LC__UNIVERSAL__CMDB_STATUS","isys_cmdb_dao_category_g_global::get_properties",false,"LC__CMDB__CATG__GLOBAL",null,"isys_cmdb_dao_category_g_global__cmdb_status"]]',
        ];

        $dao = new isys_cmdb_dao($g_comp_database);

        $listRes = $dao->retrieve(
            'SELECT *
        FROM isys_obj_type_list
        INNER JOIN isys_obj_type ON isys_obj_type__id = isys_obj_type_list__isys_obj_type__id
        ORDER BY isys_obj_type_list__isys_obj_type__id ASC;'
        );

        if (count($listRes))
        {
            $g_migration_log[] = '<strong>Migrating user specified lists</strong>';

            while ($listRow = $listRes->get_row())
            {
                $g_migration_log[] = '<span>List configuration for "' . _L($listRow['isys_obj_type__title']) . '"</span>';

                try
                {
                    $result = convertOldConfig(
                        $listRow['isys_obj_type_list__config'],
                        $listRow['isys_obj_type_list__row_clickable'],
                        $listRow['isys_obj_type_list__isys_property_2_cat__id'],
                        $listRow['isys_obj_type_list__sorting_direction']
                    );

                    $l_sql = 'UPDATE isys_obj_type_list SET
                    isys_obj_type_list__config = ' . $dao->convert_sql_text(json_encode($result['config'])) . ',
                    isys_obj_type_list__table_config = ' . $dao->convert_sql_text(serialize($result['table_config'])) . '
                    WHERE isys_obj_type_list__id = ' . $dao->convert_sql_id($listRow['isys_obj_type_list__id']) . ';';

                    $dao->update($l_sql);
                }
                catch (Exception $e)
                {
                    $g_migration_log[] = '<span class="indent text-red">List could not be updated: ' . $e->getMessage() . '</span>';
                }
            } // while
            $dao->apply_update();
        } // if

        $objectTypeRes = $dao->retrieve('SELECT isys_obj_type__id, isys_obj_type__title, isys_obj_type__const, isys_obj_type__class_name FROM isys_obj_type;');

        if (count($objectTypeRes))
        {
            $g_migration_log[] = '<strong>Migrating object type default lists</strong>';

            while ($objectTypeRow = $objectTypeRes->get_row())
            {
                $g_migration_log[] = '<span>List configuration for "' . _L($objectTypeRow['isys_obj_type__title']) . '"</span>';

                // Remove the default "list" SQL.
                isys_tenantsettings::remove('cmdb.default-object-list.sql.' . $objectTypeRow['isys_obj_type__const']);

                $defaultConfig = isys_tenantsettings::get('cmdb.default-object-list.config.' . $objectTypeRow['isys_obj_type__const'], false);

                if ($defaultConfig !== false)
                {
                    if (is_array($defaultConfig))
                    {
                        $defaultConfig = isys_format_json::encode($defaultConfig);
                    } // if

                    $result = convertOldConfig($defaultConfig, true, null, null);

                    isys_tenantsettings::set('cmdb.default-object-list.config.' . $objectTypeRow['isys_obj_type__const'], isys_format_json::encode($result['config']));
                    isys_tenantsettings::set('cmdb.default-object-table.config.' . $objectTypeRow['isys_obj_type__const'], serialize($result['table_config']));
                }
                else
                {
                    $g_migration_log[] = '<span class="indent text-grey">Does not exist, no need to migrate.</span>';
                } // if

                if (isset($baseConfig[$objectTypeRow['isys_obj_type__const']]))
                {
                    $g_migration_log[] = '<span>Base List configuration for "' . _L($objectTypeRow['isys_obj_type__title']) . '"</span>';

                    $result = convertOldConfig($baseConfig[$objectTypeRow['isys_obj_type__const']], true, null, null);

                    try
                    {
                        isys_tenantsettings::set('cmdb.base-object-list.config.' . $objectTypeRow['isys_obj_type__const'], isys_format_json::encode($result['config']));
                        isys_tenantsettings::set('cmdb.base-object-table.config.' . $objectTypeRow['isys_obj_type__const'], serialize($result['table_config']));

                        $g_migration_log[] = '<span class="indent text-green">Base List updated!</span>';
                    }
                    catch (Exception $e)
                    {
                        $g_migration_log[] = '<span class="indent text-red">The updated base list query encountered a syntax error!<br />' . $e->getMessage() . '</span>';
                    } // try
                }
                else
                {
                    if ($objectTypeRow['isys_obj_type__class_name'] != 'isys_cmdb_dao_list_objects' && method_exists($objectTypeRow['isys_obj_type__class_name'], 'get_default_list_config'))
                    {
                        $g_migration_log[] = '<span>Base List configuration for "' . _L($objectTypeRow['isys_obj_type__title']) . '"</span>';

                        $result = convertOldConfig(
                            isys_factory::get_instance($objectTypeRow['isys_obj_type__class_name'], $g_comp_database)
                                ->get_default_list_config(),
                            true,
                            null,
                            null
                        );

                        try
                        {
                            isys_tenantsettings::set('cmdb.base-object-list.config.' . $objectTypeRow['isys_obj_type__const'], isys_format_json::encode($result['config']));
                            isys_tenantsettings::set('cmdb.base-object-table.config.' . $objectTypeRow['isys_obj_type__const'], serialize($result['table_config']));

                            $g_migration_log[] = '<span class="indent text-green">Base List updated!</span>';
                        }
                        catch (Exception $e)
                        {
                            $g_migration_log[] = '<span class="indent text-red">The updated base list query encountered a syntax error!<br />' . $e->getMessage() . '</span>';
                        } // try
                    } // if
                } // if
            } // while
        } // if

        // Set flag in db.
        $this->migration_done($g_migration_identifier);

    }
    catch (Exception $e)
    {
        $g_migration_log[] = '<span class="text-bold text-red">' . $e->getMessage() . '</span>';
    } // try
} // if